// Program demonstrativ pentru calcule cu numere complexe
// Emuleaza un calculator de buzunar cu notatia poloneza inversa
//
// Comenzi admise +,-,*,/, O=opus, I=invers, C=conjugat
//
// Programator: Daniel Ioan
// Data ultimei modificari: 17.08.91

#include <complex.h>
#include <ctype.h>
#include <conio.h>
#include <stdio.h>
#include <stdlib.h>
#include <dos.h>

const int m = 19; // numar maxim de elemente in stiva
complex s[m+1];    // stiva cu numere complexe
int n = -1;       // pozitia curenta in stiva


void eroare(char *s)  // trateaza erori
{

	gotoxy(1, 23);
	printf("                                  ");
	if (s) {
		gotoxy(1, 23);
		printf("%s",s);
		sound(400);
		delay(200);
		nosound();
		}
	gotoxy(2, 18);
}

complex pop(void)
{
	if (n < 0 ) {
		eroare("Stiva goala");
		return (complex(0,0));
		}
	gotoxy(32,22 - n);
	clreol();
	return ( s[n--]);
}

void push(complex z)
{
	if (n < m ) {
		n ++;
		gotoxy(32,22 - n );
		printf("%2d: %10g%10g%10g%10g",n+1,real(z),imag(z),abs(z),arg(z));
		s[n] = z;
		}
	else  eroare("Stiva plina");
}


void invers(void)
{
if (n>= 0) {
	complex z = pop();
	if ( z != complex(0.,0.) )
		z = complex(1.,0.)/z;
	else
		eroare("Operatie imposibila");
	push(z);
	}
else
	eroare("Stiva goala");
}
void opus(void)
{
if (n>= 0) {
	complex z = pop();
	z = -z;
	push(z);
	}
else
	eroare("Stiva goala");
}

void conj(void)
{
if (n>= 0) {
	complex z = pop();
	z = conj(z);
	push(z);
	}
else
	eroare("Stiva goala");
}

void plus(void)
{
if (n>= 1) {
	complex z,z1,z2;
	z1 = pop();
	z2 = pop();
	z = z1 + z2;
	push(z);
	}
else
	eroare("Unsingur operand in stiva");
}

void minus(void)
{
if (n>= 1) {
	complex z,z1,z2;
	z2 = pop();
	z1 = pop();
	z = z1 - z2;
	push(z);
	}
else
	eroare("Unsingur operand in stiva");
}

void prod(void)
{
if (n>= 1) {
	complex z,z1,z2;
	z1 = pop();
	z2 = pop();
	z = z1 * z2;
	push(z);
	}
else
	eroare("Unsingur operand in stiva");
}

void div(void)
{
if (n>= 1) {
	complex z,z1,z2;
	z2 = pop();
	if (z2 != complex(0.,0.)) {
		z1 = pop();
		z = z1 / z2;
		push(z);
		}
	else {
		push(z2);
		eroare("Operatie imposibila");
		}
	}
else
	eroare("Unsingur operand in stiva");
}

main()
{
char c;
complex z;
float re,im;

clrscr();
cout << "      CALCULATOR PENTRU NUMERE COMPLEXE CU NOTATIA POLONEZA INVERSA\n"
	"      =============================================================\n\n\n"
"Intoduceti: \n"
"  - numar complex (Re Im)\n"
"  - operator binar (+,-,*,/)\n"
"  - O pentru Opus\n"
"  - I pentru invers\n"
"  - C pentru conjugat \n"
"  - R repeta virful stivei\n"
"  - E extrage virful stivei\n"
"  - sau X pentru iesire\n"
" apoi apasati <Enter>\n";

gotoxy(10,24);
cout << "                  nr. crt.        Re        Im     Modul       Arg";
gotoxy(1,15);

do {

	gotoxy(1, 18);
	printf(":                               \n");
	printf("                                \n");
	gotoxy(2, 18);
	fflush(stdin);
	c = toupper(getc(stdin));
	eroare(0);
	switch (c){
		case 'X': return (1);
		case 'I': invers();
			  break;
		case 'O': opus();
			  break;
		case 'C': conj();
			  break;
		case 'R': z = pop();
			  push(z);
			  push(z);
			  break;
		case 'E': z = pop();
			  break;
		case '+': plus();
			  break;
		case '-': minus();
			  break;
		case '*': prod();
			  break;
		case '/': div();
			  break;
		default:  ungetc(c,stdin);
			if (fscanf(stdin,"%f%f",&re,&im)==2){
			z = complex( re,im);
			push(z);
			}
		}
	} while(1);
} // end main
