// Program pentru analiza propagarii erorilor de calcul
// Emuleaza un calculator de buzunar cu notatia poloneza inversa
//
// Comenzi admise +,-,*,/, O=opus, I=invers, V=radical
//
// Programator: Daniel Ioan
// Data ultimei modificari: 18.08.91

#include <iostream.h>
#include <ctype.h>
#include <conio.h>
#include <stdio.h>
#include <stdlib.h>
#include <dos.h>
#include <math.h>

struct interval{
	float val;
	float err;
	};

const int m = 19;   // numar maxim de elemente in stiva
interval s[m+1];    // stiva cu intervale
int n = -1;         // pozitia curenta in stiva
float err = 1.e-7;  // eroarea relativa de rotunjire
struct interval zero = {0., 0.};

void eroare(char *s)  // trateaza erori
{

	gotoxy(1, 23);
	printf("                                  ");
	if (s) {
		gotoxy(1, 23);
		printf("%s",s);
		sound(400);
		delay(200);
		nosound();
		}
	gotoxy(2, 18);
}

interval pop(void)
{
	if (n < 0 ) {
		eroare("Stiva goala");
		return (zero);
		}
	gotoxy(32,22 - n);
	clreol();
	return ( s[n--]);
}

void push(interval z)
{
float vmin,vmax;

	if (n < m ) {
		n ++;
		gotoxy(32,22 - n );
		z.err = z.err + err;
		vmin = z.val * (1-z.err);
		vmax = z.val * (1+z.err);
		printf("%2d: %10g %9.2e %10.3e %10.3e",n+1,z.val,z.err,vmin,vmax);
		s[n] = z;
		}
	else  eroare("Stiva plina");
}


void invers(void)
{
if (n>= 0) {
	interval z = pop();
	if ( z.val != 0 )
		z.val = 1./z.val;
	else
		eroare("Operatie imposibila");
	push(z);
	}
else
	eroare("Stiva goala");
}
void opus(void)
{
if (n>= 0) {
	interval z = pop();
	z.val = -z.val;
	push(z);
	}
else
	eroare("Stiva goala");
}

void rad(void)
{
if (n>= 0) {
	interval z = pop();
	z.val = sqrt(fabs(z.val));
	z.err = z.err + err/2;
	push(z);
	}
else
	eroare("Stiva goala");
}

void plus(void)
{
if (n>= 1) {
	interval z,z1,z2;
	z1 = pop();
	z2 = pop();
	z.val = z1.val + z2.val;
	if (z.val != 0.)
		z.err = (fabs(z1.val)*z1.err + fabs(z2.val)*z2.err)/fabs(z.val);
	else
		z.err = 1.e10;
	push(z);
	}
else
	eroare("Un singur operand in stiva");
}

void minus(void)
{
if (n>= 1) {
	interval z,z1,z2;
	z2 = pop();
	z1 = pop();
	z.val = z1.val - z2.val;
	if (z.val != 0.)
		z.err = (fabs(z1.val)*z1.err + fabs(z2.val)*z2.err)/fabs(z.val);
	else
		z.err = 1.e10;
	push(z);
	}
else
	eroare("Un singur operand in stiva");
}

void prod(void)
{
if (n>= 1) {
	interval z,z1,z2;
	z1 = pop();
	z2 = pop();
	z.val = z1.val * z2.val;
	z.err = z1.err + z2.err;
	push(z);
	}
else
	eroare("Un singur operand in stiva");
}

void div(void)
{
if (n>= 1) {
	interval z,z1,z2;
	z2 = pop();
	if (z2.val != 0) {
		z1 = pop();
		z.val = z1.val / z2.val;
		z.err = z1.err + z2.err;
		push(z);
		}
	else {
		push(z2);
		eroare("Operatie imposibila");
		}
	}
else
	eroare("Un singur operand in stiva");
}

main()
{
char c;
interval z;
float val,er;
int nrin;

clrscr();
cout << "      CALCUL CU \"INTERVALE\" IN NOTATIA POLONEZA INVERSA\n"
	"      =================================================\n\n\n"
"Intoduceti: \n"
"  - valoare numar si err%\n"
"  - operator binar (+,-,*,/)\n"
"  - O pentru Opus\n"
"  - I pentru invers\n"
"  - V pentru radical\n"
"  - R repeta virful stivei\n"
"  - E extrage virful stivei\n"
"  - sau X pentru iesire\n"
" apoi apasati <Enter>\n";

gotoxy(10,24);
cout << "                  nr. crt.   Valoare  Eroare rel.  Val.min.  Val.max.";
gotoxy(1,15);

do {

	gotoxy(1, 18);
	printf(":                               \n");
	printf("                                \n");
	gotoxy(2, 18);
	fflush(stdin);
	c = toupper(getc(stdin));
	eroare(0);
	switch (c){
		case 'X': return (1);
		case 'I': invers();
			  break;
		case 'O': opus();
			  break;
		case 'V': rad();
			  break;
		case 'R': z = pop();
			  z.err -= err;
			  push(z);
			  push(z);
			  break;
		case 'E': z = pop();
			  break;
		case '+': plus();
			  break;
		case '-': minus();
			  break;
		case '*': prod();
			  break;
		case '/': div();
			  break;
		default:  ungetc(c,stdin);
			  nrin = fscanf(stdin,"%f %f",&val,&er);
			  if (nrin == 1) er = 0;
			  if (nrin != 0){
				z.val = val;
				z.err = er/100.;
				push(z);
				}
		}
	} while(1);
} // end main
