/* ===============================================================


	ERRT - program pentru analiza experimenatala a
	erorii de trunchere si a ratei de convergenta.

	Sumeaza numeric seriile:

	- Taylor pentru exp(x);
	- Taylor pentru sin(x);
	- Taylor pentru ln(x);
	- Fourier pentru functia crenel;
	- De puteri a functiei Bessel J0(x);
	- Armonica alternata (suma inverselor numerelor naturale);
	- Armonica alternata cu convergenta imbunatatita prin metoda Euler.

Intrare: x - variabila independenta la seriile de functii
Iesire : eroarea de trunchiere la diferite etape

Programator: Daniel Ioan
Data ultimei modificari: 18.08.91

===============================================================*/
#define N 101

#include <conio.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include "grafireu.c"

#define xx mat[0]
#define y1 mat[1]
#define y2 mat[2]

float Pi;
int m;  /* numarul seriei selectate */
float func(float x);
float term(int k, float x);
float bess(float x);

void main()
{
float x=0; 	/* variabila independenta */
int n = N;      /* numarul de termeni ce vor fi adunati */
int k;		/* contor termeni */
/*float xx[N],y1[N],y2[N];*/
float mat[3][N];		/* Date ce vor fi reprezentate grafic */
float y;         /* valoarea exacta a functiei */
float s = 0;	  /* suma partiala a seriei */
float t;	  /* termenul curent */
float ert;	  /* eroarea de trunchiere curenta */
float eps = 1.e-20;
Pi = 4.*atan(1.);

initial();	/* Initializare mod grafic */
restorecrtmode();

for ( k = 0; k<n; k++) {	/* Initializeaza tabele grafic*/
		xx[k] = k+1;
		y1[k] = 0;
	}

do {
	clrscr();
	printf("\n\t\tPROGRAM PENTRU STUDIUL ERORII DE TRUNCHIERE");
	printf("\n\t\t===========================================\n\n");
	printf("\n\tDetermina eroarea de trunchiere la sumarea numerica a seriei:\n");
	printf("\n\t1 - Taylor pentru exp(x);");
	printf("\n\t2 - Taylor pentru sin(x);");
	printf("\n\t3 - Taylor pentru ln(x) in intervalul 0<x<2;");
	printf("\n\t4 - Fourier pentru functia crenel;");
	printf("\n\t5 - De puteri a functiei Bessel J0(x);");
	printf("\n\t6 - Armonica alternata (suma inverselor numerelor naturale);");
	printf("\n\t7 - Armonica alternata cu convergenta imbunatatita prin metoda Euler.");
	printf("\n\n  Alegeti (min.1 max.7, altfel iesire): ");
	scanf("%d",&m);
	if (m<1 || m>7) return;
	if (m<6) {
		printf("\n  Alegeti x = ");
		scanf("%f",&x);
	}
	clrscr();
	printf("\n k  \tVal. exacta \tSuma partiala \tEroarea \tTermenul curent ");

	y = func(x);      /* valoarea exacta a functiei */
	s = 0;		  /* suma partiala a seriei */
	for (k = 0; k<n; k++) {
		t = term(k,x);
		s = s + t;
		ert = s - y;
		y2[k] = y1[k];
		y1[k] = fabs(ert);
		if ( (0<k && k<=10) || (k%(n/10)) == 0 )
			printf("\n %d\t%10g\t%10g\t%10g\t%10g",k,y,s,ert,t);
	}
	printf("\n\n   Pentru continuare apasati o tasta ");
	getch();

	/* Reprezinta grafic variatia erorii in ultimele doua cazuri: x,y1,y2 */
	setgraphmode(getgraphmode());
	for (k = 0; k<n; k++) {
		if (xx[k] < eps) xx[k] = eps;
		if (y1[k] < eps) y1[k] = eps;
		if (y2[k] < eps) y2[k] = eps;
		}
	grafic(2,n,mat,1,1);
	grid();
	label_x("Numar de iteratii");
	title("Eroarea de trunchiere");
	getch();
	restorecrtmode();

   } while(1);
}

/* =========================================================
 func (x) calculeaza valoarea exacta a seriei
 numarul m (var globala) in punctul x
 ==========================================================*/
float func(float x)
{
   switch (m) {
	case 1: return (exp(x));
	case 2: return (sin(x));
	case 3: return (log(x));
	case 4: return (( (sin(x)>0) ? 1.:-1.)*Pi/4);
	case 5: return (bess(x));
	case 6:
	case 7: return (log(2.));
	default: return(0);
	}
}

/* =========================================================
 term (k,x) calculeaza valoarea termenului k din serie
 ==========================================================*/

float term(int k, float x)
{
static float z,t;  /* varaiabila aux., termenul anterior */
if (k == 0) {
   switch (m) {
	case 2: t = x;
		break;
	case 3: z = -1;
		t = 0;
		break;
	case 4: t = 0;
		break;
	case 5: t = 1;
		break;
	case 6: z = -1;
		t = 0;
		break;
	case 7: t = 0.5;
		break;
	default:z = 1;
		t = 1;
	}
   }
else {
   switch (m) {
	case 1: t = t*x/k;
		break;
	case 2: t = -t * x * x / (2.*k+1) / (2.*k);
		break;
	case 3: z = -z * (x-1);
		t = z / k;
		break;
	case 4: z = 2*k - 1;
		t = sin(z*x)/z;
		break;
	case 5: t = -t * x * x /k /k /4.;
		break;
	case 6: z = -z;
		t = z/k;
		break;
	case 7: t = (1./(2.*k-1) - 1./(2.*k+1))/(4.*k);
		break;
	default:t = 0;
	}
    }
return (t);
}

/* =========================================================
 bess(x) calculeaza valoarea "exacta" afunctiei Bessel J0(x)
 ==========================================================*/

float bess(float x)
{
float v=0, y[2*N];
int k, nn = 2*N;
float fi,p,q,x2;


y[0] = 1;
if (fabs(x)<10){
	for (k=1;k<nn;k++) /* Calculeaza termenii seriei de puteri */
		y[k] = -y[k-1] * x * x /k /k /4.;
	for (k=nn-1;k>=0;k--) /* si ii sumeaza in ordine inversa*/
		v = v + y[k];
	}
else    {
	fi = x - Pi/4.;
	x2 = 64.* x * x;
	p = 1 - 3*3*3/(2.*x2)*(1+25*49/(12*8*x));
	q = (-1 + 75/2./x2)/(8.*x);
	v = sqrt(2/Pi/x)*(p*cos(fi)-q*sin(fi));
	}
return (v);
}
