/* Testare proceduri Jacobi si Gauss-Seidel.
Se genereaza aleator sisteme de ecuatii care se rezolva.
Rezultatele nu se afiseaza. Se calculeaza eroarea , care se afiseaza.
Sistemul nu are a[i,i] nuli.

Data : 15.02.1991
Programator : Irina Munteanu */

#define N 120
#define MAXIT 10000
#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>


void gen_sist_iter(int m,float norma);   /* Declarare proc. de generare de sisteme aleatoare */


/* Declaratiile s-au mutat temporar aici */
float mat[N][N],  /* matricea a */
      b[N], 	  /* vectorul termenilor liberi */
      x[N];	  /* vectorul rezultatelor obtinute dupa rezolvare */
int   n,	  /* dimensiunea sistemului */
      i,j,	  /* variabile contor */
      car;	  /* caracter tastat la consola */



/* Rezolvarea unui sistem de ecuatii prin metoda Frankel-Young*/
/*------------------------------------------------------------*/
/* Data : 29.01.1991
  Procedura este facuta fara pivotare.
  Restrictie : oricare a[i][i] nenul
  Intoarce : 0 = all OK
	     1 = pivot nul
	     2 = depasire nr. iteratii
	     3 = overflow */

frankel(int n,float (*a)[N],float b[N],float x[N],float omega,int *nrit,float *eps)
{

  int i,j,k,ier;
  float s,err, temp;


  /* Initializari */
  k=0;	/* contor iteratii */
  ier = 0;  /* indicator overflow */
  for (i=0; i<n; i++)
    {
    if (a[i][i] == 0) return 1; 	/* eroare */
    x[i] = 0;  /* solutia */
    }

  do
    {
    err = 0;			/* eroarea */
    for ( i=0; i<n; i++)	/* ecuatia curenta */
      {
      s = b[i];
      for (j=0; j<n; j++)	/* parcurge ecuatia */
	s -= a[i][j]*x[j];
      s = (s*omega)/a[i][i] + x[i];
      temp = fabs(s-x[i]);	/* x nou - x vechi */
      if (err < temp) err=temp;
      x[i]=s;
      }
    k++;	/* incrementeaza contor iteratii */
    if (err > 1.e+38) {ier=1;break;}
  } while ((err > *eps) && (k <= *nrit));  /* cit timp.cond. e indepl.*/
  *eps = err;
  if (k>*nrit)
    {
    *nrit = k;
    return 2;	/* eroare: nr. iteratii */
    }
  *nrit = k;
  if (ier) return 3;
  return 0;	/* all OK */
}



void main()
{

int nrit;
float norma,eps1,eps;
float omega;
clock_t start,end;


highvideo();
do{
  clrscr();
  printf("\n\t\tPROGRAM EVALUARE FRANKEL-YOUNG\n");
  printf(  "\t\t==============================\n\n\n");
  printf("Determina erori si timp de calcul la rezolvarea, prin metoda deplasarilor \n");
  printf("succesive cu suprarelaxare, a sistemelor de ecuatii liniare generate aleator.\n\n");
  printf("Introduceti numarul de ecuatii (min.2, max.%d, altfel - iesire): ",N);
  scanf("%d",&n);
  if (n<2 || n>N) break;
  printf("Introduceti norma matricii de iteratie (intre 0 si 1): ");
  scanf("%f",&norma);
  if (norma<=0. || norma>=1.) continue;
  printf("Introduceti eroarea maxima: ");
  scanf("%f",&eps1);

  /* Generare sistem de ecuatii */
  gen_sist_iter(n,norma);

  fflush(stdin);
  clrscr();
  cprintf("     Metoda Frankel-Young\n\r");
  for (i=1; i<=9; i++)
    {
    nrit = MAXIT;
    eps = eps1;
    omega = (float)i * 0.2;
    start = clock();
    frankel(n,mat,b,x,omega,&nrit,&eps);
    end = clock();
    printf("Omega = %.1f   Numar de iteratii: %d   Timp de calcul: %f secunde\n",omega,nrit,(end-start)/CLK_TCK);
    } /* for i */
  /* aici se calculeaza omega optim */
  omega = 2/(1+sqrt(1-norma*norma));
  nrit = MAXIT;
  eps = eps1;
  start = clock();
  frankel(n,mat,b,x,omega,&nrit,&eps);
  end = clock();
  printf("\nOmega optim =  %f   Numar de iteratii: %d   Timp de calcul: %f sec.\n",omega,nrit,(end-start)/CLK_TCK);

  fflush(stdin);
  printf("Pentru continuare apasati <Enter>:");
  getchar();

  } while(1);
lowvideo();

}  /* end main */

void gen_sist_iter(int n, float norm)
{
  int i,j;
  float s;

  randomize();
  for(i=0; i<n; i++)
    {
    s = 0;
    for (j=0; j<n ; j++)
      {
      mat[i][j] = ((float)(rand())/RAND_MAX  );
      s += fabs(mat[i][j]);
      }
    s -= fabs(mat[i][i]);
    mat[i][i] = s/norm;
    x[i] = ((float)(rand())/RAND_MAX);
    }
  for (i=0; i<n; i++)
    {
    s=0;
    for (j=0; j<n ; j++)
      s += mat[i][j]*x[j];
    b[i] = s;
    }
}
