/* Testare proceduri Jacobi si Gauss-Seidel.
Se genereaza aleator sisteme de ecuatii care se rezolva.
Rezultatele nu se afiseaza. Se calculeaza eroarea , care se afiseaza.
Sistemul nu are a[i,i] nuli.

Data : 15.02.1991
Programator : Irina Munteanu */

#define N 120
#define MAXIT 10000
#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>


void gen_sist_iter(int m,float norma);   /* Declarare proc. de generare de sisteme aleatoare */


/* Declaratiile s-au mutat temporar aici */
float mat[N][N],  /* matricea a */
      b[N], 	  /* vectorul termenilor liberi */
      x[N];	  /* vectorul rezultatelor obtinute dupa rezolvare */
int   n,	  /* dimensiunea sistemului */
      i,j,	  /* variabile contor */
      car;	  /* caracter tastat la consola */


/* Rezolvarea unui sistem de ecuatii prin metoda Jacobi */
/*------------------------------------------------------*/
/* Data : 28.01.1991
  Procedura este facuta fara pivotare.
  Restrictie : oricare a[i][i] nenul
  Intoarce : 0 = all OK
	     1 = pivot nul
	     2 = depasire nr. iteratii
	     3 = overflow */

jacobi(int n, float (*a)[N], float b[N], float x[N], int *nrit, float *eps)
{

  int i,j,k,lin,ier;
  float s,err, xn[N];


  /* Initializari */
  k=0;	/* contor iteratii */
  ier = 0; /* indicator overflow */
  lin = 1;
  for (i=0; i<n; i++)
    {
    if (a[i][i] == 0) return 1; 	/* eroare */
    x[i] = 0;  /* x vechi; solutia */
    }

  do
    {
    err = 0;			/* eroarea */
    for ( i=0; i<n; i++)	/* ecuatia curenta */
      {
      s = b[i];
      for (j=0; j<n; j++)	/* parcurge ecuatia */
	s -= a[i][j]*x[j];
      xn[i] = (s+a[i][i]*x[i])/a[i][i];	/* se elimina termenul cu x[i] */
      s = fabs(xn[i]-x[i]);	/* x nou - x vechi */
      if (err < s) err=s;
      }
    for ( i=0; i<n; i++)
      x[i] = xn[i];
    k++;	/* incrementeaza contor iteratii */
    if (err>1.e+38) {ier = 1;break;}
    if (k<=10 || (k%10) == 0)
      {
      printf("Iteratia: %d  Eroarea: %e\n",k,err);
      if (lin == 20)
	{
	printf("Pentru continuare apasati <Enter>:");
	getchar();
	clrscr();
	lin = 1;
	}
      else lin++;
      }
  } while ((err > *eps) && (k <= *nrit));  /* cit timp.cond. e indepl.*/
  *eps = err;
  if (k>*nrit)
    {
    *nrit = k;
    return 2;	/* eroare: nr. iteratii */
    }
  *nrit = k;
  if (ier) return 3; /* eroare:overflow */
  return 0;	/* all OK */
}


/* Rezolvarea unui sistem de ecuatii prin metoda Gauss-Seidel*/
/*-----------------------------------------------------------*/
/* Data : 28.01.1991
  Procedura este facuta fara pivotare.
  Restrictie : oricare a[i][i] nenul
  Intoarce : 0 = all OK
	     1 = pivot nul
	     2 = depasire nr. iteratii
	     3 = overflow */

seidel(int n, float (*a)[N], float b[N], float x[N], int *nrit, float *eps)
{

  int i,j,k,lin,ier;
  float s,err, temp;


  /* Initializari */
  k=0;	/* contor iteratii */
  ier = 0; /* indicator overflow */
  lin = 1;
  for (i=0; i<n; i++)
    {
    if (a[i][i] == 0) return 1; 	/* eroare */
    x[i] = 0;  /* solutia */
    }

  do
    {
    err = 0;			/* eroarea */
    for ( i=0; i<n; i++)	/* ecuatia curenta */
      {
      s = b[i];
      for (j=0; j<n; j++)	/* parcurge ecuatia */
	s -= a[i][j]*x[j];
      s = (s+a[i][i]*x[i])/a[i][i];	/* se elimina termenul cu x[i] */
      temp = fabs(s-x[i]);	/* x nou - x vechi */
      if (err < temp) err=temp;
      x[i]=s;
      }
    k++;	/* incrementeaza contor iteratii */
    if (err>1.e+38) {ier = 1;break;}
    if (k<=10 || (k%10) == 0)
      {
      printf("Iteratia: %d  Eroarea: %e\n",k,err);
      if (lin == 20)
	{
	printf("Pentru continuare apasati <Enter>:");
	getchar();
	clrscr();
	lin = 1;
	}
      else lin++;
      }
  } while ((err > *eps) && (k <= *nrit));  /* cit timp.cond. e indepl.*/
  *eps = err;
  if (k>*nrit)
    {
    *nrit = k;
    return 2;	/* eroare: nr. iteratii */
    }
  *nrit = k;
  if (ier) return 3;
  return 0;	/* all OK */
}



void main()
{

int nrit;
float norma,eps,eps1;

highvideo();
do{
  clrscr();
  printf("\n\t\tPROGRAM EVALUARE METODE ITERATIVE\n");
  printf(  "\t\t=================================\n\n\n");
  printf("Determina erori de calcul la rezolvarea, prin metodele Jacobi si \n");
  printf("Gauss-Seidel, a sistemelor de ecuatii liniare generate aleator.\n\n");
  printf("Introduceti numarul de ecuatii (min.2, max.%d, altfel - iesire): ",N);
  scanf("%d",&n);
  if (n<2 || n>N) break;
  printf("Introduceti norma matricii de iteratie (intre 0 si 1): ");
  scanf("%f",&norma);
  if (norma<=0. || norma>=1.) continue;
  printf("Introduceti eroarea maxima: ");
  scanf("%f",&eps1);

  /* Generare sistem de ecuatii */
  gen_sist_iter(n,norma);

  fflush(stdin);
  for (i=0; i<2; i++)
    {
    nrit = MAXIT;
    eps = eps1;
    clrscr();
    switch(i)
      {
      case 0:
	cprintf("     Metoda Jacobi\n\r");
	jacobi(n,mat,b,x,&nrit,&eps);
	printf("\nS-a incheiat calculul cu metoda Jacobi.\n\nPentru continuare apasati <Enter>:");
	getchar();
	break;
      case 1:
	cprintf("     Metoda Gauss-Seidel\n\r");
	seidel(n,mat,b,x,&nrit,&eps);
	printf("\nS-a incheiat calculul cu metoda Gauss-Seidel.\n\n");
      }
    } /* for i */

  fflush(stdin);
  printf("Pentru continuare apasati <Enter>:");
  getchar();

  } while(1);
lowvideo();

}  /* end main */

void gen_sist_iter(int n, float norm)
{
  int i,j;
  float s;

  randomize();
  for(i=0; i<n; i++)
    {
    s = 0;
    for (j=0; j<n ; j++)
      {
      mat[i][j] = ((float)(rand())/RAND_MAX  );
      s += fabs(mat[i][j]);
      }
    s -= fabs(mat[i][i]);
    mat[i][i] = s/norm;
    x[i] = ((float)(rand())/RAND_MAX);
    }
  for (i=0; i<n; i++)
    {
    s=0;
    for (j=0; j<n ; j++)
      s += mat[i][j]*x[j];
    b[i] = s;
    }
}
