/*=====================================================
					-- A P R O X A N --
Testare proceduri de rezolvare a sistemelor supradeterminate.
- metoda normala a celor mai mici patrate;
- metoda householder.

Se genereaza pseudo-aleator sisteme de ecuatii care se rezolva.
Fiecare linie a matricii e de forma: 1, x x2,...
Rezultatele nu se afiseaza. Se calculeaza eroarea maxima
si timpul de lucru, care se afiseaza.

Data : 28.08.1991
Data ultimei modificari: 30.08.1991
Programator : Irina Munteanu
======================================================== */

#define N 41
#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include "aprxpack.c"

void gen_sist(int m, int n);   /* Declarare proc. de generare de sisteme aleatoare */


/* Declaratiile s-au mutat temporar aici */
float mat[N][N],  /* matricea a */
      b[N], 	  /* vectorul termenilor liberi */
      xa[N],	  /* vectorul rezultatelor adevarate */
      x[N];	  /* vectorul rezultatelor obtinute dupa rezolvare */
int   n,	  /* numarul de necunoscute */
	  m,	  /* Numarul de ecuatii */
      i,j,	  /* variabile contor */
      car;	  /* caracter tastat la consola */




void main()
{

clock_t start,end; /* inceput, sfirsit rulare procedura */
float errcrt, 		/* eroarea curenta */
      rezcrt,		/* norma curenta */
      erre, errc,	/* norma erorii: euclidiana, Cebisev */
      reze, rezc,	/* norma reziduului: euclidiana, Cebisev */
      bnorme, bnormc,	/* norma lui b: euclidiana, Cebisev */
	  xanorme, xanormc;	/* norma solutiei adevarate: euclidiana, Cebisev */
float mat1[N][N], /* copia matricii a */
      b1[N]; 	  /* copia vectorului termenilor liberi */

highvideo();
do{
  clrscr();
  printf("\n\t\tLEAST SQUARES METHODS - ANALYSIS PROGRAM\n");
  printf(  "\t\t========================================\n\n\n");
  printf("Displays errors and computing time when solving randomly generated\n");
  printf("overdetermined linear systems by normal LS and Householder methods.\n\n");
  printf("Enter number of equations (min.2, max.%d, else - exit): ",N-1);
  scanf("%d",&m);
  if (m<2 || m>N-1) break;

  printf("Enter number of unknowns (min.2, max.%d, else - exit): ",N-1);
  scanf("%d",&n);
  if (n<2 || n>N-1 ) break;


  /* Generare sistem de ecuatii */
  gen_sist(m,n);

  /* ============ Metoda normala ================ */
  /* Copiere matrice a, vector b pentru rezolvare */
  for (i=0;i<m;i++)
    {
    for (j=0; j<n; j++)
      mat1[i][j] = mat[i][j];
    b1[i] = b[i];
    }

  start = clock();	/* Determina timpul scurs */
  cmmp(n,m,mat1,b1,x);
  end = clock();

  clrscr();
  cprintf("\r\nNormal LS method\r\n");

  erre = 0;	/* Calculeaza eroarea */
  errc = 0;
  reze = 0;
  rezc = 0;
  bnorme = 0;	/* Calculeaza eroarea */
  bnormc = 0;
  xanorme= 0;
  xanormc= 0;

  /* Calculeaza norma lui x adevarat si a lui b */
  for (i=0; i<m; i++)
    {
    errcrt = fabs(b[i]);
    if ( errcrt > bnormc) bnormc = errcrt;
    bnorme += errcrt*errcrt;
	}

  for (i=0; i<n; i++)
    {
    errcrt = fabs(xa[i]);
    if ( errcrt > xanormc) xanormc = errcrt;
    xanorme += errcrt*errcrt;
    }


  for (i=0; i<n; i++)
    {
    errcrt = fabs(xa[i] - x[i]);
    if (errcrt > errc) errc = errcrt;
    erre = erre + errcrt*errcrt;
	}

  for (i=0; i<m; i++)
    {
    rezcrt = 0;
    for (j=0; j<n; j++)
      rezcrt += mat[i][j]*x[j];
    rezcrt = fabs(rezcrt - b[i]);
    if (rezcrt > rezc) rezc = rezcrt;
    reze = reze + rezcrt*rezcrt;
    }

  printf("Error's norm:\t\tabsolute\trelative\n");
  printf("\t* Euclidian : \t%e\t%e\n", sqrt(erre),sqrt(erre/xanorme));
  printf("\t* Tchebishev: \t%e\t%e\n", errc, (errc/xanormc));
  printf("\nResiduum norm  :\tabsoluta\trelativa\n");
  printf("\t* Euclidian : \t%e\t%e\n", sqrt(reze), sqrt(reze/bnorme));
  printf("\t* Tchebishev: \t%e\t%e\n", rezc, (rezc/bnormc));
  printf("\nComputing time : %f seconds\n\n",(end - start)/ CLK_TCK);


  /* ========== Metoda Householder ===============*/
  /* Copiere matrice a, vector b pentru rezolvare */
  for (i=0;i<m;i++)
    {
    for (j=0; j<n; j++)
      mat1[i][j] = mat[i][j];
    b1[i] = b[i];
    }

  cprintf("\nHouseholder method\r\n");

  if (n>m)
	{
	printf("Divide by 0");
	getch();
	continue;
	}
  start = clock();	/* Determina timpul scurs */
  householder(n,m,mat1,b1,x);
  end = clock();


  erre = 0;	/* Calculeaza eroarea */
  errc = 0;
  reze = 0;
  rezc = 0;
  bnorme = 0;	/* Calculeaza eroarea */
  bnormc = 0;
  xanorme= 0;
  xanormc= 0;


  /* Calculeaza norma lui x adevarat si a lui b */
  for (i=0; i<m; i++)
    {
    errcrt = fabs(b[i]);
    if ( errcrt > bnormc) bnormc = errcrt;
    bnorme += errcrt*errcrt;
	}

  for (i=0; i<n; i++)
    {
    errcrt = fabs(xa[i]);
    if ( errcrt > xanormc) xanormc = errcrt;
    xanorme += errcrt*errcrt;
    }

  for (i=0; i<n; i++)
    {
    errcrt = fabs(xa[i] - x[i]);
    if (errcrt > errc) errc = errcrt;
    erre = erre + errcrt*errcrt;
	}

  for (i=0; i<m; i++)
    {
    rezcrt = 0;
    for (j=0; j<n; j++)
      rezcrt += mat[i][j]*x[j];
    rezcrt = fabs(rezcrt - b[i]);
    if (rezcrt > rezc) rezc = rezcrt;
    reze = reze + rezcrt*rezcrt;
    }
  printf("Error's norm:\t\tabsolute\trelative\n");
  printf("\t* Euclidian : \t%e\t%e\n", sqrt(erre),sqrt(erre/xanorme));
  printf("\t* Tchebishev: \t%e\t%e\n", errc, (errc/xanormc));
  printf("\nResiduum norm  :\tabsoluta\trelativa\n");
  printf("\t* Euclidian : \t%e\t%e\n", sqrt(reze), sqrt(reze/bnorme));
  printf("\t* Tchebishev: \t%e\t%e\n", rezc, (rezc/bnormc));
  printf("\nComputing time : %f seconds\n\n",(end - start)/ CLK_TCK);

  fflush(stdin);
  printf("Press any key ...");
  getch();

  } while(1);

}  /* end main */

void gen_sist(int m, int n)
{
  int i,j;
  float s;
  float x[N];

	/* Initializeaza generatorul de numere aleatoare */
	srand(1);

	/* Construieste tablou de aproximat */
	for (i=0; i<m; i++)
		x[i] = (float)i/(m-1);

	/* Genereaza matricea coeficientilor nec. */
	for (i=0; i<m; i++)
		{
		mat[i][0] = 1;
		for (j=1; j<n; j++)
			mat[i][j] = mat[i][j-1] * x[i];
		}
	for (i=0; i<n; i++)
		xa[i] = ((float)(rand())/RAND_MAX);

	for (i=0; i<m; i++)
		{
		s=0;
		for (j=0; j<n ; j++)
			s += mat[i][j]*xa[j];
		b[i] = s;
		}
}       /* End gen_sist */
