/* 				==   APROXLP  ==
   Program demonstrativ de aproximare liniara pe portiuni prin metoda CMMP.
   =======================================================================

   Data ultimei modificari: 19.08.1991
   Programator: Irina Munteanu

   ====================================================================*/
#define M 51
#define N M
#define z mat[0]
#define f mat[1]
#define g mat[2]

#include <math.h>
#include <time.h>
#include <graphics.h>
#include <stdlib.h>
#include <conio.h>
#include <stdio.h>
#include "grafic.c"
#include "aprxpack.c"
#include "pp_pack.c"

float interpolare(int nr);
int introd_date(void);
float eval_pol(int n, float c[N], float x);
void titlu(int nrset);

int n;	/* numarul de segmente */
int oldset;	/* numarul vechiului set de date */
int m=40;	/* numarul de puncte in care se cunoaste valoarea functiei */
float	x[M],		/* nodurile retelei de interpolare */
		xf[N],		/* punctele de fringere */
		y[M],		/* date de interpolare (valori functie) */
		yy[M],		/* date de interpolare */
		b[N],		/* coeficienti din formula de interpolare */
		xmin,		/* lim. inf. domeniu de definitie */
		xmax,		/* lim. sup. domeniu de definitie */
		mat[3][M];	/* prima linie: z = abscise
					   a doua linie: f = valori "exacte" ale functiei
					   a treia linie: g = valori aproximate
							- toate definite cu #DEFINE */

void main()
{
int nrset;	/* numarul setului de date "experimentale" */
float err;

initial();	/* Initializare mod grafic */
restorecrtmode();
oldset = 0;
do
	{
	nrset = introd_date();
	if (nrset ==  0) break;
	err = interpolare(nrset);
	setgraphmode(getgraphmode());
	redefinire(1,'o','\0',RED);
	plot(2,m,mat);
	grid();
	titlu(nrset);
	getch();
	restorecrtmode();
	printf("Eroarea patratica medie: %g\n\nApasati o tasta", err);
	getch();
	oldset = nrset;
	} while (1);
}	/* End main */


/*================
  Introducere date
==================*/
int introd_date()
{
int nr;

clrscr();
printf("\n\n\tPROGRAM DE DEMONSTRATIE PENTRU APROXIMAREA LINIARA PE PORTIUNI\n");
printf(    "\t==============================================================\n\n");
printf(" Selectati setul de date experimentale (1 - 4, altfel iesire): ");
scanf("%d",&nr);
if (nr<1 || nr>4) return (0);
printf("\nNumarul de segmente (min.1, max.20): ");
scanf("%d", &n);
if (n<1 || n>20) return (0);

return nr;
}	/* End introd_date */

/*=======================================
  Intoarce valoarea minima a intervalului
  de definitie al functiei de aproximat
==========================================*/
float minf(int nrset)
{
	switch(nrset)
		{
		case 1:
			return -3.;
		case 2:
			return -1.e4;
        case 3:
			 return -1.;
		case 4:
			 return 0.;
        default:
			 return -10.;
		}
}	/* End minf */

/*=======================================
  Intoarce valoarea maxima a intervalului
  de definitie al functiei de aproximat
==========================================*/
float maxf(int nrset)
{
	switch(nrset)
		{
		case 1:
			return 5.;
		case 2:
			return 1.e4;
        case 3:
			 return 1.;
		case 4:
			 return 10.;
        default:
			 return 10.;
		}
}	/* End maxf */


/*=======================================
  Intoarce valoarea functiei de interpolat
==========================================*/
float func(int nr, float x)
{
float val, miu0;

	switch(nr)
		{
		case 1:
			val = 2*x*x + 4*x + 8;break;
		case 2:
			miu0 = 4.* atan(1.);	/* pi */
			miu0 *= 4.*1.e-7;
			val = /*tanh(x)*/miu0*x + 1.8* tanh(1000.*miu0*x/1.8);break;
				/* Bs = 1.8; miu_r = 1000;  = curba de magnetizare */
        case 3:
			 val = -1.e-9 * (1. - exp(x/0.037));break;
				/* Is = -1.e-6;  Vt = 0.02;  = dioda */
		case 4:
			val = x/(1+x)/(1+x);
		}
	return (val + val/10.*rand()/RAND_MAX);

}	/* End func */


/*============================================
Calculeaza valorile interpolate ale functiei
==============================================*/
float interpolare(int nrset)
{
int i, j, k;
float eps;
float a[M][N];

if (nrset != oldset)
	{
	/* Initializeaza generatorul de numere aleatoare */
	randomize();

	/* Construieste tablou de aproximat */
	xmin = minf(nrset);
	xmax = maxf(nrset);
	for (k=0; k<m; k++)
		{
		x[k] = z[k] = xmin + (xmax-xmin)*k/(m-1);
		y[k] = f[k] = func(nrset, x[k]); /* In viitor s-ar putea desena doua
									grafice: y(x) si apoi f(z) cu hold */
		}
	}	/* if nrset != oldset */

/* Genereaza punctele de fringere */
for (k=0; k<=n; k++)
	xf[k] = xmin + (xmax - xmin)/n*k;

/* Genereaza matricea coeficientilor nec. */
for (i=0; i<m; i++)
	for (j=0; j<=n; j++)
		a[i][j] = 0;

for (i=0; i<m; i++)
	{
	k = cauta_seg(n, xf, x[i]);
	a[i][k] = (x[i] - xf[k+1])/(xf[k] - xf[k+1]);
	a[i][k+1] = (x[i] - xf[k])/(xf[k+1] - xf[k]);
	yy[i] = y[i];
	}

/* Daca o coloana k este nula, trebuie adaugata inca o ecuatie cu coeficientii:
	a[m+1][k-1] = -1
	a[m+1][k] = 2
	a[m+1][k+1] = -1
	yy[m+1] = 0; */

/* Rezolva sistemul prin CMMP */
/*cmmp*/householder(n+1, m, a, yy, b);

/* Evalueaza functia in cele m puncte */
for (i=0; i<m; i++)
	g[i] = eval_l(n, xf, b, z[i]);

/* Calculeaza eroarea patratica medie */
eps = 0;
for (i=0; i<m; i++)
	eps += pow( (f[i] - g[i]), 2.);
eps = sqrt (eps);

return (eps);

}	/* End interpolare */


/*==========================
  Afiseaza titlul graficului
  ==========================*/
void titlu(int nrset)
{

switch(nrset)
	{
	case 1:
		return;
	case 2:
		title("CURBA DE MAGNETIZARE"); return;
	case 3:
		title("CARACTERISTICA UNEI DIODE"); return;
	case 4:
		title("P = P(R)");
	}
}	/* End titlu */


