/*#define N 50
#include <math.h>*/
#include "gaussfp.c"

/*			= APROX_PACK =
Pachet de proceduri pentru aproximarea functiilor si rezolvarea
sistemelor prin metoda celor mai mici patrate.
==============================================================

Data ultimei modificari: 19.08.1991
Programator: Irina Munteanu

Pachetul contine urmatoarele functii:
- householder - metoda Householder pentru rezolvarea sistemelor de ecuatii
				supradeterminate
- cmmp - metoda clasica pentru rezolvarea sistemelor de ecuatii
				supradeterminate

/*=======================================================
Rezolva sisteme de ecuatii supradeterminate prin metoda celor
mai mici patrate - Householder
==============================================================*/
void householder(int n,			/* numar de necunoscute */
				int m,			/* numar de ecuatii */
			float (*a)[N],	/* matricea coeficientilor */
			float b[N], 	/* vectorul termenilor liberi */
			float x[N])		/* vectorul necunoscutelor */

{
int k, i, j;
float sigma, beta, tau, uk, s;

/* Triangularizare matrice */
for (k=0; k<n; k++)
	{
	sigma = 0;
	for (i=k; i<m; i++)
		sigma += (a[i][k]*a[i][k]);
	sigma = sqrt(sigma);
	if (a[k][k] <0) sigma = -sigma;
	uk = sigma +a[k][k];
	beta = sigma * uk;
	a[k][k] = -sigma;

	for (j=k+1; j<n; j++)	/* Parcurge coloanele urmatoare */
		{
		tau = a[k][j] * uk;
		for (i=k+1; i<m; i++)
			tau += a[i][j] * a[i][k];	/* u[i][k] = a[i][k] */
		tau /= beta;
		a[k][j] -= (tau * uk);
		for (i=k+1; i<m; i++)
			a[i][j] -= (tau * a[i][k]);
		}	/* j */
	tau = uk * b[k];	/* Inmulteste term. liber cu reflectorul u */
	for (i=k+1; i<m; i++)
		tau += a[i][k] * b[i];
	tau /= beta;
	b[k] -= (tau * uk);
	for (i=k+1; i<m; i++)
		b[i] -= (tau * a[i][k]);
	}	/* i */

/* Rezolvare sistem UAx = Ub <==> Rx = d  prin retrosubstitutie */
for (i=n-1; i>=0; i--)
	{
	s = b[i];
	for (j = i+1; j<n; j++)
		s -= (a[i][j] * x[j]);
	x[i] = s/a[i][i];
	}
} 	/* End householder */

/*=======================================================
Rezolva sisteme de ecuatii supradeterminate prin metoda celor
mai mici patrate - clasica. Det. solutia sist. A_t.A.x = A_t.b
==============================================================*/
void cmmp(int n,			/* numar de necunoscute */
		  int m,			/* numar de ecuatii */
		  float (*a)[N],	/* matricea coeficientilor */
		  float b[N], 	/* vectorul termenilor liberi */
		  float x[N])		/* vectorul necunoscutelor */

{
int k, i, j;
float an[N][N],	/* A nou */
	  bn[N];	/* b nou */

/* Detrmina matricea an si vectorul bn */
for (i=0; i<n; i++)
	{
	for (j=0; j<n; j++)
		{
		an[i][j] = 0;
		for (k=0; k<m; k++)
			an[i][j] += a[k][i]* a[k][j];
		}	/* j */
	bn[i] = 0;
	for (k=0; k<m; k++)
		bn[i] += a[k][i]*b[k];
	}	/* i */
gauss_fp(n, an, bn, x);

}	/* End cmmp */