/* 				==   DEMO_BES  ==
   Program demonstrativ de interpolare Bessel.
   ==========================================

   Data ultimei modificari: 06.08.1991
   Programatori: Daniel Ioan, Irina Munteanu

   ====================================================================*/
#define M 51
#define N M
#define z mat[0]
#define f mat[1]
#define g mat[2]

#include <math.h>
#include <conio.h>
#include <stdio.h>
#include "grafic.c"
#include "pp_pack.c"

float interpolare(int nr);
int introd_date(void);


int n;	/* numarul de puncte de interpolat */
int m=49;	/* numar puncte din grafic */
float	x[N],		/* nodurile retelei de interpolare */
		y[N],		/* date de interpolare (valori functie) */
		xmin,		/* lim. inf. domeniu de definitie */
		xmax,		/* lim. sup. domeniu de definitie */
		mat[3][M];	/* prima linie: z = abscise
					   a doua linie: f = valori exacte ale functiei
					   a treia linie: g = valori interpolate
							- toate definite cu #DEFINE */

void main()
{
int nrfunc;
float err;

initial();	/* Initializare mod grafic */
restorecrtmode();
do
	{
	nrfunc = introd_date();
	if (nrfunc ==  0) break;
	err = interpolare(nrfunc);
	setgraphmode(getgraphmode());
	plot(2,m,mat);
	grid();
	getch();
	restorecrtmode();
	printf("Eroarea Cebisev: %g\n\nApasati o tasta", err);
	getch();
	} while (1);
}	/* End main */


/*================
  Introducere date
==================*/
int introd_date()
{
int nr;

clrscr();
printf("\n\n\tPROGRAM DE DEMONSTRATIE PENTRU INTERPOLAREA BESSEL\n");
printf(    "\t==================================================\n\n");
printf("\t1  sin(x)\n\t2  exp(x)\n\t3  ln(x)\n\t4  th(x)\n\t5  Runge: 1/(1+x^2)\n\t6  |x|\n\t7  sqrt(x)\n\t0  Iesire\n\n");
printf(" Selectati optiune: ");
scanf("%d",&nr);
if (nr<1 || nr>7) return (0);
printf("\nLimita inferioara a domeniului de definitie: ");
scanf("%f", &xmin);
printf("\nLimita superioara a domeniului de definitie: ");
scanf("%f", &xmax);
printf("\nNumarul de intervale (min.1, max.20): ");
scanf("%d", &n);
if (n<1 || n>20) return (0);

return nr;
}	/* End introd_date */


/*=======================================
  Intoarce valoarea functiei de interpolat
==========================================*/
float func(int nr, float x)
{
	switch(nr)
		{
		case 1:
			return sin(x);
		case 2:
			return exp(x);
        case 3:
             return log(fabs(x));
		case 4:
			return tanh(x);
		case 5:
			return 1/(1+x*x);
        case 6:
             return fabs(x);
        case 7:
        default:
             return sqrt(fabs(x));
		}
}	/* End func */


/*============================================
Calculeaza valorile interpolate ale functiei
==============================================*/
float interpolare(nrfunc)
{
int i, k;
float eps, p;
float yd[N],	/* Derivatele in noduri */
	  c[4][N];	/* Coeficienti polinom */

/* Construieste tablou de interpolat */
for (k=0; k<=n; k++)
	{
	x[k] = xmin + (xmax-xmin)*k/n;
	y[k] = func(nrfunc, x[k]);
	}

/* Construieste functia exacta */
for (i=0; i<m; i++)
	{
	z[i] = xmin + (xmax-xmin)*i/(m-1);
	f[i] = func(nrfunc, z[i]);
	}
prep_bess(n,x,y,yd);
prep_h(n,x,y,yd,c);

/* Evalueaza interpolare */
eps = 0;
for (i=0; i<m; i++)
	{
	g[i] = eval_h(n,x,c,z[i]);
	if (  (p=fabs (g[i]  - f[i])) > eps)
		eps = p;
	}	/* i */

return (eps);

}	/* End interpolare */
