/*					==  POL_AN  ==
=======================================================================
	Program pentru analiza algoritmilor de interpolare polinomiala

Programul calculeaza eroarea si timpul de calcul pentru:
	- metoda clasica
	- metoda Newton
	- metoda Lagrange,
in functie de gradul polinomului.

Functia test de interpolat este sin:[0,pi] --> R


Data: 02.08.1991
Data ultimei modificari: 06.08.1991
Programator: Irina Munteanu

======================================================== */

#define N 51
#define xcr mat[0]
#include <time.h>
#include <math.h>
#include <stdio.h>
#include <conio.h>
#include <graphics.h>
#include "grafic.c"
#include "gaussfp.c"


prep_pol(int n, float x[N],	float y[N],	float c[N]);
prep_pol_n(int n, float x[N],	float y[N],	float c[N]);
float eval_pol(	int n, float c[N], float x);
float eval_pol_n(int n, float c[N], float x[N], float xcrt);
float eval_pol_l(int n, float x[N],	float y[N],	float xcrt);


float x[N], y[N];		/* Tabelul de interpolat */
float a[N][N];			/* Matrice auxiliara */
float	mat[5][N];	/* prima linie: z = abscise
					   a doua linie: f = valori exacte ale functiei
					   a treia linie: g = valori interpolate
							- toate definite cu #DEFINE */

void main()
{
int n,m=50;				/* Gradul polinomului de interpolare, nr.puncte in grafic */
int i,k;				/* Contoare */
float	err,			/* Eroarea */
		tprep,			/* Timp de pregatire a polinomului */
		teval;			/* Timp de evaluare a lui, in xcrt */
clock_t	start, end;		/* timp de inceput, sfirsit */
float c[N];				/* Coeficientii polinoamelor */
float xmin = 0,
	  xmax = 6.3;

do
	{
	initial();	/* Initializare mod grafic */
	restorecrtmode();
	clrscr();
	printf("\n\n\t\tPROGRAM DE ANALIZA A ALGORITMILOR DE INTERPOLARE POLINOMIALA");
	printf("\n\t\t============================================================\n\n");
	printf("Gradul polinomului (min.1, max.%d, altfel - iesire): ", N-1);
	scanf("%d", &n);
	if (n<1 || n> (N-1)) break;
	printf("\nMETODA\t\tTIMP DE PREGATIRE\tTIMP DE EVALUARE\tEROARE\n");

	/* Generare tabel de interpolat */
	for (k=0; k<=n; k++)
		{
		x[k] = xmin + (xmax-xmin)*k/n;
		y[k] = sin(x[k]);
		}
	for (i=0; i<=m; i++)
		{
		xcr[i] = xmin + i*(xmax-xmin)/m;
		mat[1][i] = sin(xcr[i]);		/* Valorile exacte ale functiei */
		}

	/* Metoda Newton */
	start = clock();
	prep_pol_n(n, x,y,c);
	end = clock();
	tprep = (end - start)/CLK_TCK;
	start = clock();
	for (i=0; i<=m; i++)
		mat[2][i] = eval_pol_n(n,c,x,xcr[i]);
	end = clock();
	teval = (end - start)/CLK_TCK/(m+1);
	err = 0;
	for (i=0; i<=m; i++)
		if (err < fabs(mat[1][i] - mat[2][i])) err = fabs(mat[1][i] - mat[2][i]);
	printf("Newton\t\t%15.6e\t\t%15.6e\t\t%g\n", tprep, teval, err);

	/* Metoda Lagrange */
	tprep = 0;
	start = clock();
	for (i=0; i<=m; i++)
		mat[3][i] = eval_pol_l(n,x,y,xcr[i]);
	end = clock();
	teval = (end - start)/CLK_TCK/(m+1);
	err = 0;
	for (i=0; i<=m; i++)
		if (err < fabs(mat[1][i] - mat[3][i])) err = fabs(mat[1][i] - mat[3][i]);
	printf("Lagrange\t%15.6e\t\t%15.6e\t\t%g\n", tprep, teval, err);

	/* Metoda clasica */
	start = clock();
	prep_pol(n, x,y,c);
	end = clock();
	tprep = (end - start)/CLK_TCK;
	start = clock();
	for (i=0; i<=m; i++)
		mat[4][i] = eval_pol(n,c,xcr[i]);
	end = clock();
	teval = (end - start)/CLK_TCK/(m+1);
	err = 0;
	for (i=0; i<=m; i++)
		if (err < fabs(mat[1][i] - mat[4][i])) err = fabs(mat[1][i] - mat[4][i]);
	printf("Clasica\t\t%15.6e\t\t%15.6e\t\t%g\n", tprep, teval, err);

	printf("\n\nApasati o tasta ...");
	getch();
	setgraphmode(getgraphmode());
	plot(4,m,mat);
	grid();
	getch();
	restorecrtmode();

	} while (1);

}	/* End main */

/*==========================================================
  Calculeaza coeficientii c0, c1, ..., cn ai polinomului
  de interpolare pentru reteaua x - y, prin metoda "clasica"
  ==========================================================*/
prep_pol(int n,			/* Gradul pol. */
		 float x[N],	/* Abscise */
		 float y[N],	/* Ordonate */
		 float c[N])	/* Coeficienti */

{
int i, j;	/* contoare */

/* Genereaza matricea sistemului liniar */
for (i=0; i<=n; i++)
	{
	a[i][0] = 1;
	for (j=1; j<=n; j++)
		a[i][j] = a[i][j-1] * x[i];
	}

/* Rezolva sistem liniar */
gauss_fp(n+1, a, y, c);

}	/* End prep_pol */

/*==========================================================
  Calculeaza coeficientii c0, c1, ..., cn din polinomul
  de interpolare Newton pentru reteaua x - y
  ==========================================================*/
prep_pol_n(int n,		/* Gradul pol. */
		 float x[N],	/* Abscise */
		 float y[N],	/* Ordonate */
		 float c[N])	/* Coeficienti */

{
int i, j;	/* contoare */
float s;

/* Genereaza matricea sistemului liniar */
for (i=0; i<=n; i++)
	{
	a[i][0] = 1;
	for (j=1; j<=i; j++)
		a[i][j] = a[i][j-1] * (x[i]- x[j-1]);
	}

/* Rezolva sistem liniar prin substitutie progresiva */
c[0] = y[0];
for (i=1; i<=n; i++)
	{
	s = y[i];
	for (j=0; j<i; j++)
		s -= a[i][j]*c[j];
	c[i] = s/a[i][i];
	}	/* i */

}	/* End prep_pol_n */


/* =================
   Evalueaza polinom
   =================*/
float eval_pol(	int n,		/* Grad polinom */
				float c[N],	/* Coeficienti */
				float x)	/* Variabila independenta */
{
int i;
float valp;		/* Valoare polinom */

valp = c[n];
for (i=n-1; i>=0; i--)
	valp = c[i]+ x*valp;

return (valp);

}	/* End eval_pol */

/* ========================
   Evalueaza polinom Newton
   ======================== */
float eval_pol_n(	int n,		/* Grad polinom */
				float c[N],	/* Coeficienti */
				float x[N],	/* Nodurile */
				float xcrt)	/* Variabila independenta */
{
int i;
float valp,		/* Valoare polinom */
	  xp;		/* Puterile lui x */

valp = c[0];
xp = 1;
for (i=1; i<=n; i++)
	{
	xp *= (xcrt-x[i-1]);
	valp += c[i]*xp;
	}
return (valp);

}	/* End eval_pol_n */

/* ===========================
   Evalueaza polinom Lagrange
   ======================== */
float eval_pol_l(	int n,		/* Grad polinom */
				float x[N],	/* Abscisele */
				float y[N],	/* Ordonatele */
				float xcrt)	/* Variabila independenta */
{
int i,j;
float valp,		/* Valoare polinom */
	  p;		/* Coeficient */

valp = 0;
for (i=0; i<=n; i++)
	{
	p = 1;
	for (j=0; j<=n; j++)
		if (i != j)
			p *= ((xcrt - x[j])/(x[i] - x[j]));
	valp += y[i]*p;
	}
return (valp);

}	/* End eval_pol_l */
