/*					==  PP_AN  ==
=======================================================================
	Program pentru analiza algoritmilor de interpolare polinomiala
						pe portiuni

Programul calculeaza eroarea si timpul de calcul pentru:
	- interpolarea liniara pe portiuni cu retea uniforma
	- interpolarea liniara pe portiuni cu retea adaptata functie
	- interpolarea Bessel
	- interpolarea Akima
	- interpolarea spline cu conditii naturale la limita (evaluare
	  cu functii B-spline).
	- interpolarea spline cu calculul aproximativ al conditiilor la limita
	  (evaluare polinomiala pe portiuni).
in functie de numarul de intervale.

Functia test de interpolat este sqrt:[0,1] --> R


Data: 08.08.1991
Data ultimei modificari: 19.08.1991
Programator: Irina Munteanu

======================================================== */

#define N 51
#define xcr mat[0]
#include <time.h>
#include <math.h>
#include <stdio.h>
#include <conio.h>
#include <graphics.h>
#include "grafic.c"
#include "pp_pack.c"



float x[N], y[N];		/* Tabelul de interpolat */
float	mat[5][N];	/* prima linie: z = abscise
					   a doua linie: f = valori exacte ale functiei
					   a treia linie: g = valori interpolate
							- toate definite cu #DEFINE */

void main()
{
int n,m=50;				/* Nr.de intervale, nr. puncte in grafic */
int i,k;				/* Contoare */
float	err,			/* Eroarea */
        tasez,          /* Timp de asezare !!! */
		tprep,			/* Timp de pregatire a polinomului */
		teval;			/* Timp de evaluare a lui, in xcrt */
clock_t	start, end;		/* timp de inceput, sfirsit */
float c[4][N],			/* Coeficientii polinoamelor */
	  yd[N];			/* Tabelul derivatelor */
float xmin = 0,
	  xmax = 1.;

do
	{
	initial();	/* Initializare mod grafic */
	restorecrtmode();
	clrscr();
	printf("\n\n  PROGRAM DE ANALIZA A ALGORITMILOR DE INTERPOLARE POLINOMIALA PE PORTIUNI");
	printf(  "\n  ========================================================================\n\n");
	printf("Numarul de intervale (min.2, max.%d, altfel - iesire): ", N-11);
	scanf("%d", &n);
	if (n<2 || n> (N-11)) break;
	printf("\nMETODA\t\tTIMP DE PREGATIRE\tTIMP DE EVALUARE\tEROARE\n");
	highvideo();
	cprintf("Retea uniforma\n\r");

	/* Generare tabel de interpolat */
	for (k=0; k<=n; k++)
		{
		x[k] = xmin + (xmax-xmin)*k/n;
		y[k] = sqrt(x[k]);
		}
	for (i=0; i<=m; i++)
		{
		xcr[i] = xmin + i*(xmax-xmin)/m;
		mat[1][i] = sqrt(xcr[i]);		/* Valorile exacte ale functiei */
		}

	/* Metoda de interpolare liniara pe portiuni uniforme */
	tprep = 0;
	start = clock();
	for (i=0; i<=m; i++)
		mat[2][i] = eval_l(n,x,y,xcr[i]);
	end = clock();
	teval = (end - start)/CLK_TCK/(m+1);
	err = 0;
	for (i=0; i<=m; i++)
		if (err < fabs(mat[1][i] - mat[2][i])) err = fabs(mat[1][i] - mat[2][i]);
	printf("Liniar uniform\t%15.6e\t\t%15.6e\t\t%g\n", tprep, teval, err);


	/* Metoda Bessel */
	start = clock();
	prep_bess(n, x,y,yd);
	prep_h(n,x,y,yd,c);
	end = clock();
	tprep = (end - start)/CLK_TCK;
	start = clock();
	for (i=0; i<=m; i++)
		mat[4][i] = eval_h(n,x,c,xcr[i]);
	end = clock();
	teval = (end - start)/CLK_TCK/(m+1);
	err = 0;
	for (i=0; i<=m; i++)
		if (err < fabs(mat[1][i] - mat[4][i])) err = fabs(mat[1][i] - mat[4][i]);
	printf("Bessel\t\t%15.6e\t\t%15.6e\t\t%g\n", tprep, teval, err);

	/* Metoda Akima */
	start = clock();
	prep_akima(n, x,y,yd);
	prep_h(n,x,y,yd,c);
	end = clock();
	tprep = (end - start)/CLK_TCK;
	start = clock();
	for (i=0; i<=m; i++)
		mat[4][i] = eval_h(n,x,c,xcr[i]);
	end = clock();
	teval = (end - start)/CLK_TCK/(m+1);
	err = 0;
	for (i=0; i<=m; i++)
		if (err < fabs(mat[1][i] - mat[4][i])) err = fabs(mat[1][i] - mat[4][i]);
	printf("Akima\t\t%15.6e\t\t%15.6e\t\t%g\n", tprep, teval, err);

	/* Metoda spline natural */
	start = clock();
	prep_spln(n, x,y,yd);
	end = clock();
	tprep = (end - start)/CLK_TCK;
	start = clock();
	for (i=0; i<=m; i++)
		mat[4][i] = eval_b(n,x,y,yd,xcr[i]);
	end = clock();
	teval = (end - start)/CLK_TCK/(m+1);
	err = 0;
	for (i=0; i<=m; i++)
		if (err < fabs(mat[1][i] - mat[4][i])) err = fabs(mat[1][i] - mat[4][i]);
	printf("B-Spline\t%15.6e\t\t%15.6e\t\t%g\n", tprep, teval, err);

	/* Metoda spline aproximativ */
	start = clock();
	prep_spla(n, x,y,yd);
	prep_h(n,x,y,yd,c);
	end = clock();
	tprep = (end - start)/CLK_TCK;
	start = clock();
	for (i=0; i<=m; i++)
		mat[4][i] = eval_h(n,x,c,xcr[i]);
	end = clock();
	teval = (end - start)/CLK_TCK/(m+1);
	err = 0;
	for (i=0; i<=m; i++)
		if (err < fabs(mat[1][i] - mat[4][i])) err = fabs(mat[1][i] - mat[4][i]);
	printf("Spline\t\t%15.6e\t\t%15.6e\t\t%g\n", tprep, teval, err);

	cprintf("Retea neuniforma adaptata la date\r\n");

	/* Metoda de interpolare liniara pe portiuni neuniforme */
	start = clock();
	asezare(n, x,y);
	end = clock();
	for (k=0; k<=n; k++)
		y[k] = sqrt(x[k]);
	tprep = tasez = (end - start)/CLK_TCK;
	start = clock();
	for (i=0; i<=m; i++)
		mat[3][i] = eval_l(n,x,y,xcr[i]);
	end = clock();
	teval = (end - start)/CLK_TCK/(m+1);
	err = 0;
	for (i=0; i<=m; i++)
		if (err < fabs(mat[1][i] - mat[3][i])) err = fabs(mat[1][i] - mat[3][i]);
	printf("Liniar          %15.6e\t\t%15.6e\t\t%g\n", tprep, teval, err);

	/* Metoda Bessel */
	start = clock();
	prep_bess(n, x,y,yd);
	prep_h(n,x,y,yd,c);
	end = clock();
	tprep = (end - start)/CLK_TCK + tasez;
	start = clock();
	for (i=0; i<=m; i++)
		mat[4][i] = eval_h(n,x,c,xcr[i]);
	end = clock();
	teval = (end - start)/CLK_TCK/(m+1);
	err = 0;
	for (i=0; i<=m; i++)
		if (err < fabs(mat[1][i] - mat[4][i])) err = fabs(mat[1][i] - mat[4][i]);
	printf("Bessel\t\t%15.6e\t\t%15.6e\t\t%g\n", tprep, teval, err);

	/* Metoda Akima */
	start = clock();
	prep_akima(n, x,y,yd);
	prep_h(n,x,y,yd,c);
	end = clock();
	tprep = (end - start)/CLK_TCK + tasez;
	start = clock();
	for (i=0; i<=m; i++)
		mat[4][i] = eval_h(n,x,c,xcr[i]);
	end = clock();
	teval = (end - start)/CLK_TCK/(m+1);
	err = 0;
	for (i=0; i<=m; i++)
		if (err < fabs(mat[1][i] - mat[4][i])) err = fabs(mat[1][i] - mat[4][i]);
	printf("Akima\t\t%15.6e\t\t%15.6e\t\t%g\n", tprep, teval, err);

	/* Metoda spline natural */
	start = clock();
	prep_spln(n, x,y,yd);
	end = clock();
	tprep = (end - start)/CLK_TCK + tasez;
	start = clock();
	for (i=0; i<=m; i++)
		mat[4][i] = eval_b(n,x,y,yd,xcr[i]);
	end = clock();
	teval = (end - start)/CLK_TCK/(m+1);
	err = 0;
	for (i=0; i<=m; i++)
		if (err < fabs(mat[1][i] - mat[4][i])) err = fabs(mat[1][i] - mat[4][i]);
	printf("B-Spline\t%15.6e\t\t%15.6e\t\t%g\n", tprep, teval, err);

	/* Metoda spline aproximativ */
	start = clock();
	prep_spla(n, x,y,yd);
	prep_h(n,x,y,yd,c);
	end = clock();
	tprep = (end - start)/CLK_TCK + tasez;
	start = clock();
	for (i=0; i<=m; i++)
		mat[4][i] = eval_h(n,x,c,xcr[i]);
	end = clock();
	teval = (end - start)/CLK_TCK/(m+1);
	err = 0;
	for (i=0; i<=m; i++)
		if (err < fabs(mat[1][i] - mat[4][i])) err = fabs(mat[1][i] - mat[4][i]);
	printf("Spline\t\t%15.6e\t\t%15.6e\t\t%g\n", tprep, teval, err);

	printf("\n\nApasati o tasta ...");
	getch();
	setgraphmode(getgraphmode());
	plot(3,m,mat);
    title("Interpolarile liniare pe portiuni");
	grid();
	getch();
	restorecrtmode();

	} while (1);

}	/* End main */

