/* =====================================================================

			D E M O C N
Program demonstrativ pentru analiza circuitelor rezistive neliniare
Dedicat circuitelor cu rezistente, surse de tensiune si diode semiconductoare
Metoda folosita este caracterizata prin:
	- aproximarea liniara pe portiuni a caracteristicii diodei;
	- metoda circuitului liniarizat (Newton);
	- rezolvarea circuitului liniar prin tehnica nodala.


   Date de intrare:
	- numar de noduri;
	- numar de laturi;
	- pentru fiecare latura:
		- nodul initial;
		- nodul final;
		- rezistenta;
		- t.e.m.;
		- existenta diodei;
	- numarul de segmente folosite in aproximarea liniara pe portiuni a
	  diodei.

   Date de iesire:
	- norma corectiei potentialelor la fiecare iteratie;
	- timpul total de calcul;
	- pentru fiecare latura: curentul;
	- puterea generata si consumata.

Data : 30.08.1991
Programator: Daniel Ioan
========================================================================== */

#define N 50
#define M 50
#define L 100
#include <stdio.h>
#include <conio.h>
#include <math.h>
#include <time.h>
#include <dos.h>
#include "pp_pack.c"
float x[M], y[M];     /* Caracteristica diodei */

/* =====================================================
   Analiza circuitelor rezistive liniare cu metoda nodala
   Intoarce cod de eroare */
int nodal(int n,         /* nr noduri */
	  int l,	 /* nr laturi*/
	  int ni[],      /* tabelul nodurilo initale*/
	  int nf[],	 /* tabelul nodurilur finale */
	  float r[],	 /* tabelul rezistentlor */
	  float e[],	 /* tabelul t.e.m. */
	  float v[])     /* Rezultate: potentialele nodurilor */

{
int i, j, n1, n2;
float is[N],  		/* termenii liberi */
      g[N][N];          /* matricea sistemului */

for (i=0; i<n; i++) {
	is[i] = 0;
	for (j=0; j<n; j++)
		g[i][j] = 0;
	}
for (i=0; i<l; i++) {
	n1 = ni[i];
	n2 = nf[i];
	if ( r[i] == 0 ) return (1);
	g[n1][n1] += 1./r[i];
	g[n2][n2] += 1./r[i];
	g[n1][n2] -= 1./r[i];
	g[n2][n1] -= 1./r[i];
	is[n1] -= e[i]/r[i];
	is[n2] += e[i]/r[i];
	}

if (gauss_fp(n-1,g,is,v)) return (2);
v[n-1] = 0;
return (0);
}  /* end nodal */

/* ===========================================================
	 Procedura Gauss fara pivotare
	 Rezolva asistemul liniar a.x = b
	 Matricea a ca si vectorul b se distrug.
	 Intoarce cod de eroare				      */
int gauss_fp(int n, float a[][N], float b[], float x[])
{	/* n = dimens. matricii
	   a = matricea coeficientilor
	   b = vectorul termenilor liberi
	   x = vectorul rezultatelor              */


int i,j,k;
float s,p;

   /* Eliminare */
for (k=0; k<n-1; k++) { /* Contor linii pivot */
     if (a[k][k] == 0) return 1;  /* Eroare */
     for (i=k+1; i<n; i++) { /* Contor linii urmatoare */
	p=a[i][k]/a[k][k];
	for (j=k; j<n; j++)  /* Contor coloane */
		a[i][j] -= p*a[k][j];
	b[i] -= p*b[k];
	}
     }

   /* Retrosubstitutie */
if (a[n-1][n-1] == 0) return 1;  /* Eroare */
x[n-1] = b[n-1]/a[n-1][n-1];

for (i=n-2; i>=0; i--) {
     s = b[i];
     for (j=n-1; j>i; j--)
	s -= a[i][j]*x[j];
     x[i] = s/a[i][i];
     }

return 0;  /* all OK */
}	/* end gauss_fp  */

/* ============================================================
	Initializaeaza tabelul de date pentru interpolare   */

void init(int m)
/* m = numarul de puncte de fringere */
{
int k;
float err, xmin, xmax;

xmin = -1;
xmax = 1;
for (k=0; k<m; k++){
	x[k] = xmin + (xmax-xmin)*k/(m-1);
	y[k] = -1.e-9 * (1. - exp(x[k]/0.039));
	}
/* err = asezare(m-1,x,y);
   printf("Eroarea de aproximare a caracteristicii = %e\n",err);
   for (k=0; k<m; k++)
	y[k] = -1.e-9 * (1. - exp(x[k]/0.039));
*/
return;
}

/* ================================================================
	Interpoleaza caracteristica diodei si determina
	curentul si conducatnta dinamica, in functie de tensiunea aplicata */

void interp(int m,	/* numar de puncte de fringere */
	   float tens, 	/* tensiune */
	   float *crt,  /* curent */
	   float *cond) /* conductanta */

{
int k;	/* segment */

k = cauta_seg(m-1,x,tens);
*cond = (y[k+1] - y[k])/(x[k+1] - x[k]);
*crt = y[k] + *cond * (tens - x[k]);
return;
}

/* ============================================================*/
void main()
{
int   n, 	  /* numarul de noduri */
      l, lin,	  /* numarul de laturi */
      i,k,	  /* variabile contor */
      m;	  /* numar de puncte de fringere */
float v[N], dv[N];/* potentiale, variatia lor*/
int   ni[N], nf[N];/* noduri initial - final */
int n1, n2;
float e[L], ee[L], /* t.e.m. reale, echivalente */
      r[L], re[L], /* rezistente reale, echivalente */
      u, tens,     /* tensiunea */
      crt, intens; /* intensitatea */
float cond,	/* conductanta */
      pas;	/* norma corectiei */
char  t[L];        /* tipul elementului */
float pc,pg;       /* puteri consumata, generata */
float eps = 1e-6;	/* erorea admisibila */
int nit = 100;          /* numar maxim de iteratii */
int kit;		/* contor de iteratii */
clock_t first, second;
float timp;

highvideo();
do{
  clrscr();
  printf("\n\tPROGRAM PENTRU ANALIZA CIRCUITELOR REZISTIVE NELINIARE\n");
  printf(  "\t======================================================\n\n\n");

  printf("\nIntroduceti numarul de laturi (min.2, max.%d, altfel - iesire): ",L);
  scanf("%d",&l);
  if (l<2 || l>L) break;
  printf("\nIntroduceti numarul de noduri (min.2, max.%d) : ",N);
  scanf("%d",&n);
  if (n<2 || n>N) continue;

  clrscr();
  for (i=0; i<l; i++) {
	cprintf("\n\rLatura %d\n\r", i+1);
	printf("Nodul initial: ");
	scanf("%d",&ni[i]);
	ni[i]--;
	if (ni[i]>n-1 || ni[i]<0) {
		printf("\nNumar de nod incorect\n");
		i--;
		continue;
		}
	printf("Nodul final: ");
	scanf("%d",&nf[i]);
	nf[i]--;
	if (nf[i]>n-1 || nf[i]<0) {
		printf("\nNumar de nod incorect\n");
		i--;
		continue;
		}

	printf("Dioda (D=da, altfel fara dioda) :");
	t[i] = toupper(getch());
	if (t[i] == 'D')
		puts("Da");
	else
		puts("Nu");
	printf("Rezistenta: ");
	scanf("%f",&r[i]);
	if (r[i] == 0 && t[i] != 'D') {
		printf("\nNu se admit laturi cu rezistenta totala nula");
		i--;
		continue;
		}
	printf("Tensiunea electromotoare: ");
	scanf("%f",&e[i]);
	}
  lin = l;         /* numar initial de laturi */
  for (k=0; k<lin; k++)
	if( t[k] == 'D' && r[k] != 0.) {
		/* adauga nod suplimentar si extrage rezistenta in latura noua*/
		if (n-1 > N || l-1 > L) {
			printf("\nDepasire memorie");
			return;
			}
		ni[l] = n;
		nf[l] = nf[k];
		nf[k] = n;
		t[l] = 'N';
		r[l] = r[k];
		e[l] = 0;
		r[k] = 0;
		n++;
		l++;
		}

  do {
  for (i=0; i<n; i++)
	v[i] = 0;	/* initializeaza potentiale */
  clrscr();
  kit = 0;
  printf("\nNumarul de puncte ale carct. neliniare (min.2, max.%d, altfel iesire): ",M);
  scanf("%d",&m);
  if (m<2 || m>M) break;
  init(m);
    first = clock();  /* Gets system time */

  do {                  /* iteratii neliniare */
	kit++;
	/* Genereaza circuit liniarizat */
	for ( k = 0; k<l; k++) {
		tens = v[ni[k]] - v[nf[k]] + e[k];
		if ( t[k] == 'D' ) {
			interp(m,tens, &crt, &cond);
			if (cond == 0.)
				cond = 1.e-12;
			re[k] =  1/cond;
			ee[k] = crt/cond;
			}
		else    {
			re[k] = r[k];
			ee[k] = tens;
			}
		}
	/* Analizeaza circuitul liniar la iteratia curenata */
	nodal(n,l,ni,nf,re,ee,dv);

	/* Calculeaza norma pasului si actualizeaza potentialele */
	pas = 0;
	for ( k=0; k<n; k++) {
		v[k] += dv[k];
		if (fabs(dv[k])>pas)
			pas = fabs(dv[k]);
		}
	printf("\nIter = %d   Norma corectiei = %f",kit,pas);
  } while (pas>eps && kit<nit);
    second = clock(); /* Gets system time again */
  timp = (second-first)/CLK_TCK;
  printf("\n\n  Timpul de calcul: %e sec.",timp);
  printf("\n\nPentru continuare apasati o tasta :");
  getch();

  /* Afiseaza rezultate */
  pc = pg = 0;
  clrscr();
  printf("\n\tRezultatele analizei\n\t====================\n\n");
  for (i=0; i<l; i++)
    {
    n1 = ni[i];
    n2 = nf[i];
    u = v[n1] - v[n2];
    if (t[i] =='D') {
	tens = u + e[i];
	interp(m, tens, &crt, &cond);
	if (crt == 0.)
		crt = 1.e-12;
	r[i] = tens/crt;
	}
    intens = (u+e[i]) / r[i];
    if (i<lin)
	printf ("Latura: %d  \tI = %f  \n", i+1, intens);
    pc += r[i]*intens*intens;
    pg += e[i]*intens;
    }
  printf("\n\tPuterea consumata: %f\n",pc);
  printf("\tPuterea generata : %f\n",pg);


  printf("\nPentru continuare apasati o tasta :");
  getch();
  } while(1);
  } while(1);
lowvideo();

}  /* end main */