/* ===============================================================


	DEMOREZ - program pentru demonstratia convergentei
	iteratiilor la rezolvarea urmatoarelor ecuatii
	algebrice neliniare:

	e^x = a0
	x + a0.lnx = a1
	a0.sinx + a1.cosx = a2
	a0 + a1.x + a2.x^2 + a3.x^3 = 0

	prin metodele:

	- bisectiei;
	- iteratiei simple;
	- tangentelor paralele (Newton-Kantorovici);
	- tangentelor (Newton);
	- secantelor.

Intrare: coeficientii a0,a1,..
	intervalul xmin,xmax
Iesire : graficul functiei si punctele corespunzatoare iteratiilor

Programator: Daniel Ioan
Data ultimei modificari: 29.08.91

===============================================================*/
#define N 100			/* numar de puncte pentru grafic */
#include <conio.h>
#include <stdio.h>
#include <math.h>
#include "grafireu.c"

float a[4];	/* tabel coeficienti */
float xmin;     /* domeniul de definitie */
float xmax, x1, x2;
float alfa = 1;   /* factor de relaxare */
float xvv;

/* Evalueaza functia f(x) asociata ecuatiei f(x) =0
==========================================================*/
float f(int neq,	/* nr ecuatie */
	float x);       /* variabila independenta */

/* Evalueaza derivata functiei f(x) asociata ecuatiei f(x) =0
=============================================================*/
float fd(int neq,	/* nr ecuatie */
	float x);       /* variabila independenta */

/*  Realizeaza iteratii si afiseaza grafic rezultatul
=====================================================*/
void iter (int neq,	/* nr ecuatie */
	int nmet);      /* numarul metodei  */

/*  Realizeaza o iteratie si intoarce noua valoare a solutiei numerice
======================================================================*/
float fiter (float xvechi,     /* ultima valoare a solutiei */
		int neq,       /* nr ecuatie */
		int nmet);     /* numarul metodei  */

/* Repezinta grafic triunghiul xvechi, f(xvechi), xnou
========================================================*/
void tri(float xv,float xn,int neq);


void main()
{
int neq; 	/* numar ecuatie*/
int nmet;       /* numar metoda */
int i;

initial();	/* Initializare mod grafic */
restorecrtmode();

do{
  clrscr();
  printf("\n\n       PROGRAM  PENTRU REZOLVAREA ECUATIILOR ALGEBRICE NELINIARE\n");
  printf(    "       =========================================================\n");
  printf("\n         Programul rezolva iterativ ecuatiile:\n");
  printf("\n		1.	e^x = a0");
  printf("\n		2. 	x + a0.lnx = a1");
  printf("\n		3.	a0.sinx + a1.cosx = a2");
  printf("\n		4.	a0 + a1.x + a2.x^2 + a3.x^3 = 0");
  printf("\n\n       Selectati ecuatie (1-4, altfel iesire): ");
  scanf("%d",&neq);
  if (neq<1 || neq> 4) 	return;
  for (i=0;i<neq;i++){
	printf("\n      Introduceti parametrul a%d: ",i);
	scanf("%f",&a[i]);
	}
  printf("\n      Introduceti limitele domeniului:\n");
  printf("            xmin = ");
  scanf("%f",&xmin);
  printf("            xmax = ");
  scanf("%f",&xmax);
  if (xmax <= xmin) 	continue;
  do {
	clrscr();
	printf("\n\n       Metode utilizate:\n\n");
	printf("		1 - bisectiei;\n");
	printf("		2 - iteratiei simple;\n");
	printf("		3 - tangentelor paralele;\n");
	printf("		4 - tangentelor (Newton);\n");
	printf("		5 - secantelor.\n");
	printf("\n\n       Selectati metoda (1-5, altfel iesire): ");
	scanf("%d",&nmet);
	if (nmet<1 || nmet> 5) 	break;
	iter(neq,nmet);
	} while (1);
  } while(1);
}
/* Evalueaza functia f(x) asociata ecuatiei f(x) =0
=====================================================*/
float f(int neq,	/* nr ecuatie */
	float x)        /* variabila independenta */

{
	switch (neq)  {
		case 1: return (exp(x) - a[0]);
		case 2: return (x + a[0] * log(fabs(x)) - a[1]);
		case 3: return (a[0]*sin(x) + a[1]*cos(x) - a[2]);
		case 4: return (a[0] + a[1]*x + a[2]*x*x + a[3]*x*x*x);
	}
return (0);
}   /* end f */


/* Evalueaza derivata functiei f(x) asociata ecuatiei f(x) =0
=============================================================*/
float fd(int neq,	/* nr ecuatie */
	float x)        /* variabila independenta */

{
	switch (neq)  {
		case 1: return (exp(x));
		case 2: return (1 + a[0] * 1/(fabs(x)));
		case 3: return (a[0]*cos(x) - a[1]*sin(x));
		case 4: return (a[1] + 2*a[2]*x + 3*a[3]*x*x);
	}
return (0);
}   /* end fd */


/*  Realizeaza iteratii si afiseaza grafic rezultatul
=====================================================*/
void iter (int neq,	/* nr ecuatie */
	int nmet)       /* numarul metodei  */
{
int k = 1;		/* contor iteratii */
int nit = 20; 		/* numarul maxim de iteratii */
float err = 1e-3;	/* eroare impusa */
float xnou, xvechi;     /* valori iteratii */
float mat[3][N];        /* matrice puncte grafice */
char c;
int i;
float y, eps;

xvechi = xmin;
xvv = xmax;
x1 = xmin;
x2 = xmax;
for (i = 0; i<N; i++){
	mat[0][i] = x1 + (x2-x1)*i/(N-1);
	mat[1][i] = 0;
	mat[2][i] = f(neq,mat[0][i]);
	}

/* Reprezinta grafic y = f(x) */
hold = 0;
setgraphmode(getgraphmode());
redefinire(1, '.', '-', 'y');
redefinire(2, '.', '-', 'b');
plot(2,N,mat);
grid();
hold = 1;
if (nmet == 1){
	tri(xmin,xmin,neq);
	tri(xmax,xmax,neq);
}
getch();

if (nmet == 3)
	/* calculeaza derivata */
	alfa = -1/fd(neq,xvechi);
else
	alfa = -1;
do {
	k++;
	xnou = fiter(xvechi,neq,nmet);
	if (nmet == 1)
		tri(xnou,xnou,neq);
	else
		tri(xvechi,xnou,neq);

/*	y = f(neq,xvechi);
	printf("\niteratia %d  \tx = %g \t y = %g",k,xnou,y);
*/
	c = getch();
	if (c == 27) break;
	eps = fabs(f(neq,xnou));
	xvv = xvechi;
	xvechi = xnou;
   } while (eps>err && k <nit);
restorecrtmode();
} /* end iter */


/*  Realizeaza o iteratie si intoarce noua valoare a solutiei numerice
======================================================================*/
float fiter(float xvechi,     /* ultima valoare a solutiei */
		int neq,	/* nr ecuatie */
		int nmet)       /* numarul metodei  */
{
float xnou=0;	/* valorea intoarsa de functie */
switch (nmet){
	case 1: /* metida bisectiei */
		xnou = (x1 + x2)/2.;
		if (f(neq,xnou)*f(neq,x1)>0.)
			x1 = xnou;
		else
			x2 = xnou;
		break;
	case 2: /* metoda iteratiei simple */
	case 3: /* metoda tangentelor parlele */
		xnou = xvechi + alfa * f(neq,xvechi);
		break;
	case 4: /* metoda tangentelor*/
		xnou = xvechi - f(neq,xvechi)/fd(neq,xvechi);
		break;
	case 5: /* metoda secantelor*/
		xnou = xvechi - f(neq,xvechi)*(xvechi-xvv)/(f(neq,xvechi)-f(neq,xvv));
		break;
	}
return (xnou);
} /* end fiter */

/* Repezinta grafic triunghiul xvechi, f(xvechi), xnou
========================================================*/
void tri(float xv,float xn,int neq)
{
float y;
float mat[3][N];
	y = f(neq,xv);
	mat[0][0] = xv;
	mat[0][1] = xv;
	mat[0][2] = xn;
	mat[1][0] = 0;
	mat[1][1] = y;
	mat[1][2] = 0;
	redefinire(1, 'o', '-','r');
	plot(1,3,mat);
} /* end tri */


