/* ===============================================================


	NELAN - program pentru analiza metodelor iterative
	de rezolvare a ecuatiilor algebrice neliniare.
	Rezolva ecuatia:

	a0 + a1.x + a2.x^2 = 0

	prin metodele:

	- bisectiei;
	- iteratiei simple;
	- tangentelor paralele (Newton-Kantorovici);
	- tangentelor (Newton);
	- secantelor.

Intrare: coeficientii a0,a1,a2
	intervalul xmin,xmax
Iesire : graficul erorii in functiei de ordinul iteratiei

Programator: Daniel Ioan
Data ultimei modificari: 30.08.91

===============================================================*/
#define N 100			/* numar de puncte pentru grafic */
#include <conio.h>
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>
#include <dos.h>
#include "grafireu.c"

float a[3];	/* tabel coeficienti */
float xa;	/* solutia adevarata */
float xminim;     /* domeniul de definitie */
float xmaxim, x1, x2;
float xini;	/* initializare */
float alfa = 1;   /* factor de relaxare */
float xvv;

/* Evalueaza functia f(x) asociata ecuatiei f(x) =0
==========================================================*/
float f(float x);       /* variabila independenta */

/* Evalueaza derivata functiei f(x) asociata ecuatiei f(x) =0
=============================================================*/
float fd(float x);       /* variabila independenta */

/*  Realizeaza iteratii si afiseaza grafic rezultatul
=====================================================*/
void iter (int nmet);      /* numarul metodei  */

/*  Realizeaza o iteratie si intoarce noua valoare a solutiei numerice
======================================================================*/
float fiter (float xvechi,     /* ultima valoare a solutiei */
		int nmet);     /* numarul metodei  */



void main()
{
int nmet;       /* numar metoda */
float delta,xa1,xa2; /* determinant si solutii exacte */
int nsol;	      /* numar de solutii exacte in intervalul xmin-xmax */

initial();	/* Initializare mod grafic */
restorecrtmode();

do{
do{
  clrscr();
  printf("\n\n   PROGRAM  PENTRU ANALIZA METODELOR DE REZOLVARE A ECUATIILOR NELINIARE\n");
  printf(    "   =====================================================================\n");
  printf("\n         Programul rezolva iterativ ecuatia  a.x^2 + b.x + c = 0:\n");
  printf("\n      Introduceti coeficientii: \n");
  printf("           a =  ");
  scanf("%f",&a[2]);
  printf("           b =  ");
  scanf("%f",&a[1]);
  printf("           c =  ");
  scanf("%f",&a[3]);
  printf("\n      Introduceti limitele domeniului:\n");
  printf("            xmin = ");
  scanf("%f",&xminim);
  printf("            xmax = ");
  scanf("%f",&xmaxim);
  if (xmaxim <= xminim) 	continue;
  printf("\n      Introduceti initializarea x0 :");
  scanf("%f",&xini);

/* Determina solutia exacta */
delta = a[1]*a[1] - 4.* a[2]*a[0];
if (delta <0){
	printf("\n    Problema nu are solutie !!\n");
	getch();
	continue;
}
xa1 = (-a[1] - sqrt(delta))/(2.* a[2]);
xa2 = (-a[1] + sqrt(delta))/(2.* a[2]);
nsol = 0;
if (xminim<=xa1 && xa1<=xmaxim){
	nsol++;
	xa = xa1;
}
if (xminim<=xa2 && xa2<=xmaxim){
	nsol++;
	xa = xa2;
}
if (nsol == 0 ){
	printf("\n    Problema nu are solutie !!\n");
	getch();
	continue;
}
if (nsol == 2 && xa1 != xa2){
	printf("\n    Problema nu are solutie unica !!\n");
	getch();
	continue;
}
break;
} while(1);

  do {
	clrscr();
	printf("\n\n       Metode utilizate:\n\n");
	printf("		1 - bisectiei;\n");
	printf("		2 - iteratiei simple;\n");
	printf("		3 - tangentelor paralele;\n");
	printf("		4 - tangentelor (Newton);\n");
	printf("		5 - secantelor.\n");
	printf("\n\n       Selectati metoda (1-5, altfel iesire): ");
	scanf("%d",&nmet);
	if (nmet<1 || nmet> 5) 	break;
	iter(nmet);
	} while (1);

    printf("\n      Pentru incheiere apasati <ESC>, altfel se reia executia  ");
	if (getch() == 27) return;

  } while(1);
}
/* Evalueaza functia f(x) asociata ecuatiei f(x) =0
=====================================================*/
float f(float x)        /* variabila independenta */

{
delay(10);
return (a[0] + a[1]*x + a[2]*x*x );
}   /* end f */


/* Evalueaza derivata functiei f(x) asociata ecuatiei f(x) =0
=============================================================*/
float fd(float x)        /* variabila independenta */

{
delay(10);
return (a[1] + 2*a[2]*x);
}   /* end fd */


/*  Realizeaza iteratii si afiseaza rezultatul
==============================================*/
void iter (int nmet)       /* numarul metodei  */
{
int k = 0;		/* contor iteratii */
int nit = N-2; 		/* numarul maxim de iteratii */
int n;			/* numar efectiv de iteratii */
float err = 1e-20;	/* eroare impusa */
float eps0 = 1e-20;
float xnou, xvechi;     /* valori iteratii */
float mat[4][N];        /* matrice puncte grafice */
float t, y, eps;
clock_t first, second;

xvechi = xini;
xvv = xmaxim;
x1 = xminim;
x2 = xmaxim;
if (nmet == 3)
	/* calculeaza derivata */
	alfa = -1/(fd(xvechi)+eps0);
else
	alfa = -1;
for ( k = 0; k<N; k++) {	/* Initializeaza tabele grafic*/
		mat[0][k] = k+1;
	}

clrscr();
printf("\n Iter. \tSolutia exacta \tSolutia num. \tEroarea \tCorectia \tReziduu ");

first = clock();  /* Gets system time */

k = 0;
do {
	xnou = fiter(xvechi,nmet);
	y = f(xnou);
	eps = fabs(xnou - xvechi);
	xvv = xvechi;
	xvechi = xnou;
	mat[1][k] = fabs(xa-xnou);
	mat[2][k] = eps;
	mat[3][k] = y;
	if ( (0<k && k<=10) || (k%(nit/10)) == 0 )
		printf("\n %d  \t%10g  \t%10g        %8f       %8f     %8f",k,xa,xnou,mat[1][k],eps,y);
	k++;
   } while (eps>err && k < nit);
   second = clock(); /* Gets system time again */
   t = (second-first)/CLK_TCK/k;
   printf("\n\n  Timpul de calcul: %e sec/iter. Pentru continuare apasati o tasta",t);
   getch();
   n = k; /* numar efectiv de iteratii */

   /* Reprezinta grafic variatia erorii si a corectiei */
   setgraphmode(getgraphmode());
   for (k = 0; k<n; k++) {
	if (mat[0][k] < eps0) mat[0][k] = eps0;
	if (mat[1][k] < eps0) mat[1][k] = eps0;
	if (mat[2][k] < eps0) mat[2][k] = eps0;
	if (mat[3][k] < eps0) mat[3][k] = eps0;
	}
   grafic(3,n-1,mat,1,1);
   grid();
   label_x("Numar de iteratii");
   title("Eroarea, corectia si reziduul");
   getch();
   restorecrtmode();
} /* end iter */

/*  Realizeaza o iteratie si intoarce noua valoare a solutiei numerice
======================================================================*/
float fiter(float xvechi,     /* ultima valoare a solutiei */
		int nmet)       /* numarul metodei  */
{
float xnou=0;	/* valorea intoarsa de functie */
switch (nmet){
	case 1: /* metida bisectiei */
		xnou = (x1 + x2)/2.;
		if (f(xnou)*f(x1)>0.)
			x1 = xnou;
		else
			x2 = xnou;
		break;
	case 2: /* metoda iteratiei simple */
	case 3: /* metoda tangentelor parlele */
		xnou = xvechi + alfa * f(xvechi);
		break;
	case 4: /* metoda tangentelor*/
		xnou = xvechi - f(xvechi)/(fd(xvechi)+1e-20);
		break;
	case 5: /* metoda secantelor*/
		xnou = xvechi - f(xvechi)*(xvechi-xvv)/(f(xvechi)-f(xvv)+1e-20);
		break;
	}
return (xnou);
} /* end fiter */
