/* =====================================================================

			D E M O T R N
Program demonstrativ pentru analiza regimului tranziotoriu al circuitelor
neliniare. Analizeaza circuite cu rezistente, bobine, condensatoare liniare,
surse de tensiune (constante sau sinusoidale) si diode semiconductoare.
Metoda folosita este caracterizata prin:
	- metoda circuitului discretizat, generat de metoda Euler impliciata
	- aproximarea liniara pe portiuni a caracteristicii diodei;
	- metoda circuitului liniarizat (Newton);
	- rezolvarea circuitului liniar prin tehnica nodala.


   Date de intrare:
	- numar de noduri;
	- numar de laturi;
	- pentru fiecare latura:
		- nodul initial;
		- nodul final;
		- tipul elementului pasiv (R,L,C,D);
		- parametrul elementului (R,L,C);
		- t.e.m. (amplitudine, frecventa, faza initiala);
	- numarul de segmente folosite in aproximarea liniara pe portiuni a
	  diodei;
	- timpul maxim de simulare;
	- semnalele de iesire (u, i);

   Date de iesire:
	- variatia in timp a semanlelor de iesire

Data : 21.09.1991
Programator: Daniel Ioan
========================================================================== */

#define N 50
#define M 50
#define L 100
#define T 50
#include <stdio.h>
#include <conio.h>
#include <math.h>
#include <time.h>
#include <dos.h>
#include <ctype.h>
#include "pp_pack.c"
#include "grafic.c"
float x[M], y[M];     /* Caracteristica diodei */
float pi;

/* ===========================================================
	 Procedura Gauss fara pivotare
	 Rezolva asistemul liniar a.x = b
	 Matricea a ca si vectorul b se distrug.
	 Intoarce cod de eroare				      */
int gauss_fp(int n, float a[][N], float b[], float x[])
{	/* n = dimens. matricii
	   a = matricea coeficientilor
	   b = vectorul termenilor liberi
	   x = vectorul rezultatelor              */


int i,j,k;
float s,p;

   /* Eliminare */
for (k=0; k<n-1; k++) { /* Contor linii pivot */
     if (a[k][k] == 0) return 1;  /* Eroare */
     for (i=k+1; i<n; i++) { /* Contor linii urmatoare */
	p=a[i][k]/a[k][k];
	for (j=k; j<n; j++)  /* Contor coloane */
		a[i][j] -= p*a[k][j];
	b[i] -= p*b[k];
	}
     }

   /* Retrosubstitutie */
if (a[n-1][n-1] == 0) return 1;  /* Eroare */
x[n-1] = b[n-1]/a[n-1][n-1];

for (i=n-2; i>=0; i--) {
     s = b[i];
     for (j=n-1; j>i; j--)
	s -= a[i][j]*x[j];
     x[i] = s/a[i][i];
     }

return 0;  /* all OK */
}	/* end gauss_fp  */

/* =====================================================
   Analiza circuitelor rezistive liniare cu metoda nodala
   Intoarce cod de eroare */
int nodal(int n,         /* nr noduri */
	  int l,	 /* nr laturi*/
	  int ni[],      /* tabelul nodurilo initale*/
	  int nf[],	 /* tabelul nodurilur finale */
	  float r[],	 /* tabelul rezistentlor */
	  float e[],	 /* tabelul t.e.m. */
	  float v[])     /* Rezultate: potentialele nodurilor */

{
int i, j, n1, n2;
float is[N],  		/* termenii liberi */
      g[N][N];          /* matricea sistemului */

for (i=0; i<n; i++) {
	is[i] = 0;
	for (j=0; j<n; j++)
		g[i][j] = 0;
	}
for (i=0; i<l; i++) {
	n1 = ni[i];
	n2 = nf[i];
	if ( r[i] == 0 ) return (1);
	g[n1][n1] += 1./r[i];
	g[n2][n2] += 1./r[i];
	g[n1][n2] -= 1./r[i];
	g[n2][n1] -= 1./r[i];
	is[n1] -= e[i]/r[i];
	is[n2] += e[i]/r[i];
	}

if (gauss_fp(n-1,g,is,v)) return (2);
v[n-1] = 0;
return (0);
}  /* end nodal */


/* ============================================================
	Initializaeaza tabelul de date pentru interpolare   */

void init(int m)
/* m = numarul de puncte de fringere */
{
int k;
float err, xmin, xmax;

xmin = -1;
xmax = 1;
for (k=0; k<m; k++){
	x[k] = xmin + (xmax-xmin)*k/(m-1);
	y[k] = -1.e-9 * (1. - exp(x[k]/0.039));
	}
/* err = asezare(m-1,x,y);
   printf("Eroarea de aproximare a caracteristicii = %e\n",err);
   for (k=0; k<m; k++)
	y[k] = -1.e-9 * (1. - exp(x[k]/0.039));
*/
return;
}

/* ================================================================
	Interpoleaza caracteristica diodei si determina
	curentul si conducatnta dinamica, in functie de tensiunea aplicata */

void interp(int m,	/* numar de puncte de fringere */
	   float tens, 	/* tensiune */
	   float *crt,  /* curent */
	   float *cond) /* conductanta */

{
int k;	/* segment */

k = cauta_seg(m-1,x,tens);
*cond = (y[k+1] - y[k])/(x[k+1] - x[k]);
*crt = y[k] + *cond * (tens - x[k]);
return;
}

/* ============================================================*/
void main()
{
int   n, 	  /* numarul de noduri */
      l, 	  /* numarul de laturi */
      j,i,k,	  /* variabile contor */
      m;	  /* numar de puncte de fringere */
float v[N], dv[N];/* potentiale, variatia lor*/
int   ni[N], nf[N];/* noduri initiale - finale */
float tem[L],e[L], ee[L], /* t.e.m. reale, echivalente */
      f[L], faza[L],      /* frecventele si fazele initiale ale tem */
      p[L],r[L], re[L],   /* parametri ((RLC), rezistente reale, echivalente */
      c[L],		  /* conditii initiale */
      u, tens,     /* tensiunea pe elementul pasiv, pe latura curenta */
      crt;         /* intensitatea din latura curenta */
float cond,	   /* conductanta elementului neliniar*/
      pas;	   /* norma corectiei */
char  t[L];        /* tipul elementului */
float eps = 1e-6;	/* erorea admisibila */
int nt, nit = M;        /* numarul pasilor de timp, numar maxim de iteratii */
int kit;		/* contor de iteratii */
float tmax, h, timp;    /* timp maxim, pas de timp, timp curent */
int li,lu;		/* laturi cu semnale de iesire */
float mat[3][T];	/* semnale de iesire */
pi = 4.*atan(1.);

initial();	/* Initializare mod grafic */
restorecrtmode();
highvideo();
do{
  clrscr();
  printf("\n   PROGRAM PENTRU ANALIZA REGIMULUI TRANZITORIU A CIRCUITELOR NELINIARE\n");
  printf(  "\  ====================================================================\n\n\n");

  printf("\nIntroduceti numarul de laturi (min.2, max.%d, altfel - iesire): ",L);
  scanf("%d",&l);
  if (l<2 || l>L) break;
  printf("\nIntroduceti numarul de noduri (min.2, max.%d) : ",N);
  scanf("%d",&n);
  if (n<2 || n>N) continue;

  clrscr();
  for (i=0; i<l; i++) {
	cprintf("\n\rLatura %d\n\r", i+1);
	printf("Nodul initial: ");
	scanf("%d",&ni[i]);
	ni[i]--;
	if (ni[i]>n-1 || ni[i]<0) {
		printf("\nNumar de nod incorect\n");
		i--;
		continue;
		}
	printf("Nodul final: ");
	scanf("%d",&nf[i]);
	nf[i]--;
	if (nf[i]>n-1 || nf[i]<0) {
		printf("\nNumar de nod incorect\n");
		i--;
		continue;
		}

	printf("Tipul elementului (L,C,D altfel R) :");
	t[i] = toupper(getch());
	switch (t[i]) {
		case 'L': puts("Bobiana");
			  printf("Inductivitatea [H]: ");
			  scanf("%f",&p[i]);
			  if (p[i] == 0 ) {
				printf("\nNu se admit laturi cu inductivitate nula");
				i--;
				continue;
				}
			  printf("Curentul initial [A]: ");
			  scanf("%f",&c[i]);
			  break;
		case 'C': puts("Condensator");
			  printf("Capacitatea : ");
			  scanf("%f",&p[i]);
			  if (p[i] == 0 ) {
				printf("\nNu se admit laturi cu capacitatea nula");
				i--;
				continue;
				}
			  printf("Tensiunea initiala [V]: ");
			  scanf("%f",&c[i]);
			  break;
		case 'D': puts("Dioda");
			  break;
		default:  t[i] = 'R';
			  puts("Rezistor");
			  printf("Rezistenta [Ohmi]: ");
			  scanf("%f",&p[i]);
			  if (p[i] == 0 ) {
				printf("\nNu se admit laturi cu rezistenta nula");
				i--;
				continue;
			   }
		}
	printf("Tensiunea electromotoare [V]: ");
	scanf("%f",&e[i]);
	if (e[i] == 0) {
		f[i] = 0;
		faza[i] = 0;
		}
	else {
		printf("Frecventa [Hz] (0 pentru c.c.): ");
		scanf("%f",&f[i]);
		if (f[i] == 0 )
			faza[i] = 0;
		else {
			printf("Faza initiala [grade]: ");
			scanf("%f",&faza[i]);
			faza[i] = faza[i] * pi / 180.;
			}
		}
	}

  clrscr();
  printf("\nTimpul maxim de simulare [s]: ");
  scanf("%f",&tmax);
  nt = T; /* Numarul de pasi de integrare */
  h = tmax / nt;

/*  printf("\nNumarul de puncte ale carct. neliniare (min.2, max.%d, altfel iesire): ",M);
  scanf("%d",&m);
  if (m<2 || m>M) break;
*/m =20;
  init(m);

  printf("\n\nNumarul laturii - curent de iesire : ");
  scanf("%d",&li);
  if (li<1 || li>l)
	li = -1;
  else
	li--;
  printf("\n\nNumarul laturii - tensiune de iesire : ");
  scanf("%d",&lu);
  if (lu<1 || lu>l)
	lu = -1;
  else
	lu--;


  for (i=0; i<n; i++)
	v[i] = 0;	/* initializeaza potentiale */
  clrscr();
  for ( i = 0; i<l; i++) { /* echivaleaza elementele reactive */
	switch (t[i]) {
		case 'L': r[i] = p[i]/h;
			  break;
		case 'C': r[i] = h/p[i];
			  break;
		case 'D': r[i] = 1.;
			  break;
		default:  r[i] = p[i];
		}
	}
  for (j = 1; j<=nt; j++) { /* parcurge pasii de timp */

	timp = j * h;
	mat[0][j-1] = timp;
	for ( i = 0; i<l; i++) { /* construieste circuit discretizat */
		if (f[i] == 0)
			tem[i] = e[i];
		else             /* evalueaza t.e.m. */
			tem[i] = e[i] * sqrt(2.) * sin(2.*pi*f[i]*timp + faza[i]);
		if (t[i] == 'L')  /* echivaleaza conditii initiale */
			tem[i] += r[i]*c[i] ;
		else if (t[i] == 'C')
			tem[i] -=  c[i];
		}
	kit = 0;
	do {                  /* iteratii neliniare */
		kit++;
		/* Genereaza circuit liniarizat */
		for ( i = 0; i<l; i++) {
			tens = v[ni[i]] - v[nf[i]] + tem[i];
			if ( t[i] == 'D' ) {
				interp(m,tens, &crt, &cond);
				if (cond == 0.)
					cond = 1.e-12;
				re[i] =  1/cond;
				ee[i] = crt/cond;
				}
			else    {
				re[i] = r[i];
				ee[i] = tens;
				}
			}
		/* Analizeaza circuitul liniar la iteratia curenata */
		nodal(n,l,ni,nf,re,ee,dv);

		/* Calculeaza norma pasului si actualizeaza potentialele */
		pas = 0;
		for ( k=0; k<n; k++) {
			v[k] += dv[k];
			if (fabs(dv[k])>pas)
				pas = fabs(dv[k]);
			}
/*		printf("\nIter = %d   Norma corectiei = %f",kit,pas);
 */
		} while (pas>eps && kit<nit); /* sfirsit iteratria neliniara */

		/* Afiseaza si salveaza semnale de iesire */
		/* Actualizeaza conditii initiale */
		for (i=0; i<l; i++) {
			u = v[ni[i]] - v[nf[i]];
			tens = u + tem[i];
			if (t[i] =='D')
				interp(m, tens, &crt, &cond);
			else
				crt = tens / r[i];

			if (i == li)
				mat[1][j-1] = crt;
			else if (i == lu)
				mat[2][j-1] = u;

			if (t[i] == 'L')
				c[i] = crt;
			else if(t[i] == 'C')
				c[i] += r[i]*crt;
			}
		if (j%22 == 0 ){
			printf("\n\nPentru continuare apasati o tasta :");
			getch();
			clrscr();
			}
		printf("\n t = %f   (%d iteratii)",timp,kit);
		if (li >=0 )
			printf("    i[%d] = %f",li+1,mat[1][j-1]);
		if (lu >=0 )
			printf("    u[%d] = %f",lu+1,mat[2][j-1]);
	} /* ciclul pasilor de timp */
  printf("\n\nPentru continuare apasati o tasta :");
  getch();
  setgraphmode(getgraphmode());
  plot(2,nt-1,mat);
  grid();
  label_x("Timp [s]");
  title("Semnale de iesire: i,u");
  getch();
  restorecrtmode();
  clrscr();

  } while(1);
lowvideo();

}  /* end main */