	/*
		[]----------------------------------------------------------[]
		|	 Nume program : PRED0                			 |
		| - Rezolva ecuatii diferentiale ordinare  			 |
		| printr-o metoda predictor-corector bazata pe o metoda 	 |
		| multipas;       							 |
		| - Predictorul se face cu metoda Adams-Bashforth de ordin 3;|
		| - Corectorul se face cu metoda Adams-Moulton de ordin 3;	 |
		| - Traseaza graficul functiei;   			             |
		| - Afiseaza solutiile exacte,aproximative si eroarea pe pas.|
		[]----------------------------------------------------------[]  */

#include <math.h>
#include <stdlib.h>
#include <conio.h>
#include "c:\mn\rezdif\euler\grafeu.cp"

# define	NMAX	400
# define    NEC	1
# define	MAX	1e6
# define 	D	double
# define	P	3			/* ordinul metodei */
# define	MAXIT 3			/* nr. maxim de iteratii */

D	a[NEC][NEC],			/* coeficientii sistemului */
	b[NEC],				/* termenii liberi */
	h,                            /* pasul de integrare */
	c[NEC],				/* conditile initiala */
	bp[6][7] = {{0,0,0,0,0,0,0},
			{0,1.,0,0,0,0,0},
			{0,3./2,-1./2,0,0,0,0},
			{0,23./12.,-16./12.,5./12.,0,0,0},
			{0,55./24,-59./24,37./24,-9./24,0,0},
			{0,1901./720,-2774./720,2616/720,-1274/720,251./5,0}},
	bc[5][6] = {{1.,0,0,0,0,0},
			{.5,.5,0,0,0,0},
			{5./12,8./12,-1./12,0,0,0},
			{9./24.,19./24.,-5./24.,1./24.,0,0},
			{251./720,646./720,-264./720,106./720,-19./720,0}},
	tmax;					/* limita de integrare */
unsigned	iter,				/* nr. iteratii */
		p;				/* ordinul metodei */
extern float val[M + 1][N];		/* matricea solutiilor */

void graf(int n)
{

	initial();
	subplot(1,1,1);
	plot(p,n,val);
	grid();
	restore();
	return;
}

void write_t(float t)
{
	gotoxy (10,24);
	cprintf("t = %4.1f",t);
	return;
}

D func (unsigned i,D *y)
{
	D	fn = 0.;
	unsigned	j;

	for ( j=0; j<=NEC-1; j++)
		fn = fn + a[i][j]*y[j];
	fn = fn + b[i];
	return (fn);
}

void runge_4 (D *y)    			/* metoda Runge Kutta, ordin 4*/
{
	D	g[NEC],s[NEC],z[NEC];
	unsigned			 i;

	for ( i=0; i<=NEC-1; i++) g[i] = func(i,y); /* calcul g1 */
	for ( i=0; i<=NEC-1; i++)
	{
		z[i] = y[i] + h*g[i]/2.;			/* argument g2 */
		s[i] = g[i];
	}
	for ( i=0; i<=NEC-1; i++)
			g[i] = func(i,z);				/* calcul g2 */
	for ( i=0; i<=NEC-1; i++)
	{
		z[i] = y[i] + h*g[i]/2.;			/* argument g3 */
		s[i] = s[i] + 2.*g[i];
	}
	for ( i=0; i<=NEC-1; i++)
			g[i] = func(i,z);				/* calcul g3 */
	for ( i=0; i<=NEC-1; i++)
	{
		z[i] = y[i] + h*g[i];			/* argument g4 */
		s[i] = s[i] + 2.*g[i];
	}
	for ( i=0; i<=NEC-1; i++)
			g[i] = func(i,z);				/* calcul g4 */
	for ( i=0; i<=NEC-1; i++)
		y[i+NEC] = y[i] + h*(s[i]+g[i])/6.;	/* solutie finala */
	i=1;
	return ;
}

int magn_test (D *val,unsigned nr,unsigned *n,unsigned *i)
{
	unsigned	max,j,ctrl = 0;

	max = nr - 1;
	for ( j=0; j<=max; j++)
		if ( fabs(val[j])>(D)MAX )
		{ val[j] = 0.;
		  ctrl = 1;
		}
	if ( ctrl )
		{
			 *n = *i - 2;
			 return 1;
		}
	return 0;
}

int runge (D *y,D *f)			          /* rezolvare cu Runge Kutta 4 */
{
	unsigned	j,i;
	D	t = 0;			          /* nod initial */

	for ( i=0; i<=NEC-1; i++)
		y[i] = c[i];                       /* conditii initiale */
	for ( i =1; i<=p-1; i++ )                /* calcul */
	{
		if ( magn_test(&y[(i-1)*NEC],NEC,&i,&i) ) return 1;
		runge_4 ( &y[(i-1)*NEC] );  	     /* calcul Runge Kutta 4 */
		t =t + h;                          /* nod nou */
	}
	for( i=1; i<=p; i++)                      /* calcul derivate */
		for( j=0; j<=NEC-1; j++)
			f[i*NEC+j] = func( j,&y[(i-1)*NEC] );
	return 0;
}

void predictie (D *y,D *s,D *f)
{
	unsigned	i,j;

	for( i=0; i<=NEC-1; i++)
	{
		s[i] = y[i];
		y[i+NEC] = y[i];
		for( j=1; j<=p; j++)
		{
			y[i+NEC] = y[i+NEC] + h*bp[p][j]*f[NEC*j+i]; /* valoare predictor */
			s[i] = s[i] + h*bc[p-1][j]*f[NEC*j+i];
		}
	}
	return;
}

void evaluare (D *y,D *f)
{
	unsigned	i;

	for( i=0; i<=NEC-1; i++)
		f[i] = func (i,y);
	return;
}

D corectie(D *y,D *s,D *f)
{
	D	oldy,eps = 0.;
	unsigned	i;

	for( i=0; i<=NEC-1; i++)
	{
		oldy = y[i];
		y[i] = s[i] + h*bc[p-1][0]*f[i];
		if (oldy!=0) eps = eps + fabs( (y[i]-oldy)/oldy );
	}
	return (eps);
}

void translatie (D *f)
{
	unsigned	i,j;

	for( i=0; i<=NEC-1; i++)
		for( j=p; j>=1; j--)
			f[j*NEC+i] = f[(j-1)*NEC+i];
	return;
}


void solve (void)			   /* rezolvare */
{
	unsigned	i,n;
	D	t = 0;			      /* nod initial */
	D	y[NMAX][NEC],			/* vector solutii */
		f[10][NEC],                  /* vector derivate */
		s[NEC];                       /* vector auxiliat */
	int	overflow;				/* indicato depasire */
	D	eps,epsmax = 0.001;

 for (p=1;p<=4;p++)
 {
	overflow = runge (y[0],f[0]);			/* primele p puncte */
	if ( !overflow )
	{
		t = p*h;
		n = (int)((tmax-t)/h + .5);			/* nr. puncte */
		for ( i=p; i<=n+p; i++)
		{
			predictie (y[i-1],s,f[0]);			/* predictie */
			evaluare (y[i],f[0]);			/* evaluare */
			iter = 0;
			do
			{
				eps = corectie (y[i],s,f[0]);	/* corectie */
				iter ++;
			} while ( fabs(eps)>epsmax && iter<=MAXIT );
			translatie (f[0]);				/* vector derivate nou */
		 }
	}
	else
		n = 0;
	for (i=0;i<=n+p;i++)
			val[p][i] = (float)y[i][NEC-1];
 }
	for (i=0;i<=n+p;i++)
		val[0][i] = (float)(i*h);
	graf (n+p);
	return;
}

int next(void)
{
	char	c;

	clrscr();
	gotoxy(5,10);
	cputs("Alta ecuatie ?[D/N]");
	c = getch();
	if ( c=='d' || c=='D' ) return 1;
	return 0;
}

void mat_input (char msg[25],char dat,D *v,int k)
{
	unsigned	j;
	char	  c[20];

	gotoxy (10,18);
	clreol();
	cputs (msg);
	for ( j=0; j<=NEC-1; j++)
	{
		gotoxy (10,20);
		clreol();
		if ( k<0 ) cprintf ("%c%1d = ",dat,j+1);
		else       cprintf ("%c%1d%1d = ",dat,k,j+1);
		c[0] = 16;
		v[j] = atof ( cgets(c) );
	}
	return;
}

void input (void) 		/* se introduc parametrii */
{
	char	s[20];
	int	i;

	s[0] = 16;
	clrscr();
	textcolor(LIGHTCYAN);
	cputs("\n\r  Programul rezolva un sistem liniar de ecuatii diferentiale de forma : \n\r\n\r");
	cputs("            /    \\     /       \\     /  \\     /  \\ \n\r");
	cputs("           |  y1' |   | a11 a12 |   | y1 |   | b1 |\n\r");
	cputs("           |  y2' | = | a21 a22 | * | y2 | + | b2 |\n\r");
	cputs("            \\    /     \\       /     \\  /     \\   /");
	cputs("\n\r\n\r  Domeniul de integrare : [0,tmax] [s]");
	cputs("\n\r  Pasul de integrare  h : tmax/400 [s]");
	cputs("\n\r  Conditii initiale : y1(0) = C1, y2(0) = C2 ");
	textcolor(YELLOW);
	gotoxy (10,16);
	cputs("tmax [in secunde]   : ");
	tmax = fabs( atof(cgets(s)) );
	for ( i=0; i<=NEC-1; i++)
		mat_input ("Coeficienti :",'a',a[i],i+1);
	mat_input ("Termeni liberi",'b',b,-1);
	mat_input ("Conditii initiale",'C',c,-1);
	if ( tmax==0. ) tmax = 1.;
	h = tmax/30.;
	textcolor(LIGHTCYAN);
	return;
}

main()
{
	do
	{
		input();
		solve();
	}while( next() );
	return;
}

