	/*
	[]-------------------------------------------------------[]
	|			 Nume program : RUNGE1                  |
	|  - Rezolva ecuatia : dy/dt = -y prin metodele Runge     |
	| Kutta si Euler explicita pentru pasi de integrare       |
	| definiti de utilizator;                                 |
	| - Intervalul de integrare este [0,tmax];		    |
      | - Conditia initiala este y(0) = 1;			    |
	| - Se afiseaza pentru cele 2 metode erorile pentru       |
	| primul pas si erorile globale;				    |
	| - Se afiseaza pentru cele 2 metode timpii de calcul;    |
	[]-------------------------------------------------------[]  */

# include <conio.h>
# include <math.h>
# include <stdlib.h>

# define 	D	double

struct time
{
	unsigned char ti_min;
	unsigned char ti_hour;
	unsigned char ti_hund;
	unsigned char ti_sec;
};

D	a = -1.,b = 0.,				/* coeficientii ecuatiei */
	h,						/* pasul de integrare */
	y0 = 1.,					/* conditia initiala */
	tmax = 4.;					/* limita de integrare */
int	iter;						/* nr. iteratii */

void gettime ( struct time *timep );

D funcan(D t)
{
	D	f;

	if (a!=0) f = ((a*y0+b)*exp(a*t) - b)/a;
	else f = b*t + y0;
	return(f);
}

D euler_exp (D y,D h,D a,D b)
{
	return ( y + h*(a*y + b) );
}

D f(D y)
{
	return (a*y + b);
}

D runge_4 (D y)
{
	D 	g1,g2,g3,g4,newy;

	g1 = f(y);
	g2 = f(y+h*g1/2);
	g3 = f(y+h*g2/2);
	g4 = f(y+h*g3);
	newy = y + h*( g1 + 2*g2 + 2*g3 + g4 )/6;
	return (newy);
}

void waitkey(int x,int y)
{
	gotoxy(6,24);
	cputs("APASA O TASTA...");
	getch();
	gotoxy(x,y);
	return;
}

void calc_err (char tip,D ye,D t,float deltat,long k)
{
	int	i=1;
	D	eps;

	eps = funcan (t) - ye;
	if (k==1L)
		if (tip=='e')
		{
			gotoxy (56,i+7);
			cprintf ("% 10.4e",eps);
		}
		else
		{
			gotoxy (56,i+15);
			cprintf ("% 10.4e",eps);
		}
	else
		if (tip=='e')
		{
			gotoxy (56,i+9);
			cprintf ("% 10.4e",eps);
			gotoxy (56,i+11);
			cprintf ("%7.3f secunde.",deltat);
		}
		else
		{
			gotoxy (56,i+17);
			cprintf ("% 10.4e",eps);
			gotoxy (56,i+19);
			cprintf ("%7.3f secunde.",deltat);
			gotoxy (1,1);clreol();
			waitkey (1,1);
		}
	return;
}

void write_t(D t,char *s)
{
	gotoxy (45,1);
	cprintf("%st = %3.1f",s,t);
	return;
}

void solve (void)
{
	unsigned long stp,i,n;
	D	ye,yr,t = 0;
	struct time	ti,tf;
	float	     deltat;
	char	se[20] = " Metoda Euler - ",
		sr[20] = "Runge Kutta 4 - ";

	ye = y0;                             	   /* conditii initiale */
	yr = y0;
	n = (long)(tmax/h+.5);                       /* nr. maxim de pasi */
	if ( h>=0.1 ) stp = 5;				   /* pas de afisare timp */
	else  stp = 100;
	gettime (&ti);					   /* moment incepere calcul */
	for ( i = 1; i <= n; i++ )                   /* calcul  Euler */
	{
		t =t + h;                              /* nod nou */
		ye = euler_exp (ye,h,a,b);  		   /* calcul Euler - explicit */
		if ( i%stp==0 ) write_t (t,se);
		if (i==1L) calc_err ('e',ye,t,0,i);	   /* calcul eroare pas 1 */
	}
	gettime (&tf);				  	   /* moment sfirsit calcul */
	i++;
	deltat = 60*(tf.ti_min - ti.ti_min) +
			 tf.ti_sec - ti.ti_sec +
			(float)(tf.ti_hund - ti.ti_hund)/100.;
	if (deltat < 1e-3) deltat = 0.001;
	calc_err ('e',yr,t,deltat,i);			   /* calcul eroare globala */
	t = 0;
	gettime (&ti);					   /* moment incepere calcul */
	for ( i = 1; i <= n; i++ )                   /* calcul Runge Kutta */
	{
		t =t + h;                              /* nod nou */
		yr = runge_4 (yr);       		   /* calcul Runge Kutta 4 */
		if ( i%stp==0 ) write_t (t,sr);
		if (i==1L) calc_err ('r',yr,t,0,i);	   /* calcul eroare pas 1 */
	}
	gettime (&tf);				  	   /* moment sfirsit calcul */
	i++;
	deltat = 60*(tf.ti_min - ti.ti_min) +
			 tf.ti_sec - ti.ti_sec +
			(float)(tf.ti_hund - ti.ti_hund)/100.;
	if (deltat < 1e-3) deltat = 0.001;
	calc_err ('r',yr,t,deltat,i);			   /* calcul eroare globala */
	return;
}


void display (void)
{
	int	i=1;

	clrscr();
	gotoxy (4,i+1);
	cprintf ("Pasul de integrare h : %8.6f\n\n\r   ",h);
	textcolor (WHITE);
	cputs ("Erori de trunchiere :\n\r");
	cputs("---------------------------------------------------------------");
	gotoxy (2,i+5);
	cputs ("* Metoda Euler explicita :");
	textcolor (LIGHTCYAN);
	gotoxy (20,i+7);
	cputs ("- eroare locala pentru pasul 1 e1 : ");
	gotoxy (20,i+9);
	cputs ("- eroare   globala             eg : ");
	gotoxy (20,i+11);
	cputs ("- timp de calcul           Teuler : ");
	textcolor (WHITE);
	gotoxy (2,i+13);
	cputs ("* Metoda Runge Kutta de ordin 4 :");
	textcolor (LIGHTCYAN);
	gotoxy (20,i+15);
	cputs ("- eroare locala pentru pasul 1 e1 : ");
	gotoxy (20,i+17);
	cputs ("- eroare    globala            eg : ");
	gotoxy (20,i+19);
	cputs ("- timp   de  calcul       Trunge4 : ");
	return;
}


int quit(void)
{
	char	hs[12];

	hs[0] = 10;
	clrscr();
	textcolor (LIGHTCYAN);
	gotoxy (4,2);
	cputs ("Programul rezolva ecuatia : dy/dt = -y.\n\r\n\r");
	cputs ("   Conditia initiala      : y0 = 1.\n\r\n\r");
	cputs ("   Interval de integrare  : t  [0,4] [s].");
	textcolor (YELLOW);
	gotoxy (4,16);
	cputs ("Introduceti pasul de integrare(<ENTER> pentru iesire) : ");
	h = fabs( atof ( cgets(hs) ) );
	if ( h==0. ) return 1;
	return 0;
}

main()
{
	while( !quit() )
	{
		display();
		solve();
	}
	return;
}

