	/*
	[]-------------------------------------------------------[]
	|			 Nume program : RUNGE2                  |
	| - Rezolva prin metoda   Runge Kutta de ordin 4,         |
	| ecuatia: a2*y''+ a1*y'+ a0*y + b0 = 0,                  |
	| reductibila la un sistem liniar de ecuatii diferentiale |
	| de forma :							    |
	|		y1'+ a11*y1 + a12*y2 + b1 = 0			    |
	|		y2'+ a21*y1 + a22*y2 + b2 = 0			    |
	|									    |
	| - Intervalul de integrare este [0,tmax] s;		    |
	| - Pasul de integrare : h=tmax/100;			    |
	| - Conditiile initiale : y1(0)=C1; y2(0)=C2;             |
	| - Reprezinta grafic solutiile obtinute;                 |
	[]-------------------------------------------------------[]  */

#include <math.h>
#include <stdlib.h>
#include <conio.h>
#include "c:\mn\rezdif\euler\grafeu.cp"

# define	NMAX	400
# define    NEC	2
# define	MAX	1e6
# define 	D	double

D	a[NEC][NEC],			/* coeficientii sistemului */
	b[NEC],				/* termenii liberi */
	h,                            /* pasul de integrare */
	c[NEC],				/* conditile initiala */
	tmax;					/* limita de integrare */
extern float val[M + 1][N];		/* matricea solutiilor */

void graf(int n,D *ye)
{
	unsigned	i,j;

	for (i=0;i<=n;i++)
	{
		val[0][i] = (float)(i*h);
		for ( j=1; j<=NEC; j++)
			val[j][i] = (float)ye[NEC*i + j-1];
	}
	initial();
	subplot(1,1,1);
	plot(NEC,n+1,val);
	grid();
	restore();
	return;
}

void write_t(float t)
{
	gotoxy (10,24);
	cprintf("t = %4.1f",t);
	return;
}

D f (unsigned i,D *y)
{
	D	fn = 0.;
	unsigned	j;

	for ( j=0; j<=NEC-1; j++)
		fn = fn + a[i][j]*y[j];
	fn = fn + b[i];
	return (fn);
}

void runge_4 (D *y)    			/* metoda Runge Kutta, ordin 4*/
{
	D	g[NEC],s[NEC],z[NEC];
	unsigned			 i;

	for ( i=0; i<=NEC-1; i++) g[i] = f(i,y); /* calcul g1 */
	for ( i=0; i<=NEC-1; i++)
	{
		z[i] = y[i] + h*g[i]/2.;			/* argument g2 */
		s[i] = g[i];
	}
	for ( i=0; i<=NEC-1; i++)
			g[i] = f(i,z);				/* calcul g2 */
	for ( i=0; i<=NEC-1; i++)
	{
		z[i] = y[i] + h*g[i]/2.;			/* argument g3 */
		s[i] = s[i] + 2.*g[i];
	}
	for ( i=0; i<=NEC-1; i++)
			g[i] = f(i,z);				/* calcul g3 */
	for ( i=0; i<=NEC-1; i++)
	{
		z[i] = y[i] + h*g[i];			/* argument g4 */
		s[i] = s[i] + 2.*g[i];
	}
	for ( i=0; i<=NEC-1; i++)
			g[i] = f(i,z);				/* calcul g4 */
	for ( i=0; i<=NEC-1; i++)
		y[i+2] = y[i] + h*(s[i]+g[i])/6.;	/* solutie finala */
	i=1;
	return ;
}

void init_mat (D *y,unsigned lin,unsigned col)
{
	unsigned	i,max;

	max = lin*col - 1;
	for ( i=0; i<=max; i++) y[i] = 0.;
	return;
}

int magn_test (D *val,unsigned nr,unsigned *n,unsigned *i)
{
	unsigned	max,j,ctrl = 0;

	max = nr - 1;
	for ( j=0; j<=max; j++)
		if ( fabs(val[j])>(D)MAX )
		{ val[j] = 0.;
		  ctrl = 1;
		}
	if ( ctrl )
		{
			 *n = *i - 2;
			 return 1;
		}
	return 0;
}

void solve (void)			   /* rezolvare */
{
	unsigned	i,n;
	D	t = 0;			          /* nod initiala */
	D	ye[NMAX][NEC];

	init_mat (ye[0],NMAX,NEC);			/* initializare cu 0 */
	for ( i=0; i<=NEC-1; i++)
		ye[0][i] = c[i];                   /* conditii initiale */
	n = (int)(tmax/h + .5);                  /* nr. maxim de pasi */
	for ( i = 1; i <= n; i++ )               /* calcul */
	{
		if ( magn_test(ye[i-1],NEC,&n,&i) ) break;
		runge_4 (ye[i-1]);        	     /* calcul Runge Kutta 4 */
		t =t + h;                          /* nod nou */
		if ( i%10==0 ) write_t ( (float)t );
	}
	graf(n,ye[0]);					/* grafic */
	return;
}

int next(void)
{
	char	c;

	clrscr();
	gotoxy(5,10);
	cputs("Alta ecuatie ?[D/N]");
	c = getch();
	if ( c=='d' || c=='D' ) return 1;
	return 0;
}

void mat_input (char msg[25],char dat,D *v,int k)
{
	unsigned	j;
	char	  c[20];

	gotoxy (10,18);
	clreol();
	cputs (msg);
	for ( j=0; j<=NEC-1; j++)
	{
		gotoxy (10,20);
		clreol();
		if ( k<0 ) cprintf ("%c%1d = ",dat,j+1);
		else       cprintf ("%c%1d%1d = ",dat,k,j+1);
		c[0] = 16;
		v[j] = atof ( cgets(c) );
	}
	return;
}

void input (void) 		/* se introduc parametrii */
{
	char	s[20];
	int	i;

	s[0] = 16;
	clrscr();
	textcolor(LIGHTCYAN);
	cputs("\n\r  Programul rezolva un sistem liniar de ecuatii diferentiale de forma : \n\r\n\r");
	cputs("            /    \\     /       \\     /  \\     /  \\ \n\r");
	cputs("           |  y1' |   | a11 a12 |   | y1 |   | b1 |\n\r");
	cputs("           |  y2' | = | a21 a22 | * | y2 | + | b2 |\n\r");
	cputs("            \\    /     \\       /     \\  /     \\   /");
	cputs("\n\r\n\r  Domeniul de integrare : [0,tmax] [s]");
	cputs("\n\r  Pasul de integrare  h : tmax/400 [s]");
	cputs("\n\r  Conditii initiale : y1(0) = C1, y2(0) = C2 ");
	textcolor(YELLOW);
	gotoxy (10,16);
	cputs("tmax [in secunde]   : ");
	tmax = fabs( atof(cgets(s)) );
	for ( i=0; i<=NEC-1; i++)
		mat_input ("Coeficienti :",'a',a[i],i+1);
	mat_input ("Termeni liberi",'b',b,-1);
	mat_input ("Conditii initiale",'C',c,-1);
	if ( tmax==0. ) tmax = 1.;
	h = tmax/398.;
	textcolor(LIGHTCYAN);
	return;
}

main()
{
	do
	{
		input();
		solve();
	}while( next() );
	return;
}

