	  /*[]------------------------------------------------------------[]
		|   Nume program : TRANZ.EXE;						  		   |
		| - Rezolva circuite de curent continuu in regim tranzitoriu;  |
		| - O latura de circuit poate contine un element pasiv (R/L/C) |
		|    de valoare nenula, inseriat cu o sursa ideala 			   |
		|    de tensiune (poate fi de valoare nula);		           |
		| - In paralel cu latura poate exista o sursa de			   |
		|   curent (poate fi de valoare nula);						   |
		| - Sensul pozitiv pentru marimile ce cracterizeaza o latura   |
		|   (surse,curenti,diferenta de potential la borne) este de    |
		|   la nodul initial spre cel final;						   |
		| - Pentru laturile ce contin elemente de circuit reactive,    |
		|	trebuie specificate conditiile initiale;				   |
		| - Circuitul nu trebuie sa contina impedante cuplate;		   |
		| - La iesire se afiseaza si se reprezinta grafic (selectiv)   |
		|   curentii prin laturi si potentialele nodurilor circuitului |
		| - Se considera potential de referinta (nul), potentialul     |
		|	nodului de indice maxim;								   |
		[]------------------------------------------------------------[] */

# include <conio.h>
# include <math.h>

# include "traexdat.c"							/* contine blocul de variabile */
												/* definite in alte module */
# include "tralu.c"								/* contine functiile de rezolvare */
												/* a sistemului potentialelor la */
												/* la noduri prin metoda LU */

D   t,											/* timpul */
	magnt,										/* scara de afisare */
	curent[MAXL],								/* curenti prin laturi */
	potential[MAXN],							/* potentiale la noduri */
	g[MAXN][MAXN];								/* matrice conductante */

char	tu[8];									/* unitate de masura pentru t */

extern D	r[MAXL],							/* rezistente echivalente */
			e[MAXL];							/* surse echivalente */
extern U	maxstep,                            /* pas maxim */
			plotstep;							/* pas de afisare */

U			plotcount;							/* index matrice grafica */

extern float val[M+1][N];
char *strcpy(char *dest, const char *src);

# include "tradisp.c"							/* contine functiile de afisare /
												/* in tabel si grafic */
int quit(void)
{
	char	c;

	window (1,1,80,25);
	clrscr();
	gotoxy(5,10);
	cputs("Circuit nou?( [D]/N )");
	c = getch();
	if ( c=='d' || c=='D' ) return 0;
	return 1;
}

void message(void)
{
	clrscr();
	gotoxy(5,10);
	cputs("Circuit prost formulat!!!");
	getch();
}

void init_circuit (U step)
{
	U	ini,fin,i;

	for ( i=0; i<=l-1; i++)
	{
		ini = ni[i]-1;
		fin = nf[i]-1;
		if ( !step )				/* la t=0 se considera conditiile initiale */
			switch ( tipe[i] )
			{
			case 'L' : e[i] = evals[i] + r[i]*ci[i]; break;
			case 'C' : e[i] = evals[i] - ci[i];
			}
		else						/* altfel se considera marimile calculate */
									/* la pasul anterior */
			switch ( tipe[i] )
			{
			case 'L' : e[i] = evals[i] + r[i]*curent[i]; break;
			case 'C' : e[i] = potential[fin] - potential[ini];
			}
	}
}

void make_g (void)									/* creare matrice conductante */
{
	U	i,j;
	U	n1,n2;

	for (i=0; i<=l-1; i++)                          /* initializare */
		for (j=0; j<=n-1; j++)
				  g[i][j] = 0;
	for (i=0; i<l; i++)
    {
		n1 = ni[i]-1;
		n2 = nf[i]-1;
		g[n1][n1] += 1./r[i];
		g[n2][n2] += 1./r[i];
		g[n1][n2] -= 1./r[i];
		g[n2][n1] -= 1./r[i];
	}
}

void make_is (D *is)
{
	U	n1,n2,i;

	for (i=0; i<=n-1; i++)							/* initializare */
				is[i] = 0;
	for (i=0; i<=n-1; i++)
    {
		n1 = ni[i]-1;
		n2 = nf[i]-1;
		is[n1] = is[n1] - e[i]/r[i] - jvals[i] ;
		is[n2] = is[n2] + e[i]/r[i] + jvals[i] ;
	}
}

U solve_circuit (U step)
{
	D	is[MAXN];
	int	ier;
	U	ini,fin,i;

	if ( !step )
	{										/* la primul pas */
		make_g();							/* matrce conductante */
		ier = fact_lu (n,g);				/* factorizare matrice g */
		if (ier) return 1;
	}
	make_is (is);							/* vector curenti de scurt */
	rez_lu (n,g,is,potential);              /* determinare potentiale la noduri */
	potential[n-1] = 0;	     				/* potential de referinta */
	for ( i=0; i<=l-1; i++)					/* calcul curenti prin laturi */
	{
		ini = ni[i]-1;
		fin = nf[i]-1;
		curent[i] = ( e[i]+potential[ini]-potential[fin] )/r[i];
	}
	return 0;
}

void main()
{
	U 	step,er;

	do
	{
		input_data ();
		t = 0;									/* moment initial */
		step = 0;                               /* pas initial */
		plotcount = 0;
		for ( step=0; step<=maxstep; step++)
		{
			t += h;								/* moment de timp urmator */
			init_circuit (step);				/* calcul valori initiale */
			er = solve_circuit (step); 			/* rezolvare circuit la momentul t */
			if (er) {message();break;}			/* in caz de eroare, mesaj */
			tabel (step);						/* afisare in tabel */
			fill_plot_mat (step);		        /* completare matrice grafica */
		}
		if (!er && nplot)
		{
			wait();
			graf (plotcount,nplot,tu);			/* reprezentare grafica */
		}
	} while (!quit());
}
