/*****************************************************************************

    Program de demonstratie pentru analiza modala
    a matricilor folosind algoritmul HQ.

    La executia programului:
     - se introduce ordinul matricii;
     - se genereaza aleator o matrice de ordinul specificat;
     - se face analiza modala a matricii;
     - se afiseaza:
       - timpul de calcul;
       - eroarea absoulta;
       - eroarea relativa.

    Autor: Mihai Lazarescu, 17.12.1991
    (C) L.M.N., Mihai Lazarescu.

*****************************************************************************/

#include <complex.h>
#include <conio.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys\timeb.h>
#include <values.h>

#include "hqr2_f_c.h"
#include "elmhes.h"
#include "eltran.h"


#define NM         (20+1)       // Dimensiunea maxima a problemei
                                // (indicele 0 nu se foloseste).

typedef double tipmat;

tipmat  a[NM][NM];              // Matricea de analizat (forma reala)
complex ac[NM][NM];             // Matricea de analizat (forma complexa)
tipmat  z[NM][NM];              // Matricea vectorilor proprii (data de HQR2)
complex t[NM][NM];              // Matricea vectorilor proprii ("despachetati" din z
complex d[NM][NM];              // Matricea valorilor proprii (din wr si wi)
complex r1[NM][NM];
complex r2[NM][NM];             // Rezultate partiale

int     n;                      // Ordinul matricilor
int     low;
int     igh;
int     intchg[NM];
tipmat  wr[NM];
tipmat  wi[NM];                 // Auxiliare



/* Calculeaza intervalul de timp scurs intre t2 si t1 */
long delta_t(timeb *t2, timeb *t1)
/*
  Calculeaza diferenta de timp t2 - t1.
*/
{
	long  sec;                 /* Diferenta de timp in sec. */
    short ms;                  /* Diferenta de timp in milisec. */

    sec = t2->time - t1->time;
    ms  = t2->millitm - t1->millitm;

    return(sec*1000 + ms);
}

long cron_on(int on = 1)
/*
  Intoarce timpul in secunde scurs intre
  apelul de pronire si cel de oprire
*/
{
	static timeb timp_on;        /* Timpul la pornire */
	timeb        timp_off;       /* Timpul la oprire */

	if (on) {
        ftime(&timp_on);
        return(0);
	}
    ftime(&timp_off);
    return(delta_t(&timp_off, &timp_on));
}

/* Oprirea cronometrului */
long cron_off(void)
{
    return(cron_on(0));
}

int citeste_int(void)
{
    const MAXINTCAR = 3;
	struct {
		char max;
		char nin;
		char sir[MAXINTCAR+1];
	} s = { MAXINTCAR+1 };

	cgets((char*)&s);
	printf("\n");
	return(atoi(s.sir));
}

void afis_titlu(char s[])
{
    char    subl[81] = "";              // Sublinierea
    int     len      = strlen(s);       // Lungimea titlului
    int     blen     = 0;               // Numarul de blancuri initiale

    while (s[blen++] == ' ') ;          // Numararea blancurilor
    blen--;
    memset(subl, ' ', blen);            // Inserarea blancurilor
    memset(subl+blen, '=', len-blen);   // Inserarea sublinierii
    subl[len+blen] = '\0';              // Terminatorul sirului
    cprintf("%s\n\r%s\n\r", s, subl);   // Afisarea titlului si sublinierii
}

void afis_stitlu(char s[])
{
    printf("%s\n", s);                  // Afisarea subtitlului
}

float element(int init = 0)
// Genereaza un numar aleator real intre -1...1.
// Inainte de folosire, se initializeaza printr-un apel cu parametrul 1.
{
    float   x = 0.0;

    if (init)               // Initializarea generatorului aleator
        randomize();
    else {
        x = (float)random(MAXINT) / (float)MAXINT;
        if (random(2) == 0)
            x = -x;
    }

    return x;
}

int gen_matrice(int *n, tipmat m[], complex mc[])
{
    int     max = NM-1;

    do {
        cprintf("\nOrdinul matricii M (min. 1, max. %d, 0-iesire): ", max);
        *n = citeste_int();
        if (*n <= 0) return 1;              // Iesire
    } while (*n < 1 || *n > max);

    printf("\n\nSe genereaza matricea . . . ");
    element(1);                             // Initializare
    for (int i = 1; i <= *n; i++) {
        for (int j = 1; j <= *n; j++) {
            m[NM*i+j]  = 1000*element();     // Elemente intre -1000...1000
            mc[NM*i+j] = complex(m[NM*i+j], 0.0);
        }
    }
    printf("Complet.");
    return 0;
}

void form_t_d(void)
// Formeaza matricile T a vectorilor proprii si D a valorilor proprii.
{
    int     i, j;       // Contoare

    for (i = 1; i <= n; i++) {
        for (j = 1; j <= n; j++)
            d[i][j] = 0.0;
        d[i][i] = complex(wr[i], wi[i]);
    }

    for (i = 1; i <= n; i++)                // Parcurge valorile proprii
        if (wi[i] == 0.0)                   // Valoarea proprie are p. i. nula
            for (j = 1; j <= n; j++)
                t[j][i] = complex(z[j][i], 0.0);
        else if (wi[i] > 0.0)               // Valoarea proprie are p. i. poz.
            for (j = 1; j <= n; j++)
                t[j][i] = complex(z[j][i], z[j][i+1]);
        else                                // Valoarea proprie are p. i. neg.
            for (j = 1; j <= n; j++)
                t[j][i] = complex(z[j][i-1], -z[j][i-1]);
}

void inmul_mat(int nm, int n, complex *a, complex *b, complex *r)
// Inmulteste doua matrici patrate de ordin n: R = A*B.
// Dimensiunea maxima a matricilor este nm.
{
    int     i, j, k;
    complex s;

    for (k = 1; k <= n; k++)            // Parcurge coloanele matricii B
        for (i = 1; i <= n; i++) {      // Parcurge liniile matricii A
            s = complex(0.0, 0.0);
            for (j = 1; j <= n; j++)    // Parcurge coloanele matricii A
                s += a[nm*i+j] * b[nm*j+k];
            r[nm*i+k] = s;
        }
}

void calc_err(int nm, int n, complex ac[], complex t[], complex d[], float *ea, float *er)
{
    float   emax = 0.0;     // Eroarea maxima
    float   nor  = 0.0;     // Norma | TD |
    float   dif  = 0.0;     // Diferenta curenta
    float   norc = 0.0;     // Norma curenta
    int     i, j;           // Contoare

    inmul_mat(nm, n, ac, t, &r1[0][0]);
    inmul_mat(nm, n, t, d, &r2[0][0]);

    for (i = 1; i <= n; i++) {
        norc = 0.0;
        for (j = 1; j <= n; j++) {
            dif   = abs(r1[i][j] - r2[i][j]);
            norc += abs(r2[i][j]);
            if (dif > emax) emax = dif;
        }
        if (norc > nor) nor = norc;
    }
    *ea = emax;
    *er = emax / nor;
}

void afiseaza_rez(long timp, float ea, float er)
{
    char    t[]   =
"                        Analiza algoritmului";

    afis_titlu(t);
    printf("\n\n\n\n");
    printf("\n\tOrdinul matricii: %d", n);
    printf("\n");
    printf("\n\tTimpul de calcul: %.2f s", timp/1000.0);
    printf("\n\tEroarea absoluta: %.2E", ea);
    printf("\n\tEroarea relativa: %.3G %%", er*100.0);

    printf("\n\nApasati o tasta pentru continuare . . .");
    getch();
    clrscr();
}


void main(void)
{
    char    titlu[]    = "\
              ANALIZA MODALA A MATRICILOR FOLOSIND ALGORITMUL QR";
    char    stitlu[]   = "\
           Programul face analiza modala a unei matrici cu ordinul\n\
           specificat de utilizator, generata aleator  si afiseaza\n\
           timpul de calcul, eroarea absoluta si eroarea relativa.";
    int     coderr = 0;
    long    timp   = 0;
    float   ea     = 0.0;       // Eroarea absoluta
    float   er     = 0.0;       // Eroarea relativa
    tipmat  teps   = 1.0;
    tipmat  unu    = teps + 1.0;
    tipmat  machep = teps;         // Precizia de reprezentare a masinii

    // Determina precizia de reprezentare a masinii
    while (unu != 1.0) {
        machep  = teps;
        teps   /= 2.0;
        unu     = teps + 1.0;
    }

    highvideo();
    for (; ; ) {
        clrscr();
        afis_titlu(titlu);
        afis_stitlu(stitlu);
        printf("\n");
        if (gen_matrice(&n, &a[0][0], &ac[0][0]) != 0) break;

        printf("\nSe face analiza modala . . . ");
        low = 1;
        igh = n;
        cron_on();
        elmhes(NM, n, low, igh, &a[0][0], intchg);
        eltran(NM, n, low, igh, &a[0][0], intchg, &z[0][0]);
        coderr = hqr2_f_c(NM, n, low, igh, &a[0][0], wr, wi, &z[0][0], machep);
        timp   = cron_off();
        printf("Complet.");

        printf("\nSe calculeaza eroarea . . . ");
        form_t_d();
        calc_err(NM, n, &ac[0][0], &t[0][0], &d[0][0], &ea, &er);
        printf("Complet.");

        clrscr();
        if (coderr == 0)
            afiseaza_rez(timp, ea, er);
        else {
            printf("\n\n\n\n\n\n\n\n\n\n\n\n\t\t\tEroare !");
            printf("\n\n\n\n\nApasati o tasta . . .");
            getch();
        }
    }
    clrscr();
    lowvideo();
}
