/****************************************************************************
 *                                                                          *
 *    Program pentru trasarea caracteristicilor de frecventa                *
 *    ale unui circuit format din rezistoare, bobine si condensatoare.      *
 *                                                                          *
 *                                                                          *
 *    Data:         14.10.1991                                              *
 *    Programator:  Mihai Lazarescu                                         *
 *                                                                          *
 *    (C) L.M.N. - Mihai Lazarescu                                          *
 *                                                                          *
 ****************************************************************************/


/* Definirea constantelor */
#define NN      30		/* Nr. maxim de noduri (64 <=> 64Ko) */
#define NL      30		/* Nr. maxim de laturi */


/* Fisiere incluse */
#include <complex.h>
#include <conio.h>
#include <ctype.h>
#include <graphics.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "grafireu.h"
#include "gausscx.h"



/* Variabile */
complex	y[NN * NN];		/* Matricea admitantelor */
complex	is_v[NN];		/* Vectorul curentilor de scurtcircuit */

float	val[3][N];	    /* Valorile de reprezentat grafic */

int		nnod;			/* Nr. de noduri */
int		nlat;			/* Nr. de laturi */
int		dimmat;			/* Dimensiunea matricei */

int		tip[NL];		/* Tipul laturii */
float	p[NL];			/* Parametrul laturii */

int		ni[NL];			/* Nr. nodului initial */
int		nf[NL];			/* Nr. nodului final */

int		nrlatin;		/* Nr. laturii de intrare */
int		nrlaties;		/* Nr. laturii de iesire */

int		tipin;			/* Tipul semnalului de intrare */
int		tipies;			/* Tipul semnalului de iesire */

int		nrinterv;		/* Numarul de intervale de frecventa */

float	omegamin;		/* Frecventa minima */
float	omegamax;		/* Frecventa maxima */

float	PI;				/* Numarul PI */



int citeste_int(void)
{
#define MAXINTCAR 3
	struct {
		char max;
		char nin;
		char sir[MAXINTCAR+1];
	} s = { MAXINTCAR+1 };

	cgets((char*)&s);
	printf("\n");
	return(atoi(s.sir));

#undef MAXINTCAR
}

float citeste_real(void)
{
#define MAXREALCAR 20
	struct {
		char max;
		char nin;
		char sir[MAXREALCAR+1];
	} s = { MAXREALCAR+1 };

	cgets((char*)&s);
	printf("\n");
	return(atof(s.sir));

#undef MAXREALCAR
}

int introd_parcirc(void)
/* Introduce parametrii circuitului */
/* Intoarce 1 daca este introdusa o valoare eronata, */
/* altfel intoarce 0 */
/* Sint initializate variabilele: */
/*	- nlat cu numarul de laturi ale circuitului */
/*	- nnod cu numarul de noduri ale circuitului */
{
	/* Introducerea numarului de laturi */
	do {
		cprintf("\n\rIntroduceti numarul de laturi (min. 2, max. %d, 0-iesire): ", NL);
		nlat = citeste_int();
		if (nlat == 0) return(1);
	} while (nlat < 2 || nlat > NL);

	/* Introducerea numarului de noduri */
	do {
		cprintf("\n\rIntroduceti numarul de noduri (min. 2, max. %d, 0-iesire): ", NN);
		nnod = citeste_int();
		if (nnod == 0) return(1);
	} while (nnod < 2 || nnod > NN);

	return(0);
}

/* Introduce laturile circuitului */
void introd_circ(void)
/* Sint initializate variabilele: */
/*	- ni[]  cu nodurile initiale ale laturilor */
/*	- nf[]  cu nodurile finale ale laturilor */
/*	- p[]   cu parametrii laturilor */
/*	- tip[] cu tipul laturilor */
{
	int		i;			/* Contor */

	for (i = 0; i < nlat; i++) {	/* Parcurge laturile circuitului */
		p[i] = 0;					/* Initializeaza parametrul laturii */

		/* Introduce nodurile laturii */
		cprintf("\n\rLatura numarul %d (din %d):", i+1, nlat);
		printf("\nNodul initial (1...%d): ", nnod);
		ni[i] = citeste_int();
		if (ni[i] > nnod || ni[i] < 1) {	/* Nr. nodului este incorect */
			cprintf("\n\rNumar de nod incorect!\n\r");
			i--;
			continue;
		}
		ni[i]--;		/* Pentru indexare in matrice */

		printf("Nodul final (1...%d)  : ", nnod);
		nf[i] = citeste_int();
		if (nf[i] > nnod || nf[i] < 1) {	/* Nr. nodului este incorect */
			cprintf("\n\rNumar de nod incorect!\n\r");
			i--;
			continue;
		}
		nf[i]--;		/* Pentru indexare in matrice */

		/* Introduce tipul laturii */
		printf("Tipul laturii (R - implicit, G, L sau C): ");
		tip[i] = toupper(getch());	/* Tipul laturii */

		/* Introduce parametrul laturii */
		switch(tip[i]) {
			case 'L':
				printf("%c\nInductivitatea: ", tip[i]);
				p[i] = citeste_real();
				if (p[i] == 0) {
					cprintf("\n\rNu se admit laturi de inductanta nula!\n\r");
					i--;
				}
				break;
			case 'C':
				printf("%c\nCapacitatea: ", tip[i]);
				p[i] = citeste_real();
				if (p[i] == 0) {
					cprintf("\n\rNu se admit laturi de capacitate nula!\n\r");
					i--;
				}
				break;
			case 'G':
				printf("%c\nConductanta: ", tip[i]);
				p[i] = citeste_real();
				break;

			default:
				tip[i] = 'R';
			case 'R':
				printf("%c\nRezistenta: ", tip[i]);
				p[i] = citeste_real();
				if (p[i] == 0) {
					cprintf("\n\rNu se admit laturi de rezistenta nula!\n\r");
					i--;
				}
		}
	}
}

/* Introducerea intrarii si iesirii */
void introd_in_ies(void)
/* Sint initializate variabilele: */
/*	- nrlatin  cu numarul laturii de intrare */
/*	- nrlaties cu numarul laturii de iesire */
/*	- tipin    cu tipul semnalului de intrare */
/*	- tipies   cu tipul semnalului de iesire */
{
	/* Latura si semnalul de intrare */
	do {
		printf("\nIntroduceti numarul laturii de intrare: ");
		nrlatin = citeste_int();
		nrlatin--;
	} while (nrlatin < 0 || nrlatin > nlat - 1);
	if (tip[nrlatin] == 'G' && p[nrlatin] == 0.) {
		tipin = 'I';
		printf("Excitatie in curent.");
	}
	else {
		printf("Introduceti tipul semnalului de intrare (U, I): ");
		tipin = toupper(getch());
		if (tipin != 'I')
			tipin = 'U';
		printf("%c", (char)tipin);
	}

	/* Latura si semnalul de iesire */
	do {
		printf("\n\nIntroduceti numarul laturii de iesire: ");
		nrlaties = citeste_int();
		nrlaties--;
	} while (nrlaties < 0 || nrlaties > nlat - 1);
	if (tip[nrlaties] == 'G' && p[nrlaties] == 0.) {
		tipies = 'U';
		printf("Iesire in tensiune.");
	}
	else {
		printf("Introduceti tipul semnalului de iesire (U, I): ");
		tipies = toupper(getch());
		if (tipies != 'I')
			tipies = 'U';
		printf("%c", (char)tipies);
	}
}

/* Introduce intervalul de frecventa */
void introd_frecv(void)
/* Sint initializate variabilele: */
/*	- omegamin cu frecventa minima */
/*	- omegamax cu frecventa maxima */
/*	- nrinterv cu numarul de intervale */
{
	cprintf("\n\rIntroduceti intervalul de frecventa:");
	do {
		printf("\n\t - frecventa minima (pozitiva): ");
		omegamin = citeste_real();
		omegamin *= 2 * PI;
	} while (omegamin <= 0.);
	do {
		printf("\t - frecventa maxima (pozitiva): ");
		omegamax = citeste_real();
		omegamax *= 2 * PI;
	} while (omegamax <= omegamin);
	do {							/* Se va modifica N din GRAFIREU.H */
		printf("\t - numarul de intervale (min. 2, max. %d): ", N-1);
		nrinterv = citeste_int();
	} while (nrinterv < 2 || nrinterv > N-1);
}

int rezolvare(float omega)
/* Rezolva circuitul */
{
	int		n1, n2;		/* Nodul initial si final al laturii curente */
	complex yl;			/* Impedanta curenta */

	int     i;
	int     j;			/* Contoare */
	int     ier;		/* Indicator de eroare */


	dimmat = nnod - 1;	/* Dimensiunea matricei */

	/* Initializarea matricilor: y si is_v */
	for (i = 0; i < dimmat; i++) {
		is_v[i] = complex(0., 0.);
		for (j = 0; j < dimmat; j++)
			y[NN*i +j] = complex(0., 0.);
	}

	/* Calculeaza matricele Y si Is */
	for (i = 0; i < nlat; i++) {
		n1 = ni[i];
		n2 = nf[i];
		switch (tip[i]) {
			case 'L': yl = complex(0., -1./(p[i]*omega));
				break;
			case 'C': yl = complex(0., p[i]*omega);
				break;
			case 'G': yl = complex(p[i], 0.);
				break;
			default:
			case 'R': yl = complex(1./p[i], 0.);
		}
		if (n1 != dimmat)
			y[NN*n1 + n1] += yl;
		if (n2 != dimmat)
			y[NN*n2 + n2] += yl;
		if (n1 != dimmat && n2 != dimmat) {
			y[NN*n1 + n2] -= yl;
			y[NN*n2 + n1] -= yl;
		}

		if (i == nrlatin)
			if (tipin == 'U') {
				is_v[n1] = -yl;
				is_v[n2] =  yl;
			}
			else {
				is_v[n1] = complex(-1., 0.);
				is_v[n2] = complex( 1., 0.);
			}
	}

	/* Rezolvarea sistemului */
	ier = gauss_cplx(NN, dimmat, y, is_v);
	is_v[dimmat] = complex(0., 0.);		/* Potentialul nodului de referinta */

	return(ier);
}

int introddate(void)
/* Introducerea datelor circuitului */
{
	clrscr();

	/* Titlul */
	cprintf(    "                        CARACTERISTICI DE FRECVENTA");
	cprintf("\n\r                        ===========================");
    printf( "\n\n            Programul traseaza  caracteristicile  de frecventa si");
    printf(   "\n            de faza ale unui circuit liniar de curent alternativ.");

	printf("\n\n\n\n");

	/* Introducerea parametrilor circuitului */
	if (introd_parcirc())
		return(1);

	/* Introducerea laturilor circuitului */
	clrscr();
	introd_circ();

	/* Specificarea intrarii si iesirii */
	clrscr();
	introd_in_ies();

	/* Introducerea intervalului de frecventa */
	clrscr();
	introd_frecv();

	return(0);
}

int calculrez(int nrfrec, float omega)
/* Calculeaza rezultatele */
{
	int		n1, n2;		/* Nodul initial si final al laturii curente */
	complex yl;			/* Impedanta curenta */
	complex	u;			/* Tensiunea pe latura */
	complex	i;			/* Curentul prin latura */

	float	intens;		/* Valoarea efectiva a curentului printr-o latura */
	float	tens;		/* Valoarea efectiva a tensiunii pe o latura */
	float	fi;			/* Faza */

	int		ier;		/* Indicator de eroare */


	/* Rezolvarea circuitului */
	ier = rezolvare(omega);

	/* Extragerea rezultatelor */
	switch (tip[nrlaties]) {
		case 'L': yl = complex(0., -1./(p[nrlaties]*omega));
			break;
		case 'C': yl = complex(0., p[nrlaties]*omega);
			break;
		case 'G': yl = complex(p[nrlaties], 0.);
        	break;
		default:
		case 'R': yl = complex(1./p[nrlaties], 0.);
	}

	n1 = ni[nrlaties];
	n2 = nf[nrlaties];				/* Nodurile initial si final ale laturii */

	u = is_v[n1] - is_v[n2];		/* Tensiunea la bornele laturii */
	i = u * yl;						/* Curentul prin latura */
	if (nrlaties == nrlatin)
		if (tipin == 'I')
			i = complex(1., 0.);
		else
			i = (u + complex(1., 0.)) * yl;

	/* Valorile efective */
	intens = sqrt(norm(i));		/* Modulul curentului prin latura */
	tens   = sqrt(norm(u));		/* Modulul tensiunii la bornele lat. */
	/* Faza */
	fi     = (tipies == 'I') ? arg(i) : arg(u);	/* Faza */
	fi    *= 180. / PI;

	/* Memorarea rezultatelor */
	val[0][nrfrec] = omega / (2 * PI);
	val[1][nrfrec] = (tipies == 'I') ? intens : tens;
	val[2][nrfrec] = fi;

	return(ier);
}

void afisrez(void)
/* Afisarea rezultatelor */
{
	int		i;			/* Numarul frecventei curente */

	/* Afisarea rezultatelor */
	cprintf(    "                        Rezultatele");
	cprintf("\n\r                        ===========");

	for (i = 0; i <= nrinterv; i++) {
        printf("\nNr. pct.: %-3d   Frecventa: %.2E   Val. ef.: %+.2E   Faza: %+.3G", i+1, val[0][i], val[1][i], val[2][i]);
		if ((i+1) % 22 == 0) {
			printf("\nPentru continuare apasati o tasta: ");
			getch();
			clrscr();
		}
	}
	if ((nrinterv+1) % 22 != 0) {
		printf("\nPentru continuare apasati o tasta: ");
		getch();
		clrscr();
	}

	/* Reprezentarea grafica */

	/* Caracteristica de frecventa */
	setgraphmode(getgraphmode());
	subplot(1, 2, 1);
	loglog(1, 					/* Numarul de grafice */
		nrinterv+1,				/* Numarul de puncte */
		val);					/* val[0] = x; restul: cite un y */
	grid();
	title("Caracteristica de frecventa");
	label_x("Frecventa");
	label_y("Modulul");

	/* Caracteristica de faza */
	subplot(1, 2, 2);
	for (i = 0; i <= nrinterv; i++)
		val[1][i] = val[2][i];
	semilogx(1,					/* Numarul de grafice */
		nrinterv+1,				/* Numarul de puncte */
		val);					/* val[0] = x; restul: cite un y */
	grid();
	title("Caracteristica de faza");
	label_x("Frecventa");
	label_y("Faza");
	getch();
	restorecrtmode();
}


void main(void)
{
	float	omega;				/* Frecventa curenta */
	float	rataomega;			/* Pasul de frecventa */

	int		ier;				/* Indicator de eroare */


	PI = atan(1) * 4;			/* Initializare */

	initial();
	restorecrtmode();			/* Pentru pachetul grafic */

	highvideo();
	do {
		int intervcrt;			/* Numarul intervalului de frecventa curent */

		if (introddate())		/* Introducerea datelor */
			break;				/* Terminarea executiei */

		clrscr();
		printf("\n\n\n\n\n\n\n\n\n\n\t\t\t\tC A L C U L E");

        omega = omegamin; 
		rataomega = pow((omegamax / omegamin), 1. / nrinterv);	/* Pasul de frecv. */
		for (intervcrt = 0; intervcrt <= nrinterv; intervcrt++) {
			ier = calculrez(intervcrt, omega);		/* Rezolvarea circ. */
			if (ier) {
				cprintf("\n\n\rCircuit prost formulat!\n\r");
				break;		/* Eroare la rezolvare */
			}
			omega *= rataomega;
		}
		if (!ier) {
			clrscr();
			afisrez();		/* Afisarea si reprezentarea grafica a rez. */
		}

	} while(1);
	lowvideo();
}
