#include <math.h>

void elmhes(int nm, int n, int low, int igh, double *a, int intchg[])
{
//
      int       i, j, m, la, kp1, mm1, mp1;
      double    x, y;

//     This subroutine is a translation of the algol procedure ELMHES,
//     Num. math. 12, 349-368(1968) by Martin and Wilkinson.
//     Handbook for auto. comp., vol.II-Linear algebra, 339-358(1971).

//     Given a real general matrix, this subroutine
//     reduces a submatrix situated in rows and columns
//     low through igh to upper Hessenberg form by
//     stabilized elementary similarity transformations.

//     On input:

//     nm must be set to the row dimension of two-dimensional
//       array parameters as declared in the calling program
//       dimension statement,

//     n is the order of the matrix,

//     low and igh are integers determined by the balancing
//       subroutine  BALANC.    If  BALANC  has not been used,
//       set low=1, igh=n,

//     a contains the input matrix.

//     On output:

//     a contains the Hessenberg matrix.  The multipliers
//       which were used in the reduction are stored in the
//       remaining triangle under the Hessenberg matrix,

//     intchg contains information on the rows and columns
//       interchanged in the reduction.
//       Only elements low through igh are used.

//     Questions and comments should be directed to B. S. Garbow,
//     Applied mathematics division, Argonne National Laboratory

//     ------------------------------------------------------------------

    la  = igh - 1;
    kp1 = low + 1;
    if (la < kp1) return;

    for (m = kp1; m <= la; m++) {
        mm1 = m - 1;
        x = 0.0;
        i = m;

        for (j = m; j <= igh; j++) {
            if (fabs(a[nm*j+mm1]) <= fabs(x)) continue;
            x = a[nm*j+mm1];
            i = j;
        }

        intchg[m] = i;
        if (i != m) {       // ***** Interchange rows and columns of a
            for (j = mm1; j <= n; j++) {
                y = a[nm*i+j];
                a[nm*i+j] = a[nm*m+j];
                a[nm*m+j] = y;
            }

            for (j = 1; j <= igh; j++) {
                y = a[nm*j+i];
                a[nm*j+i] = a[nm*j+m];
                a[nm*j+m] = y;
            }
        }       // ***** End interchange

        if (x == 0.0) continue;
        mp1 = m + 1;

        for (i = mp1; i <= igh; i++) {
            y = a[nm*i+mm1];
            if (y == 0.0) continue;
            y           /= x;
            a[nm*i+mm1]  = y;
            for (j = m; j <= n; j++)
                a[nm*i+j] -= y * a[nm*m+j];

            for (j = 1; j <= igh; j++)
                a[nm*j+m] += y * a[nm*j+i];
        }
    }
}
