// Fisiere incluse
#include <complex.h>
#include <conio.h>
#include <graphics.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys\timeb.h>
#include "fourierx.h"
#include "grafireu.h"
#include "y_tab.h"


// Constante
#define NI	    200			// Nr. maxim de intervale (numar par)
							// (trebuie sa fie egal cu N din GRAFIREU.H)
#define NA	    100			// Nr. maxim de armonici
							// (trebuie sa fie mai mic decit N din GRAFIREU.H)
#define MAXEXPR	72			// Lungimea maxima a expresiei functiei

// Variabile
char	s1[MAXEXPR+1];	// Expresia functiei pe [T/2...T]
char	s2[MAXEXPR+1];	// Expresia functiei pe [0...T/2]
int		n;				// Numarul de intervale
int		m;				// Numarul de armonici
float	T;				// Perioada
float	w;				// Pulsatia
complex	c[NI+2];		// Coeficientii descompunerii
float	t[NI+2];		// Discretizarea domeniului de definitie al functiei
complex	f[NI+2];		// Valorile functiei
float	val[3][N];		// Functia si aproximarea ei (pentru grafice)

float	PI;				// Numarul PI



void cflush(void)
{
    while (kbhit()) getch();
}

int citeste_int(void)
{
#define MAXINTCAR 3
	struct {
		char max;
		char nin;
		char sir[MAXINTCAR+1];
	} s = { MAXINTCAR+1 };

	cgets((char*)&s);
	printf("\n");
	return(atoi(s.sir));

#undef MAXINTCAR
}

float citeste_real(void)
{
#define MAXREALCAR 20
	struct {
		char max;
		char nin;
		char sir[MAXREALCAR+1];
	} s = { MAXREALCAR+1 };

	cgets((char*)&s);
	printf("\n");
	return(atof(s.sir));

#undef MAXREALCAR
}

// Intoarce o valoare nenula daca este o cerere de HELP
int estehelp(char *s)
{
	return(strstr(s, "?") != NULL);
}

// Verifica sintactic expresia transmisa ca parametru
int veriffunc(char sir[])
{
    char sirex[MAXEXPR + 5];        // 4 car. rezerva pt. valoarea lui t
    int coderr = 0;                 // Cod de eroare

    if (estehelp(sir)) {            // Se cere HELP-ul parserului
        clrscr();
        parser("?", &coderr, 1);    // Se afiseaza HELP-ul
    }
    else {
        sprintf(sirex, "t=0;");     // Introduce valoarea argumentului expresiei
        strcat(sirex, sir);         // Adauga expresia
        parser(sirex, &coderr, 1);  // Verifica sintactic expresia
    }
    return(coderr);                 // Intoarce codul de eroare
}

// Evalueaza expresia in punctul x transmis ca parametru */
double evalfunc(char sir[], double t, int *coderr)
{
    char sirex[MAXEXPR + 31];       // 30 de car. rezerva pt. val. lui t

    sprintf(sirex, "t=%.20e;", t);  // Introduce valoarea argumentului expresiei
    strcat(sirex, sir);             // Adauga expresia
    return(parser(sirex, coderr, 0));  // Evalueaza expresia
}

// Introduce expresiile functiei pe intervalele [0..T/2] si [T/2..T]
int introdfunc(char s1[], char s2[])
{
    struct {
        char max;                // Numarul maxim de caractere permis
        char nin;                // Numarul de caractere introduse
        char sir[MAXEXPR + 1];    // Sirul introdus
    } sstr1, sstr2;

    sstr1.max = MAXEXPR;
    sstr2.max = MAXEXPR;
    do {
        printf("\n\nIntroduceti expresia functiei f pe intervalul [0..T/2] (? = Help):");
        printf("\nf(t) = ");
        fflush(stdin);                        // Curata bufferul
        cgets((char *)&sstr1);                // Citeste expresia
        if (sstr1.nin == 0) return(1);
    } while(veriffunc(sstr1.sir) != 0);       // Verifica sintaxa expr.
    do {
        printf("\n\nIntroduceti expresia functiei f pe intervalul [T/2..T] (? = Help):");
        printf("\nf(t) = ");
        fflush(stdin);                        // Curata bufferul
        cgets((char *)&sstr2);                // Citeste expresia
        if (sstr2.nin == 0) return(1);
    } while(veriffunc(sstr2.sir) != 0);       // Verifica sintaxa expr.
    strcpy(s1, sstr1.sir);
    strcpy(s2, sstr2.sir);

    return(0);
}

void titlu(void)
{
	clrscr();

	// Titlul
	cprintf(    "                            ANALIZA ALGORITMILOR");
	cprintf("\n\r                            ====================");
	printf( "\n\n      Programul determina coeficientii Fourier pentru o  functie data prin");
	printf(   "\n      trei metode si calculeaza seria Fourier trunchiata. Afiseaza eroarea");
	printf(   "\n      de aproximare a functiei  prin seria trunchiata si timpul de calcul.");
}

// Introducerea perioadei
int introd_perioada(float *T)
{
	do {
		printf("\n\nPerioada functiei, T = ");
		*T = citeste_real();
	} while (*T < 0.);

	if (*T == 0.) return(1);

	return(0);
}

// Introducerea numarului de intervale
int introd_nrinterv(int *n, int max)
{
	do {
		printf("\nNumarul de intervale pe semiperioada (min. 1, max. %d): ", max);
		*n = citeste_int();
		if (*n == 0) return(1);		// Iesire
	} while (*n < 1 || *n > max);
	*n *= 2;

	return(0);
}

// Introducerea numarului de puncte
int introd_nrpct(int *n, int max)
{
	do {
		printf("\nNumarul de puncte (min. 2, max. %d): ", max);
		*n = citeste_int();
		if (*n == 0) return(1);			// Iesire
	} while (*n < 2 || *n > max);

	return(0);
}

// Introducerea numarului de puncte pentru algoritmul FFT
int FFT_introd_nrpct(int *n, int max)
{
    int     ntemp = 0;

	do {
		printf("\nNumarul de puncte (min. 2, max. %d, putere a lui 2): ", max);
		ntemp = citeste_int();
		if (ntemp == 0) return(1);          // Iesire
	} while (ntemp < 2 || ntemp > max);

    // Aproximeaza la puterea lui 2 cea mai apropiata inferior
    for (*n = 1; *n < max && *n < ntemp; *n *= 2) ;
    if (*n > max || *n > ntemp) *n /= 2;

	return(0);
}

// Introducerea numarului de armonici
int introd_nrarm(int *m, int max1, int max2)
{
    int     min = max1;     // Minimul maximelor

    if (max2 < max1) min = max2;
	do {
		printf("\nNumarul de armonici (min. 1, max. %d): ", min);
		*m = citeste_int();
		if (*m == 0) return(1);
	} while (*m < 1 || *m > min);

	return(0);
}

// Calculeaza functia
void calculfunc(char *sir, int n, float xmin, float xmax, float x[], complex f[])
{
	float	xc;		// Punctul curent
	float	dx;		// Pasul lui x
	int		i;		// Contor
    int     cod;    // Cod de eroare

	xc = xmin;
	dx = (xmax - xmin) / n;
	for (i = 0; i <= n; i++) {
		f[i] = evalfunc(sir, xc, &cod);
		x[i] = xc;
		xc  += dx;
	}
}

void discr_functie(int n, float T, char *s1, char *s2, float t[], complex f[])
{
	// Liniarizarea functiei pe portiuni
	// Pe intervalul [0...T/2]
	calculfunc(s1, n/2, 0., T/2., &t[0], &f[0]);

	// Pe intervalul [T/2...T]
	calculfunc(s2, n/2, T/2., T, &t[n/2+1], &f[n/2+1]);
}

// Discretizeaza intervalul [0...T] in n intervale echidistante
void FFT_discr_functie(int n, float T, char *s1, char *s2, float t[], complex f[])
{
	float	T2 = T/2.;			// Mijlocul perioadei
	float	dt = T/n;			// Pasul lui t
	float	tc = 0.;			// Punctul curent
	int		i;					// Contor
    int     cod;                // Cod de eroare

	for (i = 0; i <= n; i++, tc += dt) {
		if (tc <= T2)
			f[i] = evalfunc(s1, tc, &cod);
		else
			f[i] = evalfunc(s2, tc, &cod);
		t[i] = tc;
	}
}

void afis_functie(int n, float x[], complex f[])
{
	int		i;				// Numarul frecventei curente

	for (i = 0; i < n; i++) {
		val[0][i] = x[i];
		val[1][i] = real(f[i]);
	}
	setgraphmode(getgraphmode());
	plot(1, 					// Numarul de grafice
		n,						// Numarul de puncte
		val);					// val[0] = x; restul: cite un y
	grid();
	title("Functia");
	label_x("Timp");
	label_y("Valoare");
}

void afisrez(int m, int n, complex f[])
{
	int		i;				// Numarul frecventei curente
	char	titlu[81];		// Titlul graficului
	char	prep[] = "de ";
	char	acord = 'i';

	for (i = 0; i < n; i++)
		val[2][i] = real(f[i]);
	setgraphmode(getgraphmode());
	plot(2, 					// Numarul de grafice
		n,						// Numarul de puncte
		val);					// val[0] = x; restul: cite un y
	grid();
	if (m%100 < 20 && m%100 != 0)
		prep[0] = '\0';
	if (m == 1)
		acord = 'a';
	sprintf(titlu, "Functia si aproximarea Fourier cu %d %sarmonic%c", m, prep, acord);
	title(titlu);
	label_x("Timp");
	label_y("Valoare");
}

// Calculeaza eroarea maxima de aproximare a functiei
float calc_err(int n)
{
    float   y1, y2;         // Variabile auxiliare
	int		i;              // Contor
	float	err = 0.;       // Eroarea curenta

	for (i = 0; i < n-1; i++) {
        y1 = val[2][i+1] - val[1][i+1];
        y1 *= y1;
        y2 = val[2][i] - val[1][i];
        y2 *= y2;

        err += (y1+y2) * (val[0][i+1] - val[0][i]);
	}
    return(sqrt(err/2.));
}

// Afiseaza eroarea de aproximare
void afis_mas(float err, long t_coef, long t_eval)
{
	clrscr();
	cprintf(    "                                   REZULTATE");
	cprintf("\n\r                                   =========");
	printf("\n\n\n\n\n\tEroarea maxima a aproximarii                  : %8.2E", err);
	printf(      "\n\n\tTimpul de calcul al coeficientilor Fourier    : %6.2f s", (float)t_coef/1000.);
	printf(      "\n\n\tTimpul de evaluare a seriei Fourier trunchiate: %6.2f s", (float)t_eval/1000.);
	getch();
}

// Calculeaza intervalul de timp scurs intre t2 si t1
long delta_t(timeb *t2, timeb *t1)
{
	long  sec;                 // Diferenta de timp in sec.
    short ms;                  // Diferenta de timp in milisec.

    sec = t2->time - t1->time;
    ms  = t2->millitm - t1->millitm;

    return(sec*1000 + ms);
}

#pragma warn -def
#pragma warn -stv
long cron_on(int on = 1)
// Intoarce timpul in secunde scurs intre
// apelul de pronire si cel de oprire

{
	static timeb timp_on;        // Timpul la pornire
	timeb        timp_off;       // Timpul la oprire

	if (on) {
        ftime(&timp_on);
        return(0);
	}
    ftime(&timp_off);
    return(delta_t(&timp_off, &timp_on));
}
#pragma warn +def
#pragma warn +stv

// Oprirea cronometrului
long cron_off(void)
{
    return(cron_on(0));
}

// Introducerea datelor pentru seria Fourier
int serie_introddate(void)
{
	// Introducerea functiei
	if (introdfunc(s1, s2))
		return(1);

	// Introducerea perioadei
	if (introd_perioada(&T))
		return(1);

	// Introducerea numarului de intervale
	if (introd_nrinterv(&n, NI/2))
		return(1);

	// Introducerea numarului de armonici
	if (introd_nrarm(&m, n/2, NA))
		return(1);

	return(0);
}

// Introducerea datelor pentru transformata discreta
int trans_discr_introddate(void)
{
	// Introducerea functiei
	if (introdfunc(s1, s2))
		return(1);

	// Introducerea perioadei
	if (introd_perioada(&T))
		return(1);

	// Introducerea numarului de puncte
	if (introd_nrpct(&n, NI))
		return(1);

	// Introducerea numarului de armonici
    m = n;

	return(0);
}

// Introducerea datelor pentru algoritmul FFT
int FFT_introddate(void)
{
	// Introducerea functiei
	if (introdfunc(s1, s2))
		return(1);

	// Introducerea perioadei
	if (introd_perioada(&T))
		return(1);

	// Introducerea numarului de puncte
	if (FFT_introd_nrpct(&n, NI))           // Este si numarul de armonici
		return(1);

	return(0);
}

// Analiza seriei Fourier
void serie(void)
{
	float	err;				// Eroarea de aproximare
	long    timp_dir;			// Timpul de calcul al coef. Fourier
	long    timp_inv;			// Timpul de evaluare al seriei Fourier

	do {
		// Titlul
		cprintf(    "                         ANALIZA METODEI TRAPEZELOR");
		cprintf("\n\r                         ==========================");
		printf("\n\n\n");

		if (serie_introddate())		// Introducerea datelor
			return;					// Terminarea testarii

		w = 2*PI / T;

		setgraphmode(getgraphmode());

		// Discr. functiei
		discr_functie(n, T, s1, s2, t, f);	// n - numarul de intervale
		afis_functie(n+2, t, f);

		// Calculul coeficientilor Fourier ai functiei introduse
		cron_on();
		Fourier_lpp(n+2, t, f, m, c);
		timp_dir = cron_off();

		// Calculul seriei Fourier trunchiate
		cron_on();
		eval_Fourier(n+2, t, f, m, c);
		timp_inv = cron_off();

		// Afisarea rezultatelor
		afisrez(m, n+2, f);

		// Calcularea erorii de aproximare maxime
		err = calc_err(n+2);

        cflush();
		restore();

		afis_mas(err, timp_dir, timp_inv);

		clrscr();

	} while (1);
}

// Analiza transformatei Fourier discrete
void transf_discr(void)
{
	float	err;				// Eroarea de aproximare
	long    timp_dir;			// Timpul de calcul al transf. Fourier directe
	long    timp_inv;			// Timpul de calcul al transf. Fourier inverse

	do {
		// Titlul
		cprintf(    "                      ANALIZA METODEI DREPTUNGHIURUILOR");
		cprintf("\n\r                      =================================");
		printf("\n\n\n");

		if (trans_discr_introddate())	// Introducerea datelor
			return;						// Terminarea testarii

		setgraphmode(getgraphmode());

		// Discr. functiei
		FFT_discr_functie(n-1, T, s1, s2, t, f);	// n - numarul de puncte
		afis_functie(n, t, f);

		// Calculul coeficientilor Fourier ai functiei introduse
		cron_on();
		transf_F(1, n, f, c);
		timp_dir = cron_off();

		// Calculul seriei Fourier trunchiate
		cron_on();
		transf_F(0, n, c, f);
		timp_inv = cron_off();

		// Afisarea rezultatelor
		afisrez(n/2, n, f);

		// Calcularea erorii de aproximare maxime
		err = calc_err(n);

        cflush();
		restore();

		afis_mas(err, timp_dir, timp_inv);

		clrscr();

	} while (1);
}

// Analiza transformatei Fourier discrete optimizate
void transf_discr_opt(void)
{
	float	err;				// Eroarea de aproximare
	long    timp_dir;			// Timpul de calcul al transf. Fourier directe
	long    timp_inv;			// Timpul de calcul al transf. Fourier inverse

	do {
		// Titlul
		cprintf(    "                 ANALIZA METODEI DREPTUNGHIURUILOR OPTIMIZATA");
		cprintf("\n\r                 ============================================");
		printf("\n\n\n");

		if (trans_discr_introddate())	// Introducerea datelor
			return;						// Terminarea testarii

		setgraphmode(getgraphmode());

		// Discr. functiei
		FFT_discr_functie(n-1, T, s1, s2, t, f);	// n - numarul de puncte
		afis_functie(n, t, f);

		// Calculul coeficientilor Fourier ai functiei introduse
		cron_on();
		transf_F_opt(1, n, f, c);
		timp_dir = cron_off();

		// Calculul seriei Fourier trunchiate
		cron_on();
		transf_F_opt(0, n, c, f);
		timp_inv = cron_off();

		// Afisarea rezultatelor
		afisrez(n/2, n, f);

		// Calcularea erorii de aproximare maxime
		err = calc_err(n);

        cflush();
		restore();

		afis_mas(err, timp_dir, timp_inv);

		clrscr();

	} while (1);
}

// Analiza algoritmului FFT
void transf_FFT(void)
{
	float	err;				// Eroarea de aproximare
	long    timp_dir;			// Timpul de calcul al transf. Fourier directe
	long    timp_inv;			// Timpul de calcul al transf. Fourier inverse

	do {
		// Titlul
		cprintf(    "                           ANALIZA ALGORITMULUI FFT");
		cprintf("\n\r                           ========================");
		printf("\n\n\n");

		if (FFT_introddate())			// Introducerea datelor
			return;						// Terminarea testarii

		setgraphmode(getgraphmode());

		// Discr. functiei
		FFT_discr_functie(n-1, T, s1, s2, t, f);	// n - numarul de puncte
		afis_functie(n, t, f);

		// Calculul coeficientilor Fourier ai functiei introduse
		cron_on();
		FFT(f, n, 0);
		timp_dir = cron_off();

		// Calculul seriei Fourier trunchiate
		cron_on();
		FFT(f, n, 1);
		timp_inv = cron_off();

		// Afisarea rezultatelor
		afisrez(n/2, n, f);

		// Calcularea erorii de aproximare maxime
		err = calc_err(n);

        cflush();
		restore();

		afis_mas(err, timp_dir, timp_inv);

		clrscr();

	} while (1);
}


void main(void)
{
	int		tr;					// Selectorul rutinei de analizat

	PI = atan(1.) * 4.;			// Initializare

	initial();
	restorecrtmode();			// Pentru pachetul grafic

	highvideo();
	do {
		titlu();

		printf("\n\n\n");
		printf(  "\n\t\t\t1 - Metoda trapezelor");
		printf(  "\n\t\t\t2 - Metoda dreptunghiurilor");
		printf(  "\n\t\t\t3 - Metoda dreptunghiurilor optimizata");
		printf(  "\n\t\t\t4 - Algoritmul FFT");
		printf("\n\n\t\t\t0 - IESIRE");
		printf("\n\n\t\t\tSelectati metoda de analizat: ");

		tr = citeste_int();
		if (tr == 0) break;			// Terminarea executiei

		clrscr();
		switch (tr) {
			case 1:
				serie();
				break;
			case 2:
				transf_discr();
				break;
            case 3:
                transf_discr_opt();
                break;
			case 4:
				transf_FFT();
				break;
		}

	} while(1);
	lowvideo();
}
