/* Procedura Gauss pentru sisteme complexe */
/*-----------------------------------------*/
/* Data: 14.10.1991
   Programator: Mihai Lazarescu */

/* Fisiere incluse */
#include <complex.h>

int gauss_cplx(int nm, int n, complex a[], complex b[])
{   /* nm = dimensiunea problemei
       n  = dimensiunea matricei
       a  = matricea coeficientilor
       b  = vectorul termenilor liberi/solutie
       Matricea a si vectorul b se distrug.
       Intoarce: ier = 0 - Ok;
                 ier = 1 - eroare. Pivotul nu e diferit de 0. */

#define EPSILON 1.0E-10

    int     i;          /* Contorul liniei de referinta curente */
    int     j;          /* Contorul coloanei curente */
    int     k;          /* Contorul liniei curente */

    complex p;          /* Factor */
    complex s;          /* Suma partiala */


    /* Eliminarea */
    for (i = 0; i < n; i++) {
        for (k = i+1; k < n; k++) {
            if (abs(a[nm*i+i]) < EPSILON)
                return(1);                  /* Eroare: pivot nul */
            p = -a[nm*k+i] / a[nm*i+i];
            b[k] += b[i] * p;
            for (j = i+1; j < n; j++)
                a[nm*k+j] += a[nm*i+j] * p;
        }
    }
    if (abs(a[nm*(n-1)+n-1]) < EPSILON)
        return(1);                          /* Eroare: pivot nul */

    /* Retrosubstitutia */
    for (i = n-1; i >= 0; i--) {
        s = complex(0., 0.);
        for (k = i+1; k < n; k++)
            s += b[k] * a[nm*i+k];
        b[i] = (b[i] - s) / a[nm*i+i];
    }
    return(0);

#undef EPSILON
}
