#include <complex.h>
#include <stdlib.h>

int hqr2_f_c(int nm, int n, int low, int igh, double *h, double wr[], double wi[], double *z, float machep)
{
      int       i, j, k, l, m, en, ii, jj, ll, mm, na, nn,
                its, mp2, enm2, ierr, notlas;
      double    p, q, r, s, t, w, x, y, ra, sa, vi, vr, zz, norm;
      complex   z3;
//
#define sign(a, b)          (((b) >= 0) ? (a) : (-a))
//
//
//
//
//     This subroutine is a translation of the ALGOL procedure HQR2,
//     num. math. 16, 181-204(1970) by Peters and Wilkinson.
//     Handbook for auto. comp., vol.II-linear algebra, 372-395(1971).
//
//     This subroutine finds the eigenvalues and eigenvectors
//     of a real upper hessenberg matrix by the QR method.  The
//     eigenvectors of a real general matrix can also be found
//     if  ELMHES  and  ELTRAN  or  ORTHES  and    ORTRAN    have
//     been used to reduce this general matrix to hessenberg form
//     and to accumulate the similarity transformations.
//
//     On input:
//
//     nm must be set to the row dimension of two-dimensional
//       array parameters as declared in the calling program
//       dimension statement,
//
//     n is the order of the matrix,
//
//     low and igh are integers determined by the balancing
//       subroutine  balanc.    If  BALANC  has not been used,
//       set low=1, igh=n,
//
//     h contains the upper hessenberg matrix,
//
//     z contains the transformation matrix produced by  eltran
//       after the reduction by  elmhes, or by  ortran  after the
//       reduction by  orthes, if performed.    if the eigenvectors
//       of the hessenberg matrix are desired, z must contain the
//       identity matrix.
//
//     on output-
//
//     h has been destroyed,
//
//     wr and wi contain the real and imaginary parts,
//       respectively, of the eigenvalues.  the eigenvalues
//       are unordered except that complex conjugate pairs
//       of values appear consecutively with the eigenvalue
//       having the positive imaginary part first.  if an
//       error exit is made, the eigenvalues should be correct
//       for indices ierr+1,...,n,
//
//     z contains the real and imaginary parts of the eigenvectors.
//       if the i-th eigenvalue is real, the i-th column of z
//       contains its eigenvector.  if the i-th eigenvalue is complex
//       with positive imaginary part, the i-th and (i+1)-th
//       columns of z contain the real and imaginary parts of its
//       eigenvector.  the eigenvectors are unnormalized.  if an
//       error exit is made, none of the eigenvectors has been found,
//
//     ierr is set to
//       zero       for normal return,
//       j          if the j-th eigenvalue has not been
//              determined after 30 iterations.
//
//     arithmetic is real except for the replacement of the algol
//     procedure cdiv by complex division.
//
//     questions and comments should be directed to b. s. garbow,
//     applied mathematics division, argonne national laboratory
//
//     ------------------------------------------------------------------
//
//     ********** machep is a machine dependent parameter specifying
//         the relative precision of floating point arithmetic.
//
//         **********
//
      ierr = 0;
      norm = 0.0;
      k    = 1;
//     ********** Store roots isolated by balanc
//         and compute matrix norm **********
      for (i = 1; i <= n; i++) {
          for (j = k; j <= n; j++)
              norm += fabs(h[nm*(i)+j]);
          k = i;
          if (i >= low && i <= igh) continue;
          wr[i] = h[nm*(i)+i];
          wi[i] = 0.0;
      }
//
      en = igh;
      t  = 0.0;
//     ********** Search for next eigenvalues **********
E60:  if (en < low) goto E340;
      its = 0;
      na = en - 1;
      enm2 = na - 1;
//     ********** Look for single small sub-diagonal element
//         for l=en step -1 until low do -- **********
E70:  for (ll = low; ll <= en; ll++) {
          l = en + low - ll;
          if (l == low) break;
          s = fabs(h[nm*(l-1)+l-1]) + fabs(h[nm*(l)+l]);
          if (s == 0.0) s = norm;
          if (fabs(h[nm*(l)+l-1]) <= machep * s) break;
      }
//     ********** Form shift **********
      x = h[nm*(en)+en];
      if (l == en) goto E270;
      y = h[nm*(na)+na];
      w = h[nm*(en)+na] * h[nm*(na)+en];
      if (l == na) goto E280;
      if (its == 30) goto E1000;
      if (its != 10 && its != 20) goto E130;
//     ********** Form exceptional shift **********
      t += x;
//
      for (i = low; i <= en; i++)
          h[nm*(i)+i] -= x;
//
      s = fabs(h[nm*(en)+na]) + fabs(h[nm*(na)+enm2]);
      x = 0.75 * s;
      y = x;
      w = -0.4375 * s * s;
E130: its++;
//     ********** Look for two consecutive small
//         sub-diagonal elements.
//         for m=en-2 step -1 until l do -- **********
      for (mm = l; mm <= enm2; mm++) {
          m = enm2 + l - mm;
          zz = h[nm*(m)+m];
          r = x - zz;
          s = y - zz;
          p = (r * s - w) / h[nm*(m+1)+m] + h[nm*(m)+m+1];
          q = h[nm*(m+1)+m+1] - zz - r - s;
          r = h[nm*(m+2)+m+1];
          s = fabs(p) + fabs(q) + fabs(r);
          p = p / s;
          q = q / s;
          r = r / s;
          if (m == l) break;
          if (fabs(h[nm*(m)+m-1]) * (fabs(q) + fabs(r)) <= machep * fabs(p)
              * (fabs(h[nm*(m-1)+m-1]) + fabs(zz) + fabs(h[nm*(m+1)+m+1]))) break;
      }
//
      mp2 = m + 2;
//
      for (i = mp2; i <= en; i++) {
          h[nm*(i)+i-2] = 0.0;
          if (i == mp2) continue;
          h[nm*(i)+i-3] = 0.0;
      }
//     ********** Double qr step involving rows l to en and
//         columns m to en **********
      for (k = m; k <= na; k++) {
          notlas = (k != na);
          if (k == m) goto E170;
              p = h[nm*(k)+k-1];
              q = h[nm*(k+1)+k-1];
              r = 0.0;
              if (notlas) r = h[nm*(k+2)+k-1];
              x = fabs(p) + fabs(q) + fabs(r);
              if (x == 0.0) continue;
              p /= x;
              q /= x;
              r /= x;
E170:     s = sign(sqrt(p*p+q*q+r*r), p);
          if (k == m) goto E180;
              h[nm*(k)+k-1] = -s * x;
              goto E190;
E180:     if (l != m) h[nm*(k)+k-1] = -h[nm*(k)+k-1];
E190:     p += s;
          x = p / s;
          y = q / s;
          zz = r / s;
          q /= p;
          r /= p;
//     ********** Row modification **********
          for (j = k; j <= n; j++) {
              p = h[nm*(k)+j] + q * h[nm*(k+1)+j];
              if (!notlas) goto E200;
                  p += r * h[nm*(k+2)+j];
                  h[nm*(k+2)+j] -= p * zz;
E200:         h[nm*(k+1)+j] -= p * y;
              h[nm*(k)+j]   -= p * x;
          }
//
          j = min(en, k+3);
//     ********** Column modification **********
          for (i = 1; i <= j; i++) {
              p = x * h[nm*(i)+k] + y * h[nm*(i)+k+1];
              if (!notlas) goto E220;
                  p = p + zz * h[nm*(i)+k+2];
                  h[nm*(i)+k+2] -= p * r;
E220:         h[nm*(i)+k+1] -= p * q;
              h[nm*(i)+k]   -= p;
          }
//     ********** Accumulate transformations **********
          for (i = low; i <= igh; i++) {
              p = x * z[nm*(i)+k] + y * z[nm*(i)+k+1];
              if (!notlas) goto E240;
                  p += zz * z[nm*(i)+k+2];
                  z[nm*(i)+k+2] -= p * r;
E240:         z[nm*(i)+k+1] -= p * q;
              z[nm*(i)+k]   -= p;
          }
//
      }
//
      goto E70;
//     ********** One root found **********
E270: h[nm*(en)+en] = x + t;
      wr[en] = h[nm*(en)+en];
      wi[en] = 0.0;
      en = na;
      goto E60;
//     ********** Two roots found **********
E280: p = (y - x) / 2.0;
      q = p * p + w;
      zz = sqrt(fabs(q));
      h[nm*(en)+en] = x + t;
      x = h[nm*(en)+en];
      h[nm*(na)+na] = y + t;
      if (q < 0.0) goto E320;
//     ********** Real pair **********
      zz = p + sign(zz, p);
      wr[na] = x + zz;
      wr[en] = wr[na];
      if (zz != 0.0) wr[en] = x - w / zz;
      wi[na] = 0.0;
      wi[en] = 0.0;
      x = h[nm*(en)+na];
      s = fabs(x) + fabs(zz);
      p = x / s;
      q = zz / s;
      r = sqrt(p*p+q*q);
      p = p / r;
      q = q / r;
//     ********** Row modification **********
      for (j = na; j <= n; j++) {
          zz = h[nm*(na)+j];
          h[nm*(na)+j] = q * zz + p * h[nm*(en)+j];
          h[nm*(en)+j] = q * h[nm*(en)+j] - p * zz;
      }
//     ********** Column modification **********
      for (i = 1; i <= en; i++) {
          zz = h[nm*(i)+na];
          h[nm*(i)+na] = q * zz + p * h[nm*(i)+en];
          h[nm*(i)+en] = q * h[nm*(i)+en] - p * zz;
      }
//     ********** Accumulate transformations **********
      for (i = low; i <= igh; i++) {
          zz = z[nm*(i)+na];
          z[nm*(i)+na] = q * zz + p * z[nm*(i)+en];
          z[nm*(i)+en] = q * z[nm*(i)+en] - p * zz;
      }
//
      goto E330;
//     ********** Complex pair **********
E320: wr[na] = x + p;
      wr[en] = x + p;
      wi[na] = zz;
      wi[en] = -zz;
E330: en = enm2;
      goto E60;
//     ********** All roots found.  Backsubstitute to find
//         vectors of upper triangular form **********
E340: if (norm == 0.0) goto E1001;
//     ********** for en=n step -1 until 1 do -- **********
      for (nn = 1; nn <= n; nn++) {
          en = n + 1 - nn;
          p = wr[en];
          q = wi[en];
          na = en - 1;
          if (q < 0.0) goto E710;
          if (q == 0.0) goto E600;
              if (q > 0.0) continue;
//     ********** Real vector **********
E600:     m = en;
          h[nm*(en)+en] = 1.0;
          if (na == 0) continue;
//     ********** for i=en-1 step -1 until 1 do -- **********
          for (ii = 1; ii <= na; ii++) {
              i = en - ii;
              w = h[nm*(i)+i] - p;
              r = h[nm*(i)+en];
              if (m > na) goto E620;
                  for (j = m; j <= na; j++)
                      r += h[nm*(i)+j] * h[nm*(j)+en];
E620:         if (wi[i] >= 0.0) goto E630;
                  zz = w;
                  s = r;
                  continue;
E630:         m = i;
              if (wi[i] != 0.0) goto E640;
                  t = w;
                  if (w == 0.0) t = machep * norm;
                  h[nm*(i)+en] = -r / t;
                  continue;
//     ********** Solve real equations **********
E640:         x = h[nm*(i)+i+1];
              y = h[nm*(i+1)+i];
              q = (wr[i] - p) * (wr[i] - p) + wi[i] * wi[i];
              t = (x * s - zz * r) / q;
              h[nm*(i)+en] = t;
              if (fabs(x) <= fabs(zz)) goto E650;
                  h[nm*(i+1)+en] = (-r - w * t) / x;
                  continue;
E650:         h[nm*(i+1)+en] = (-s - y * t) / zz;
          }
//     ********** End real vector **********
          continue;
//     ********** Complex vector **********
E710:     m = na;
//     ********** Last vector component chosen imaginary so that
//                       eigenvector matrix is triangular **********
          if (fabs(h[nm*(en)+na]) <= fabs(h[nm*(na)+en])) goto E720;
              h[nm*(na)+na] = q / h[nm*(en)+na];
              h[nm*(na)+en] = -(h[nm*(en)+en] - p) / h[nm*(en)+na];
              goto E730;
E720:     z3 = complex(0.0,-h[nm*(na)+en]) / complex(h[nm*(na)+na]-p,q);
          h[nm*(na)+na] = real(z3);
          h[nm*(na)+en] = imag(z3);
E730:     h[nm*(en)+na] = 0.0;
          h[nm*(en)+en] = 1.0;
          enm2 = na - 1;
          if (enm2 == 0) continue;
//     ********** for i=en-2 step -1 until 1 do -- **********
          for (ii = 1; ii <= enm2; ii++) {
              i = na - ii;
              w = h[nm*(i)+i] - p;
              ra = 0.0;
              sa = h[nm*(i)+en];
//
              for (j = m; j <= na; j++) {
                  ra += h[nm*(i)+j] * h[nm*(j)+na];
                  sa += h[nm*(i)+j] * h[nm*(j)+en];
              }
//
              if (wi[i] >= 0.0) goto E770;
                  zz = w;
                  r = ra;
                  s = sa;
                  continue;
E770:         m = i;
              if (wi[i] != 0.0) goto E780;
                  z3 = complex(-ra,-sa) / complex(w,q);
                  h[nm*(i)+na] = real(z3);
                  h[nm*(i)+en] = imag(z3);
                  continue;
//     ********** Solve complex equations **********
E780:         x = h[nm*(i)+i+1];
              y = h[nm*(i+1)+i];
              vr = (wr[i] - p) * (wr[i] - p) + wi[i] * wi[i] - q * q;
              vi = (wr[i] - p) * 2.0 * q;
              if (vr == 0.0 && vi == 0.0)
                  vr = machep * norm * (fabs(w) + fabs(q) + fabs(x) + fabs(y) + fabs(zz));
              z3 = complex(x*r-zz*ra+q*sa,x*s-zz*sa-q*ra) / complex(vr,vi);
              h[nm*(i)+na] = real(z3);
              h[nm*(i)+en] = imag(z3);
              if (fabs(x) <= fabs(zz) + fabs(q)) goto E785;
                  h[nm*(i+1)+na] = (-ra - w * h[nm*(i)+na] + q * h[nm*(i)+en]) / x;
                  h[nm*(i+1)+en] = (-sa - w * h[nm*(i)+en] - q * h[nm*(i)+na]) / x;
                  continue;
E785:         z3 = complex(-r-y*h[nm*(i)+na],-s-y*h[nm*(i)+en]) / complex(zz,q);
              h[nm*(i+1)+na] = real(z3);
              h[nm*(i+1)+en] = imag(z3);
          }
//     ********** End complex vector **********
      }
//     ********** End back substitution.
//         Vectors of isolated roots **********
      for (i = 1; i <= n; i++) {
          if (i >= low && i <= igh) continue;
//
          for (j = i; j <= n; j++)
              z[nm*(i)+j] = h[nm*(i)+j];
//
      }
//     ********** Multiply by transformation matrix to give
//         vectors of original full matrix.
//         for j=n step -1 until low do -- **********
      for (jj = low; jj <= n; jj++) {
          j = n + low - jj;
          m = min(j, igh);
//
          for (i = low; i <= igh; i++) {
              zz = 0.0;
//
              for (k = low; k <= m; k++)
                  zz += z[nm*(i)+k] * h[nm*(k)+j];
//
              z[nm*(i)+j] = zz;
          }
      }
//
      goto E1001;
//     ********** Set error -- no convergence to an
//         eigenvalue after 30 iterations **********
E1000:
      ierr = en;
E1001:
      return ierr;
}
