/*
    Functii pentru calculul inversei unei matrici:
     - gaussm  calculeaza inversa unei matrici indexata de la 0;
     - gaussm1 calculeaza inversa unei matrici indexata de la 1.


    Autor: Mihai Lazarescu, 10.12.1991
*/

#include <alloc.h>
#include <math.h>

int gaussm(int nm, int n, double *a, float machep)
{
/*
    nm = dimensiunea matricii de inversat
    n  = ordinul matricii de inversat
    a  = matricea de inversat (la intrare)/inversa (la iesire)
    Matricea a se considera indexata de la 0.
    Rutina intoarce codul:
     - 0 pentru terminare corecta;
     - 1 pentru eroarea de memorie insuficienta;
     - 2 pentru eroarea de matrice singulara.
*/

    int     i, j, k;            /* Contoare */
    int     n2  = 2 * n;
    double  s   = 0.0;
    double  p   = 0.0;          /* Auxiliare */
    double  *at = NULL;         /* Matricea bordata */


    at = (double *)calloc(2*n*n, sizeof(double));
    if (at == NULL) return 1;    /* Eroare: memorie insuficienta */

    /* Copierea si bordarea matricii de inversat */
    for (i = 0; i < n; i++) {
        for (j = 0; j < n; j++) {
            at[n2*i+j]   = a[nm*i+j];
            at[n2*i+j+n] = 0.0;
        }
        at[n2*i+i+n] = 1.0;
    }

    /* Eliminarea */
    for (k = 0; k < n-1; k++) {             /* Contor linii pivot */
        for (i = k+1; i < n; i++) {         /* Contor linii urmatoare */
            if (fabs(at[n2*k+k]) < machep)  /* Matrice singulara */
                return 2;

            p = at[n2*i+k] / at[n2*k+k];
            for (j = k+1; j < n2; j++)      /* Contor coloane */
                at[n2*i+j] -= p * at[n2*k+j];
        }
    }

    /* Retrosubstitutia */
    for (k = n; k < n2; k++) {
        if (fabs(at[n2*(n-1)+n-1]) < machep)  /* Matrice singulara */
            return 2;

        at[n2*(n-1)+k] /= at[n2*(n-1)+n-1];
        for (i = n-2; i >= 0; i--) {
            s = at[n2*i+k];
            for (j = n-1; j > i; j--)
                s -= at[n2*i+j] * at[n2*j+k];
            at[n2*i+k] = s / at[n2*i+i];
        }
    }

    /* Copierea inversei in matricea initiala */
    for (i = 0; i < n; i++)
        for (j = 0; j < n; j++)
            a[nm*i+j] = at[n2*i+j+n];

    free(at);
    return 0;
}

int gaussm1(int nm, int n, double *a, float machep)
{
/*
    nm = dimensiunea matricii de inversat
    n  = ordinul matricii de inversat
    a  = matricea de inversat (la intrare)/inversa (la iesire)
    Matricea a se considera indexata de la 1.
    Rutina intoarce codul:
     - 0 pentru terminare corecta;
     - 1 pentru eroarea de memorie insuficienta;
     - 2 pentru eroarea de matrice singulara.
*/

    int     i, j, k;            /* Contoare */
    int     n2  = 2 * n;
    double  s   = 0.0;
    double  p   = 0.0;          /* Auxiliare */
    double  *at = NULL;         /* Matricea bordata */


    at = (double *)calloc(2*(n+1)*(n+1), sizeof(double));
    if (at == NULL) return 1;    /* Eroare: memorie insuficienta */

    /* Copierea si bordarea matricii de inversat */
    for (i = 1; i <= n; i++) {
        for (j = 1; j <= n; j++) {
            at[n2*i+j]   = a[nm*i+j];
            at[n2*i+j+n] = 0.0;
        }
        at[n2*i+i+n] = 1.0;
    }

    /* Eliminarea */
    for (k = 1; k < n; k++) {               /* Contor linii pivot */
        for (i = k+1; i <= n; i++) {        /* Contor linii urmatoare */
            if (fabs(at[n2*k+k]) < machep)  /* Matrice singulara */
                return 2;

            p = at[n2*i+k] / at[n2*k+k];
            for (j = k+1; j <= n2; j++)     /* Contor coloane */
                at[n2*i+j] -= p * at[n2*k+j];
        }
    }

    /* Retrosubstitutia */
    for (k = n+1; k <= n2; k++) {
        at[n2*n+k] /= at[n2*n+n];
        for (i = n-1; i > 0; i--) {
            s = at[n2*i+k];
            for (j = n; j > i; j--)
                s -= at[n2*i+j] * at[n2*j+k];
            at[n2*i+k] = s / at[n2*i+i];
        }
    }

    /* Copierea inversei in matricea initiala */
    for (i = 1; i <= n; i++)
        for (j = 1; j <= n; j++)
            a[nm*i+j] = at[n2*i+j+n];

    free(at);
    return 0;
}
