/*****************************************************************************

    Programul determina radacinile unui polinom folosind
    analiza modala a matricii companion asociate polinomului.

    La executia programului:
     - se introduce gradul polinomului;
     - se introduc coeficientii polinomului incepind cu coeficientul
       termenului de grad maxim;
     - se afiseaza:
       - solutiile ecuatiei: P(x) = 0 (valorile proprii ale matricii
         companion asociate polinomului).

    Autor: Mihai Lazarescu, 17.12.1991
    (C) L.M.N., Mihai Lazarescu.

*****************************************************************************/


#include <conio.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "hqr2.h"
#include "elmhes.h"
#include "eltran.h"


#define NM         (20+1)       // Dimensiunea maxima a problemei
                                // Indicele 0 nu se foloseste

typedef double tipmat;

tipmat  a[NM][NM];              // Matricea de analizat
tipmat  z[NM][NM];              // Matricea vectorilor proprii
float   an;                     // Coeficientul termenului de ordin maxim
int     n;                      // Ordinul problemei
tipmat  wr[NM];                 // Valorile proprii (partea reala)
tipmat  wi[NM];                 // Valorile proprii (partea imaginara)
int     low;
int     igh;
int     intchg[NM];              // Auxiliare



int citeste_int(void)
{
    const MAXINTCAR = 3;
	struct {
		char max;
		char nin;
		char sir[MAXINTCAR+1];
	} s = { MAXINTCAR+1 };

	cgets((char*)&s);
	printf("\n");
	return(atoi(s.sir));
}

float citeste_real(void)
{
    const MAXREALCAR = 20;
	struct {
		char max;
		char nin;
		char sir[MAXREALCAR+1];
	} s = { MAXREALCAR+1 };

	cgets((char*)&s);
	printf("\n");
    return(atof(s.sir));
}

int citeste_coef(int *n, tipmat *p, float *an)
{
    int     max = NM-1;

    do {
        cprintf("Gradul polinomului P (min. 1, max. %d, 0-iesire): ", max);
        *n = citeste_int();
        if (*n <= 0) return 1;          // Iesire
    } while (*n < 1 || *n > max);


    printf("\n\n");
    cprintf("Coeficientii polinomului, incepind cel al lui x^%d:\n\r", *n);
    for (; ; ) {
        printf("a%d = ", *n);
        if ((*an = citeste_real()) != 0.0)
            break;
        else
            cprintf("Coeficientul trebuie sa fie nenul!\n\r");
    }
    for (int i = *n; i >= 1; i--) {
        printf("a%d = ", i-1);
        p[NM*(*n)+i] = citeste_real();
    }
    return 0;
}

void companion(int n, tipmat *a, float an)
{
    int     i, j;

    for (i = 1; i < n; i++)
        for (j = 1; j <= n; j++)
            a[NM*i+j] = 0.0;

    for (i = 1; i <= n; i++)
        a[NM*n+i] /= -an;

    for (i = 2; i <= n; i++)
        a[NM*(i-1)+i] = 1.0;
}

void afis_titlu(char s[])
{
    char    subl[81] = "";              // Sublinierea
    int     len      = strlen(s);       // Lungimea titlului
    int     blen     = 0;               // Numarul de blancuri initiale

    while (s[blen++] == ' ') ;          // Numararea blancurilor
    blen--;
    memset(subl, ' ', blen);            // Inserarea blancurilor
    memset(subl+blen, '=', len-blen);   // Inserarea sublinierii
    subl[len+blen] = '\0';              // Terminatorul sirului
    cprintf("%s\n\r%s\n\r", s, subl);   // Afisarea titlului si sublinierii
}

void afis_stitlu(char s[])
{
    printf("%s\n", s);                  // Afisarea subtitlului
}

void afiseaza_rez(void)
{
    char    t[]   =
"                      Radacinile polinomului P";
    int     i;      // Contor

    afis_titlu(t);
    for (i = 1; i <= n; i++) {
        printf("\n%2d)\t", i);
        printf("% .2E %+.2E*i", wr[i], wi[i]);
        if (i % 20 == 0) {
            printf("\n\nApasati o tasta pentru continuare . . .");
            getch();
            clrscr();
            afis_titlu(t);
            printf("\n");
        }
    }
    if ((i-1) % 20 != 0) {
        printf("\n\nApasati o tasta pentru continuare . . .");
        getch();
    }
    clrscr();
}


void main(void)
{
    char    t[]    = "\
       DETERMINAREA RADACINILOR UNUI POLINOM FOLOSIND ANALIZA MODALA";
    char    st[]   = "\
     Programul determina radacinile polinomului introdus  de utilizator\n\
     folosind analiza modala a matricii companion asociate polinomului.";
    int     coderr = 0;
    tipmat  teps   = 1.0;
    tipmat  unu    = teps + 1.0;
    tipmat  machep = teps;         // Precizia de reprezentare a masinii

    // Determina precizia de reprezentare a masinii
    while (unu != 1.0) {
        machep  = teps;
        teps   /= 2.0;
        unu     = teps + 1.0;
    }

    highvideo();
    for (; ; ) {
        clrscr();
        afis_titlu(t);
        afis_stitlu(st);
        printf("\n\n\n");
        if (citeste_coef(&n, &a[0][0], &an) != 0) break;
        companion(n, &a[0][0], an);

        low = 1;
        igh = n;
        elmhes(NM, n, low, igh, &a[0][0], intchg);
        eltran(NM, n, low, igh, &a[0][0], intchg, &z[0][0]);
        coderr = hqr2(NM, n, low, igh, &a[0][0], wr, wi, &z[0][0], machep);
        clrscr();
        if (coderr == 0)
            afiseaza_rez();
        else {
            printf("\n\n\n\n\n\n\n\n\n\n\n\n\t\t\tEroare !");
            printf("\n\n\n\n\nApasati o tasta . . .");
            getch();
        }
    }
    clrscr();
    lowvideo();
}
