/****************************************************************************
 *                                                                          *
 *    Program pentru determinarea raspunsului unui circuit                  *
 *    format din rezistoare, bobine si condensatoare excitat                *
 *    cu un impuls.                                                         *
 *                                                                          *
 *                                                                          *
 *    Data:         31.10.1991                                              *
 *    Programator:  Mihai Lazarescu                                         *
 *                                                                          *
 *    (C) 1991 L.M.N., Mihai Lazarescu                                      *
 *                                                                          *
 ****************************************************************************/


/* Definirea constantelor */
#define NN      30		/* Nr. maxim de noduri (64 <=> 64Ko) */
#define NL      30		/* Nr. maxim de laturi */
#define NI     200      /* Nr. maxim de intervale */
#define NA    NI/2      /* Nr. maxim de armonici */
#define MAXEXPR 72      /* Nr. maxim de caractere in descrierea expresiei functiei */


/* Fisiere incluse */
#include <complex.h>
#include <conio.h>
#include <ctype.h>
#include <graphics.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "fourier.h"
#include "grafireu.h"
#include "gausscx.h"
#include "y_tab.h"


/* Variabile */
complex	Y[NN*NN];       /* Matricea admitantelor */
complex	Is_v[NN];       /* Vectorul curentilor de scurtcircuit */

complex	carfr[NA+1];    /* Caracteristica de frecventa a circuitului */

int		nnod;           /* Nr. de noduri */
int		nlat;           /* Nr. de laturi */
int		dimmat;         /* Dimensiunea matricei */

int		tip[NL];        /* Tipul laturii */
float	p[NL];          /* Parametrul laturii */

int		ni[NL];         /* Nr. nodului initial */
int		nf[NL];         /* Nr. nodului final */

int		nrlatin;        /* Nr. laturii de intrare */
int		nrlaties;       /* Nr. laturii de iesire */

int		tipin;          /* Tipul semnalului de intrare */
int		tipies;         /* Tipul semnalului de iesire */

double	a[NA+1];        /* Coeficientii in sin ai descompunerii Fourier */
double	b[NA+1];        /* Coeficientii in cos ai descompunerii Fourier */
double	x[NI+2];
double	y[NI+2];        /* Descrierea functiei/raspunsului */

char    sir[MAXEXPR+1]; /* Sirul de descriere a expresiei functiei */

int     n;              /* Numarul de intervale de discretizare */
int     m;              /* Numarul de armonici */
double   w;              /* Pulsatia fundamentalei */
double   T;              /* Perioada functiei */

const float PI = 3.14159265;


int citeste_int(void)
{
#define MAXINTCAR 3
	struct {
		char max;
		char nin;
		char sir[MAXINTCAR+1];
	} s = { MAXINTCAR+1 };

	cgets((char*)&s);
	printf("\n");
	return(atoi(s.sir));

#undef MAXINTCAR
}

float citeste_real(void)
{
#define MAXREALCAR 20
	struct {
		char max;
		char nin;
		char sir[MAXREALCAR+1];
	} s = { MAXREALCAR+1 };

	cgets((char*)&s);
	printf("\n");
	return(atof(s.sir));

#undef MAXREALCAR
}

int introd_parcirc(void)
/* Introduce parametrii circuitului */
/* Intoarce 1 daca este introdusa o valoare eronata, */
/* altfel intoarce 0 */
/* Sint initializate variabilele: */
/*	- nlat cu numarul de laturi ale circuitului */
/*	- nnod cu numarul de noduri ale circuitului */
{
	/* Introducerea numarului de laturi */
	do {
		cprintf("\n\rIntroduceti numarul de laturi (min. 2, max. %d, 0-iesire): ", NL);
		nlat = citeste_int();
		if (nlat == 0) return(1);
	} while (nlat < 2 || nlat > NL);

	/* Introducerea numarului de noduri */
	do {
		cprintf("\n\n\rIntroduceti numarul de noduri (min. 2, max. %d, 0-iesire): ", NN);
		nnod = citeste_int();
		if (nnod == 0) return(1);
	} while (nnod < 2 || nnod > NN);

	return(0);
}

/* Introduce laturile circuitului */
void introd_circ(void)
/* Sint initializate variabilele: */
/*	- ni[]  cu nodurile initiale ale laturilor */
/*	- nf[]  cu nodurile finale ale laturilor */
/*	- p[]   cu parametrii laturilor */
/*	- tip[] cu tipul laturilor */
{
	int		i;			/* Contor */

	for (i = 0; i < nlat; i++) {	/* Parcurge laturile circuitului */
		p[i] = 0;					/* Initializeaza parametrul laturii */

		/* Introduce nodurile laturii */
		cprintf("\n\rLatura numarul %d (din %d):", i+1, nlat);
		do {
			printf("\nNodul initial (1...%d): ", nnod);
			ni[i] = citeste_int();
			if (ni[i] > nnod || ni[i] < 1)		/* Nr. nodului este incorect */
				cprintf("\n\rNumarul nodului este incorect!\n\r");
			else {
				ni[i]--;		/* Pentru indexare in matrice */
				break;
			}
		} while (1);

		do {
			printf("Nodul final (1...%d)  : ", nnod);
			nf[i] = citeste_int();
			if (nf[i] > nnod || nf[i] < 1)		/* Nr. nodului este incorect */
				cprintf("\n\rNumarul nodului este incorect!\n\r");
			else {
				nf[i]--;		/* Pentru indexare in matrice */
				break;
			}
		} while (1);

		/* Introduce tipul laturii */
		printf("Tipul laturii (R - implicit, G, L sau C): ");
		fflush(stdin);
		tip[i] = toupper(getch());	/* Tipul laturii */

		/* Introduce parametrul laturii */
		do {
			switch(tip[i]) {
				case 'L':
					printf("%c\nInductivitatea: ", tip[i]);
					p[i] = citeste_real();
					if (p[i] == 0) {
						cprintf("\n\rNu se admit laturi de inductanta nula!\n\r");
						continue;
					}
					break;
				case 'C':
					printf("%c\nCapacitatea: ", tip[i]);
					p[i] = citeste_real();
					if (p[i] == 0) {
						cprintf("\n\rNu se admit laturi de capacitate nula!\n\r");
						continue;
					}
					break;
				case 'G':
					printf("%c\nConductanta: ", tip[i]);
					p[i] = citeste_real();
					break;

				default:
					tip[i] = 'R';
				case 'R':
					printf("%c\nRezistenta: ", tip[i]);
					p[i] = citeste_real();
					if (p[i] == 0) {
						cprintf("\n\rNu se admit laturi de rezistenta nula!\n\r");
						continue;
					}
			}
			break;
		} while (1);
	}
}

/* Introducerea intrarii si iesirii */
void introd_in_ies(void)
/* Sint initializate variabilele: */
/*	- nrlatin  cu numarul laturii de intrare */
/*	- nrlaties cu numarul laturii de iesire */
/*	- tipin    cu tipul semnalului de intrare */
/*	- tipies   cu tipul semnalului de iesire */
{
	/* Latura si semnalul de intrare */
	do {
		printf("\nIntroduceti numarul laturii de intrare: ");
		nrlatin = citeste_int();
	} while (nrlatin < 1 || nrlatin > nlat);
	nrlatin--;		/* Pentru indexare */
	if (tip[nrlatin] == 'G' && p[nrlatin] == 0.) {
		tipin = 'I';
		printf("Excitatie in curent.");
	}
	else {
		printf("Introduceti tipul semnalului de intrare (U, I): ");
		tipin = toupper(getch());
		if (tipin != 'I')
			tipin = 'U';
		printf("%c", (char)tipin);
	}

	/* Latura si semnalul de iesire */
	do {
		printf("\n\nIntroduceti numarul laturii de iesire: ");
		nrlaties = citeste_int();
	} while (nrlaties < 1 || nrlaties > nlat);
	nrlaties--;			/* Pentru indexare */
	if (tip[nrlaties] == 'G' && p[nrlaties] == 0.) {
		tipies = 'U';
		printf("Iesire in tensiune.");
	}
	else {
		printf("Introduceti tipul semnalului de iesire (U, I): ");
		tipies = toupper(getch());
		if (tipies != 'I')
			tipies = 'U';
		printf("%c", (char)tipies);
	}
}

/* Introducerea datelor */
/* Intoarce o valoare nenula daca este o cerere de HELP */
int estehelp(char *s)
{
	return(strstr(s, "?") != NULL);
}

/* Verifica sintactic expresia transmisa ca parametru */
int veriffunc(char sir[])
{
    char sirex[MAXEXPR + 5];        /* 4 car. rezerva pt. valoarea lui t */
    int coderr;                     /* Cod de eroare */

    if (estehelp(sir)) {            /* Se cere HELP-ul parserului */
        clrscr();
        parser("?", &coderr, 1);    /* Se afiseaza HELP-ul */
    }
    sprintf(sirex, "t=0;");         /* Introduce valoarea argumentului expresiei */
    strcat(sirex, sir);             /* Adauga expresia */
    parser(sirex, &coderr, 1);      /* Verifica sintactic expresia */
    return(coderr);                 /* Intoarce codul de eroare */
}

/* Evalueaza expresia in punctul x transmis ca parametru */
double evalfunc(char sir[], double t, int *coderr)
{
    char sirex[MAXEXPR + 31];       /* 30 de car. rezerva pt. val. lui t */

    sprintf(sirex, "t=%.20e;", t);  /* Introduce valoarea argumentului expresiei */
    strcat(sirex, sir);             /* Adauga expresia */
    return(parser(sirex, coderr, 0));  /* Evalueaza expresia */
}

/* Introduce expresiile functiei pe intervalele [0..T/2] si [T/2..T] */
int fourier_introdfunc(char s[])
{
    struct {
        char max;                /* Numarul maxim de caractere permis */
        char nin;                /* Numarul de caractere introduse */
        char sir[MAXEXPR + 1];    /* Sirul introdus */
    } sstr;

    sstr.max = MAXEXPR;
    do {
        printf("\n\nIntroduceti expresia functiei f (? = Help):");
        printf("\nf(t) = ");
        fflush(stdin);                        /* Curata bufferul */
        cgets((char *)&sstr);                 /* Citeste expresia */
        if (sstr.nin == 0) return(1);
    } while(veriffunc(sstr.sir) != 0);        /* Verifica sintaxa expr. */
    strcpy(s, sstr.sir);

    return(0);
}

/* Introducerea perioadei */
int fourier_introd_perioada(void)
{
    do {
        printf("\n\nTimpul de simulare: ");
        T = citeste_real();
    } while (T < 0.0);

    if (T == 0.0) return(1);

    return(0);
}

/* Introducerea numarului de intervale */
int introd_nrinterv(void)
{
    do {
        printf("\nNumarul de intervale (min. 1, max. %d): ", NI);
        n = citeste_int();
        if (n == 0) return(1);        /* Iesire */
    } while (n < 1 || n > NI);

    return(0);
}

int fourier_introddate(void)
{
    /* Introducerea functiei */
    if (fourier_introdfunc(sir))
        return(1);

    /* Introducerea perioadei */
    if (fourier_introd_perioada())
        return(1);
    w = PI / T;

    /* Introducerea numarului de intervale */
    if (introd_nrinterv())
        return(1);

    /* Introducerea numarului de armonici */
    m = n/2;

    return(0);
}

int introddate(void)
/* Introducerea datelor circuitului */
{
	clrscr();

	/* Titlul */
	cprintf(    "                      ANALIZA IN REGIM TRANZITORIU");
	cprintf("\n\r                      ============================");
    printf( "\n\n         Programul determina raspunsul  in regim  tranzitoriu al");
    printf(   "\n         unui  circuit  liniar  de  curent  alternativ  folosind");
    printf(   "\n         descompunerea in serie Fourier a semnalului de intrare.");

	printf("\n\n\n\n");

	/* Introducerea parametrilor circuitului */
	if (introd_parcirc())
		return(1);

	/* Introducerea laturilor circuitului */
	clrscr();
	introd_circ();

	/* Specificarea intrarii si iesirii */
	clrscr();
	introd_in_ies();

	/* Introducerea excitatiei */
	clrscr();
	if (fourier_introddate())
		return(1);

	return(0);
}

/* Calculeaza functia */
void fourier_calculfunc(char *sir, int n, double xmin, double xmax, double x[], double r[])
{
	double	xc;		/* Punctul curent */
	double	dx;		/* Pasul lui x */
	int		i;		/* Contor */
    int     coderr; /* Cod de eroare */

	xc = xmin;
	dx = (xmax - xmin) / n;
	for (i = 0; i <= n; i++) {
		r[i] = evalfunc(sir, xc, &coderr);
		x[i] = xc;
		xc  += dx;
	}
}

void discr_functie(void)
{
	/* Liniarizarea functiei pe portiuni */
	fourier_calculfunc(sir, n, 0.0, T, x, y);
}

int rezolvare(double omega)
/* Rezolva circuitul */
{
	int		n1, n2;		/* Nodul initial si final al laturii curente */
	complex yl;			/* Impedanta curenta */

	int     i;
	int     j;			/* Contoare */
	int     ier;		/* Indicator de eroare */


	dimmat = nnod - 1;	/* Dimensiunea matricei */

	/* Initializarea matricilor: Y si Is_v */
	for (i = 0; i < dimmat; i++) {
		Is_v[i] = complex(0., 0.);
		for (j = 0; j < dimmat; j++)
			Y[NN*i +j] = complex(0., 0.);
	}

	/* Calculeaza matricele Y si Is */
	for (i = 0; i < nlat; i++) {
		n1 = ni[i];
		n2 = nf[i];
		switch (tip[i]) {
			case 'L': yl = complex(0., -1. / (p[i]*omega));
				break;
			case 'C': yl = complex(0., p[i]*omega);
				break;
			case 'G': yl = complex(p[i], 0.);
				break;
			default:
			case 'R': yl = complex(1. / p[i], 0.);
		}
		if (n1 != dimmat)
			Y[NN*n1 + n1] += yl;
		if (n2 != dimmat)
			Y[NN*n2 + n2] += yl;
		if (n1 != dimmat && n2 != dimmat) {
			Y[NN*n1 + n2] -= yl;
			Y[NN*n2 + n1] -= yl;
		}

		if (i == nrlatin)
			if (tipin == 'U') {
				Is_v[n1] = -yl;
				Is_v[n2] =  yl;
			}
			else {
				Is_v[n1] = complex(-1., 0.);
				Is_v[n2] = complex( 1., 0.);
			}
	}

	/* Rezolvarea sistemului */
	ier = gauss_cplx(NN, dimmat, Y, Is_v);
	Is_v[dimmat] = complex(0., 0.);		/* Potentialul nodului de referinta */

	return(ier);
}

int calculrez(int nrfrec, double omega)
/* Calculeaza rezultatele */
{
	int		n1, n2;		/* Nodul initial si final al laturii curente */
	complex yl;			/* Impedanta curenta */
	complex	u;			/* Tensiunea pe latura */
	complex	i;			/* Curentul prin latura */

	int		ier;		/* Indicator de eroare */


	/* Rezolvarea circuitului */
	ier = rezolvare(omega);

	/* Extragerea rezultatelor */
	switch (tip[nrlaties]) {
		case 'L': yl = complex(0., -1. / (p[nrlaties]*omega));
			break;
		case 'C': yl = complex(0., p[nrlaties]*omega);
			break;
		case 'G': yl = complex(p[nrlaties], 0.);
			break;
		default:
		case 'R': yl = complex(1. / p[nrlaties], 0.);
	}

	n1 = ni[nrlaties];
	n2 = nf[nrlaties];				/* Nodurile initial si final ale laturii */

	u = Is_v[n1] - Is_v[n2];		/* Tensiunea la bornele laturii */
	i = u * yl;						/* Curentul prin latura */
	if (nrlaties == nrlatin)
		if (tipin == 'I')
			i = complex(1., 0.);
		else
			i = (u + complex(1., 0.)) * yl;

	/* Memorarea rezultatelor */
	carfr[nrfrec] = (tipies == 'I') ? i : u;	/* Carac. de frecv. */

	return(ier);
}

void spectru_ies(int m, double a[], double b[], double w)
{
	float	val[2][N];		/* Valorile de reprezentat grafic */
	float	modul[NA+1];	/* Modulul armonicilor */
	int		i;				/* Contor */
	extern	hold;			/* 1 - graficul se pastreaza */

	w /= 2*PI;			/* Frecventa */

	subplot(2, 2, 4);
	/* Determinarea modulului armonicilor */
	val[1][0] = modul[0] = 0.;		/* c.c. */
	for (i = 1; i <= m; i++)
		modul[i] = sqrt(a[i]*a[i] + b[i]*b[i]);
	for (i = 1; i < m; i++) {
		val[0][i-1] = w*i;
		val[1][i-1] = (modul[i] + modul[i+1]) / 2.;
	}

	plot(1, 					/* Numarul de grafice */
		m-1,						/* Numarul de puncte */
		val);					/* val[0] = x; restul: cite un y */
	grid();
	title("Spectrul iesirii");
	label_x("Frecventa");
	label_y("Modul");
}

/* Calculeaza raspunsul circuitului */
void raspunscirc(
	complex carfr[],
	int m, double a[], double b[],
	int n, double x[], double y[],
	double w)
{
	int		i;		/* Contor */
	complex	z;

	for (i = 0; i <= m; i++) {
		z    = carfr[i] * complex(a[i], b[i]);
		a[i] = real(z);
		b[i] = imag(z);
	}

	spectru_ies(m, a, b, w);
	eval_Fourier(
		n, x, y,
		m, a, b,
		w
	);
}

void afis_functie(int n, double x[], double y[])
{
	float	val[2][N];		/* Valorile de reprezentat grafic */
	int		i;				/* Numarul frecventei curente */

	for (i = 0; i < n; i++) {
		val[0][i] = x[i];
		val[1][i] = y[i];
	}
	subplot(2, 2, 1);
	plot(1,                     /* Numarul de grafice */
		n,                      /* Numarul de puncte */
		val);                   /* val[0] = x; restul: cite un y */
	grid();
	title("Semnalul de intrare");
	label_x("Timp");
	label_y("Valoare");
}

void afis_funct_aprox(int n, double x[], double y[], int m, double a[], double b[], double w)
{
    float    val[2][N];        /* Valorile de reprezentat grafic */
    int      i;                /* Numarul frecventei curente */

    eval_Fourier(n, x, y, m, a, b, w);
    for (i = 0; i < n; i++) {
        val[0][i] = x[i];
        val[1][i] = y[i];
    }
    subplot(2, 2, 1);
    plot(1,                     /* Numarul de grafice */
         n,                     /* Numarul de puncte */
         val);                  /* val[0] = x; restul: cite un y */
    grid();
    title("Aproximatia intrarii");
    label_x("Timp");
    label_y("Valoare");
}

void spectru_in(int m, double a[], double b[], double w)
{
    float   val[2][N];      /* Valorile de reprezentat grafic */
    float   modul[NA+1];    /* Modulul armonicilor */
    int     i;              /* Contor */

    w /= 2*PI;              /* Frecventa */

    subplot(2, 2, 2);

    /* Determinarea modulului armonicilor */
    val[1][0] = modul[0] = 0.;		/* c.c. */
    for (i = 1; i <= m; i++)
	    modul[i] = sqrt(a[i]*a[i] + b[i]*b[i]);
    for (i = 1; i < m; i++) {
	    val[0][i-1] = w*i;
	    val[1][i-1] = (modul[i] + modul[i+1]) / 2.;
    }

    plot(1,                     /* Numarul de grafice */
         m-1,                   /* Numarul de puncte */
         val);                  /* val[0] = x; restul: cite un y */
    grid();
    title("Spectrul intrarii");
    label_x("Frecventa");
    label_y("Modul");
}

void afisrez(int n, double x[], double y[])
/* Afisarea rezultatelor */
{
	float	val[2][N];		/* Valorile de reprezentat grafic */
	int		i;				/* Numarul frecventei curente */

	for (i = 0; i < n; i++) {
		val[0][i] = x[i];
		val[1][i] = y[i];
	}
	subplot(2, 2, 3);
	plot(1,                     /* Numarul de grafice */
		n,                      /* Numarul de puncte */
		val);                   /* val[0] = x; restul: cite un y */
	grid();
	title("Aproximatia iesirii");
	label_x("Timp");
	label_y("Valoare");
}


void main(void)
{
	double	omega;				/* Frecventa curenta */
	int		ier;				/* Indicator de eroare */


	initial();
	restorecrtmode();			/* Pentru pachetul grafic */

	highvideo();
	do {
		int armcrt;				/* Numarul armonicii curente */

		if (introddate())		/* Introducerea datelor */
			break;				/* Terminarea executiei */

		setgraphmode(getgraphmode());

		/* Discr. functiei */
		discr_functie();
		afis_functie(n+1, x, y);

		/* Calculul coeficientilor Fourier ai functiei introduse */
		Fourier_lpp(n+1, x, y, m, a, b, w);
		spectru_in(m, a, b, w);
        afis_funct_aprox(n+1, x, y, m, a, b, w);

		omega = w;
		for (armcrt = 0; armcrt <= m; armcrt++) {
			double	w;

			if (armcrt == 0)
				w = omega / 100.;
			else
				w = omega * armcrt;
			ier = calculrez(armcrt, w);		/* Rezolvarea circ. */
			if (ier) {
				cprintf("\n\n\rCircuit prost formulat!\n\r");
				break;		/* Eroare la rezolvare */
			}
		}
		if (ier) continue;	/* A fost o eroare */

		/* Calculul raspunsului circuitului */
		raspunscirc(
			carfr,
			m, a, b,
			n+1, x, y,
			w);                     /* Raspunsul circuitului */

		/* Afisarea rezultatelor */
		afisrez(n+1, x, y);           /* Repr. graf. a rez. */

		restore();

	} while(1);
	lowvideo();
}
