#include <alloc.h>
#include <complex.h>
#include <conio.h>
#include <ctype.h>
#include <graphics.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "elmhes.h"
#include "eltran.h"
#include "gauss.h"
#include "grafireu.h"
#include "hqr2.h"
#include "matinv.h"


#define NN         30           // Numarul maxim de noduri
#define NL         30           // Numarul maxim de laturi
#define NB          5           // Numarul maxim de bobine
#define NT        200           // Numarul de intervale de timp de simulare
#define NCT         4           // Numarul de constante de timp pe care se face simularea
#define NM         NN           // Dimensiunea maxima a problemei
                                // Indicele 0 nu se foloseste
#define MINVAL  1E-30           // Valoarea minima considerata nenula

typedef double tipmat;


tipmat  r[NB+1][NB+1];          // Matricea rezistentelor multiportului
tipmat  fr[NB+1][NB+1];         // Functia de matrice

int     nnod;                   // Numarul de noduri
int     nlat;                   // Numarul de laturi
int     nbob;                   // Numarul de bobine

int     tip[NL];                // Tipul laturii
float   p[NL];                  // Parametrul laturii
tipmat  c[NB];                  // Conditia initiala a laturii
float   l[NB];                  // Parametrii bobinelor

int     ni[NL];                 // Numarul nodului initial
int     nf[NL];                 // Numarul nodului final

tipmat  z[NB+1][NB+1];          // Matricea vectorilor proprii
tipmat  wr[NB+1];               // Valorile proprii (partea reala)
tipmat  wi[NB+1];               // Valorile proprii (partea imaginara)
int     intchg[NB+1];           // Auxiliara
tipmat  iz[NB+1][NB+1];         // Inversa matricei vectorilor proprii

float   val[NB][NT];            // Valorile de reprezentat grafic



int citeste_int(void)
{
    const MAXINTCAR = 3;
    struct {
        char max;
        char nin;
        char sir[MAXINTCAR+1];
    } s = { MAXINTCAR+1 };

    cgets((char*)&s);
    printf("\n");
    return(atoi(s.sir));
}

float citeste_real(void)
{
    const MAXREALCAR = 20;
    struct {
        char max;
        char nin;
        char sir[MAXREALCAR+1];
    } s = { MAXREALCAR+1 };

    cgets((char*)&s);
    printf("\n");
    return(atof(s.sir));
}

void afis_titlu(char s[])
{
    char    subl[81] = "";              // Sublinierea
    int     len      = strlen(s);       // Lungimea titlului
    int     blen     = 0;               // Numarul de blancuri initiale

    while (s[blen++] == ' ') ;          // Numararea blancurilor
    blen--;
    memset(subl, ' ', blen);            // Inserarea blancurilor
    memset(subl+blen, '=', len-blen);   // Inserarea sublinierii
    subl[len+blen] = '\0';              // Terminatorul sirului
    cprintf("%s\n\r%s\n\r", s, subl);   // Afisarea titlului si sublinierii
}

void afis_stitlu(char s[])
{
    printf("%s\n", s);                  // Afisarea subtitlului
}

void asteapta(int lin_lib = 0)
{
    while (lin_lib-- > 0)
        printf("\n");

    printf("Apasati o tasta pentru continuare . . . ");
    getch();
}

void eroare(void)
{
    clrscr();
    printf("\n\n\n\n\n\n\n\n\n\n\n\n\t\t\t\t");
    cprintf("E R O A R E   !");
    asteapta(10);
}

int introd_parcirc(void)
// Introduce parametrii circuitului
// Intoarce 1 daca este introdusa o valoare eronata,
// altfel intoarce 0
// Sint initializate variabilele:
//  - nlat cu numarul de laturi ale circuitului
//  - nnod cu numarul de noduri ale circuitului
{
    // Introducerea numarului de laturi
    do {
        cprintf("\n\rIntroduceti numarul de laturi (min. 2, max. %d, 0-iesire): ", NL);
        nlat = citeste_int();
        if (nlat == 0) return(1);
    } while (nlat < 2 || nlat > NL);

    // Introducerea numarului de noduri
    do {
        cprintf("\n\rIntroduceti numarul de noduri (min. 2, max. %d, 0-iesire): ", NN);
        nnod = citeste_int();
        if (nnod == 0) return(1);
    } while (nnod < 2 || nnod > NN);

    return(0);
}

// Introduce laturile circuitului
void introd_circ(void)
// Sint initializate variabilele:
//  - ni[]  cu nodurile initiale ale laturilor
//  - nf[]  cu nodurile finale ale laturilor
//  - p[]   cu parametrii laturilor
//  - tip[] cu tipul laturilor
{
    nbob = 0;               // Initializeaza numarul de bobine;
    for (int i = 0; i < nlat; i++) {    // Parcurge laturile circuitului
        p[i] = 0.0;         // Initializeaza parametrul laturii

        // Introduce nodurile laturii
        cprintf("\n\rLatura numarul %d (din %d):", i+1, nlat);
        printf("\nNodul initial (1...%d): ", nnod);
        ni[i] = citeste_int();
        if (ni[i] > nnod || ni[i] < 1) {    // Nr. nodului este incorect
            cprintf("\n\rNumar de nod incorect!\n\r");
            i--;
            continue;
        }
        ni[i]--;        // Pentru indexare in matrice

        printf("Nodul final (1...%d)  : ", nnod);
        nf[i] = citeste_int();
        if (nf[i] > nnod || nf[i] < 1) {    // Nr. nodului este incorect
            cprintf("\n\rNumar de nod incorect!\n\r");
            i--;
            continue;
        }
        nf[i]--;        // Pentru indexare in matrice

        // Introduce tipul laturii
        printf("Tipul laturii (R - implicit sau L): ");
        tip[i] = toupper(getch());     // Tipul laturii

        // Introduce parametrul laturii
        switch(tip[i]) {
            case 'L':
                if (nbob < NB) {
                    printf("%c\nInductivitatea: ", tip[i]);
                    l[nbob] = p[i] = citeste_real();
                    if (p[i] == 0) {
                        cprintf("\n\rNu se admit laturi de inductanta nula!\n\r");
                        i--;
                        continue;
                    }
                    else {
                        printf("Curentul initial prin bobina: ");
                        c[nbob] = citeste_real();
                        nbob++;          // O noua bobina a fost introdusa
                    }
                }
                else {
                    cprintf("\n\rNumarul maxim de bobine (%d) a fost atins deja!\n\r", NB);
                    i--;
                    continue;
                }
                break;

            default:
                tip[i] = 'R';
            case 'R':
                printf("%c\nRezistenta: ", tip[i]);
                p[i] = citeste_real();
                if (p[i] == 0) {
                    cprintf("\n\rNu se admit laturi de rezistenta nula!\n\r");
                    i--;
                    continue;
                }
        }
    }
}

void afis_ct(int dim, tipmat ct[], float machep)
// ct este indexat de la 1
{
    char    t[] =
"                    Constantele de timp ale circuitului";
    int     i;          // Contor

    clrscr();
    afis_titlu(t);
    printf("\n\n\n");
    if (dim >= 1)
        for (i = 1; i <= dim; i++) {
            tipmat  c = -ct[i];     // Constanta de timp curenta

            if (fabs(c) < machep) c = machep;
            c = 1.0 / c;
            printf("%d)\t%.2E\n", i, (float)c);
        }
    else
        printf("Circuitul este rezistiv, nu are constante de timp.");

    asteapta(3);
    clrscr();
}

void inmul_mat(
     int nma, int na, int ma, tipmat *a,
     int nmb,         int mb, tipmat *b,
     int nmr,                 tipmat *r)
// Inmulteste doua matrici:
//  - A de ordin na, ma si cu dimensiunea liniei nma;
//  - B de ordin ma, mb si cu dimensiunea liniei nmb;
//  - R = A * B, cu dimensiunea liniei nmr.
// Matricile sint indexate de la 1.
{
    int     i, j, k;
    tipmat  s;

    for (k = 1; k <= mb; k++)           // Parcurge coloanele matricii B
        for (i = 1; i <= na; i++) {     // Parcurge liniile matricii A
            s = 0.0;
            for (j = 1; j <= ma; j++)    // Parcurge coloanele matricii A
                s += a[nma*i+j] * b[nmb*j+k];
            r[nmr*i+k] = s;
        }
}

typedef double (*functie)(double);

int func_matr(
    tipmat  *fa,            // Matricea in care se depune functia calculata
    tipmat  *z,             // Matricea vectorilor proprii ai matricii
    tipmat  *iz,            // Inversa matricii z
    tipmat  wr[],           // Vectorul valorilor proprii ale matricii
    int     nm,             // Dimensiunea maxima a matricilor
    int     n,              // Ordinul matricilor
    float   t,              // Argumentul functiei de matrice de calculat
    functie f               // Functia de calculat
    )
// Matricile sint indexate incepind cu 1.
{
    tipmat  *d   = NULL;
    tipmat  *f1a = NULL;

    d   = (tipmat *)calloc((n+1)*(n+1), sizeof(tipmat));
    if (d == NULL) return 1;                    // Eroare
    f1a = (tipmat *)calloc((n+1)*(n+1), sizeof(tipmat));
    if (f1a == NULL) {
        free(d);
        return 1;           // Eroare
    }

    for (int i = 1; i <= n; i++)
        d[nm*i+i] = f(wr[i]*t);

    inmul_mat(nm, n, n, z  , nm, n, d , nm, f1a);
    inmul_mat(nm, n, n, f1a, nm, n, iz, nm, fa);

    free(d);
    free(f1a);

    return 0;
}

int rezolvare(float machep)
// Rezolva circuitul
{
    int     n1     = 0,
            n2     = 0;     // Nodul initial si final al laturii curente
    int     n1i    = 0,
            n2i    = 0;     // Nodul initial si final al bobinei curente (index i)
    int     n1j    = 0,
            n2j    = 0;     // Nodul initial si final al bobinei curente (index j)

    tipmat  gl     = 0.0;   // Conductanta curenta

    int     i      = 0,
            j      = 0,
            ii     = 0,
            jj     = 0;     // Contoare

    float   t      = 0.0;   // Timpul curent
    float   dt     = 0.0;   // Pasul de timp de simulare
    tipmat  x[NB+1][2];     // Raspunsul circuitului la pasul de simulare curent

    int     coderr = 0;     // Cod de eroare

    tipmat  *g     = NULL;  // Matricea admitantelor
    tipmat  *x0    = NULL;  // Vectorul conditiilor initiale

    if (nbob == 0) {        // Circuitul nu contine bobine
        afis_ct(nbob, wr, machep);
        return 0;
    }

    // Alocarea si initializarea matricii G[NN][NN]
    g = (tipmat *)calloc(NN*NN, sizeof(tipmat));
    if (g == NULL) return 1;    // Eroare la alocare

    // Calculeaza matricea G
    for (i = 0; i < nlat; i++) {
        if (tip[i] == 'L') continue;

        n1 = ni[i], n2 = nf[i];
        gl = 1.0 / p[i];

        if (n1 != nnod-1)
            g[NN*n1+n1] += gl;
        if (n2 != nnod-1)
            g[NN*n2+n2] += gl;
        if (n1 != nnod-1 && n2 != nnod-1) {
            g[NN*n1+n2] -= gl;
            g[NN*n2+n1] -= gl;
        }
    }

    coderr = gaussm(NN, nnod-1, g, MINVAL);     // Inversarea matricii G
    if (coderr != 0) return coderr;             // Eroare la inversare

    // Prelucreaza matricea inversata si determina
    // matricea rezistentelor multiportului.
    for (jj = j = 0; j < nlat; j++) {
        if (tip[j] != 'L') continue;

        // Elementul curent este o bobina
        jj++;
        n1j = ni[j], n2j = nf[j];
        for (ii = i = 0; i < nlat; i++) {
            if (tip[i] != 'L') continue;

            // Elementul curent este o bobina
            ii++;
            n1i = ni[i], n2i = nf[i];
            r[ii][jj] = g[NN*n1i+n1j] + g[NN*n2i+n2j] - g[NN*n1i+n2j] - g[NN*n2i+n1j];
        }
    }
    free(g);

    // Calculeaza matricea sistemului
    for (i = 1; i <= nbob; i++)
        for (j = 1; j <= nbob; j++)
            r[i][j] /= -l[j-1];

    // Analiza modala
    elmhes(NB+1, nbob, 1, nbob, &r[0][0], intchg);
    eltran(NB+1, nbob, 1, nbob, &r[0][0], intchg, &z[0][0]);
    coderr = hqr2(NB+1, nbob, 1, nbob, &r[0][0], wr, wi, &z[0][0], machep);
    if (coderr != 0) return coderr;

    // Afisarea constantelor de timp
    afis_ct(nbob, wr, machep);

    // Initializarea matricii iz pentru calcularea inversei
    for (i = 1; i <= nbob; i++)
        for (j = 1; j <= nbob; j++)
            iz[i][j] = z[i][j];

    coderr = gaussm1(NB+1, nbob, &iz[0][0], MINVAL);    // Inversarea matricii iz
    if (coderr != 0) return coderr;                     // Eroare

    // Cautarea constantei de timp maxime
    for (i = 1; i <= nbob; i++)
        if (wr[i] != 0 && (float)NCT / -wr[i] > t)
            t = (float)NCT / -wr[i];
    dt = t / NT;

    // Initializarea vectorului conditiilor initiale (indexat de la 1)
    x0 = (tipmat *)calloc(2*(NB+1), sizeof(tipmat));
    for (i = 1; i <= nbob; i++)
        x0[2*i+1] = c[i-1];

    // Simularea regimului tranzitoriu
    for (t = 0.0, i = 1; i <= NT; i++, t += dt) {
        coderr = func_matr(&fr[0][0], &z[0][0], &iz[0][0], wr, NB+1, nbob, t, exp);
        if (coderr != 0) return coderr;
        inmul_mat(NB+1, nbob, nbob, &fr[0][0], 2, 1, x0, 2, &x[0][0]);
        val[0][i-1] = t;
        for (j = 1; j <= nbob; j++)
            val[j][i-1] = x[j][1];
    }
    free(x0);

    return coderr;
}

void afis_rezult(void)
{
    if (nbob == 0) return;      // Nu sint bobine, nu e nimic de reprezentat

    setgraphmode(getgraphmode());

    plot(nbob,                  // Numarul de grafice
         NT,                    // Numarul de puncte
         val);                  // val[0] = x; restul: cite un y
    grid();
    title("Variatia in timp a variabilelor de stare");
    label_x("Timp");
    label_y("Curent");
    restore();
}


void main(void)
{
    char    t[]    = "\
            SIMULAREA REGIMULUI TRANZITORIU FOLOSIND ANALIZA MODALA";
    char    st[]   = "\
    Programul determina constantele de timp si simuleaza regimul tranzitoriu\n\
    al  unui circuit  pasiv, format  din rezistoare  si bobine liniare, fara\n\
    cuplaje mutuale. Sint afisate constantele de timp ale circuitului precum\n\
    si evolutia in timp a curentilor prin bobine.";
    int     coderr = 0;
    tipmat  teps   = 1.0;
    tipmat  unu    = teps + 1.0;
    tipmat  machep = teps;         // Precizia de reprezentare a masinii


    initial();
    restorecrtmode();           // Pentru pachetul grafic

    // Determina precizia de reprezentare a masinii
    while (unu != 1.0) {
        machep  = teps;
        teps   /= 2.0;
        unu     = teps + 1.0;
    }

    highvideo();
    for (; ; ) {
        clrscr();
        afis_titlu(t);
        afis_stitlu(st);

        printf("\n");
        if (introd_parcirc() != 0) break;

        clrscr();
        introd_circ();

        clrscr();
        coderr = rezolvare(machep);
        if (coderr != 0) {
            eroare();
            continue;
        }

        clrscr();
        afis_rezult();
    }
    clrscr();
    lowvideo();
}
