
#line 31 "\LMT\YACC\FUNC.Y"

/* Definitii generale */
#define TRUE          1
#define FALSE         0                   /* Rezultatele expresiilor logice */
#define MAXNUME      10                   /* Lungimea maxima a unui identificator */
#define MAXVAR       10                   /* Numarul maxim de variabile */
#define NCOLMON      79                   /* Numarul de coloane pe ecran - 1 */
#define NCOLHELP      6                   /* Nr. de coloane pe care se afiseaza help-ul */
#define SPSENSIB      YYP_vedesp = TRUE   /* Analiza lexicala e sensibila la spatii */
#define SPINSENSIB    YYP_vedesp = FALSE  /* Analiza lexicala e insensibila la spatii */
#define COMPARA(x, y) (strcmp((x), (y)) == 0)  /* TRUE daca sirul x e egal cu y */
#define SETERR(cod)   (YYP_errcod = cod)  /* Seteaza codul erorii */

/* Definitiile operatorilor */
#define ATRIB    '='
#define PLUS     '+'
#define MINUS    '-'
#define INMUL    '*'
#define IMPAR    '/'
#define PUTERE   '^'

/* Definitiile parantezelor */
#define PARDES   '('
#define PARINC   ')'

/* Definitiile semnelor speciale */
#define SEPINSTR ';'
#define REALSEPZ '.'
#define REALEXP  'e'
#define HELP     '?'

/* Definitii de tip */
union YYSTACKTYPE {
    char   c;
    int    i;
    double d;
    char   s[MAXNUME];
};
#define YYSTYPE union YYSTACKTYPE       /* Tipul stivei parserului */
typedef struct {
    char   s[MAXNUME];
    double x;
} VARIAB;              /* Tipul unei variabile */

/* Variabilele globale */
int    YYP_versint;        /* 1 pentru o verificare sintactica, fara eval. */
double YYP_rezultat;       /* Rezultatul evaluarii expresiei */
int    YYP_evaluat;        /* TRUE daca a fost evaluata expresia */
VARIAB YYP_var[MAXVAR];    /* Tabela de variabile */
int    YYP_findex;         /* Indexul functiei apelate curent */
int    YYP_vedesp;         /* TRUE - analiza lexicala e sensibila la spatii */
int    YYP_index;          /* Pozitia curenta de citire a expresiei */
int    YYP_errindex;       /* Pozitia erorii detectate */
int    YYP_errcod;         /* Codul mesajului de eroare */
int    YYP_existaerr;      /* TRUE daca apare cel putin o eroare */
int    YYP_existamaterr;   /* TRUE daca apare cel putin o eroare matematica */
char   *YYP_errstr;        /* Se afiseaza in continuarea mesajului de eroare */

/* Prototipurile functiilor matematice */
#include <math.h>
double YYP_step(double x);
double YYP_sgn(double x);


/* Numele sub care sint recunoscute functiile de catre parser */
char *numefunc[] = {
    "acos",       "asin",
    "atan",       "cos",
    "cosh",       "exp",
    "abs",        "ln",
    "lg",         "sin",
    "sinh",       "radical",
    "tan",        "tanh",
    "step",       "sgn",       /* Numele functiilor matematice disponibile */
    "^",                       /* "Numele" ridicarii la putere - pentru afisarea erorii */
    ""                         /* Terminatorul tabelului */
};

/* Functiile corespunzatoare numelor definite */
double (*YYP_f[])(double x) = {
    acos,          asin,
    atan,          cos,
    cosh,          exp,
    fabs,          log,
    log10,         sin,
    sinh,          sqrt,
    tan,           tanh,
    YYP_step,      YYP_sgn
};

char loperatori[] = {
    ATRIB,      PLUS,       MINUS,      INMUL,
    IMPAR,      PUTERE,     PARDES,     PARINC,  /* Operatorii recunoscuti */
    '\0'                                         /* Terminatorul tabelului */
};

/* Codurile erorilor recunoscute */
enum {
    NOERR,        /* Nici o eroare */
    ERROPTOR,     /* Lipseste operatorul */
    ERROPAND,     /* Lipseste operandul */
    ERRPARINC,    /* Lipseste paranteza inchisa */
    ERRIDNEC,     /* Identificator necunoscut */
    ERRFNEC,      /* Functie necunoscuta */
    ERRNUPAR,     /* Lipseste parametrul functiei */
    ERRPARAM,     /* Parametrul functiei este incorect */
    ERRSEPINSTR,  /* Lipseste separatorul de instructiuni */
    ERRNRVAR,     /* Prea multe variabile */
    ERRNUNEG,     /* Nu este admisa o expresie negativa */
    ERRSINT       /* Eroare de sintaxa */
};

/* Mesajele de eroare corespunzatoare codurilor de eroare definite */
char *YYP_errmsg[] = {
    "lipseste operatorul",
    "lipseste operandul",
    "lipseste paranteza inchisa",
    "identificator necunoscut",
    "functie necunoscuta",
    "lipseste parametrul functiei",
    "parametrul functiei este incorect",
    "lipseste separatorul de instructiuni",
    "prea multe variabile",
    "nu este admisa o valoare negativa",
    "eroare de sintaxa"
};

#define T_ATRIB 257
#define T_OPTOR1 258
#define T_OPTOR2 259
#define T_OPTOR3 260
#define UNAR 261
#define T_NUMREAL 262
#define T_IDENTIF 263
#define T_SPATIU 264
#define T_SEPINSTR 265
#define T_PARDES 266
#define T_PARINC 267
#define T_HELP 268
#define T_TERMIN 0
#define yyclearin yychar = -1
#define yyerrok yyerrflag = 0
extern int yychar;
extern short yyerrflag;
#ifndef YYMAXDEPTH
#define YYMAXDEPTH 150
#endif
YYSTYPE yylval, yyval;
#define YYERRCODE 256

#line 387 "\LMT\YACC\FUNC.Y"

/*************************************************
 *                                               *
 *             Sectiunea de programe             *
 *                                               *
 *************************************************/


/* Prototipuri de functii */
#include <conio.h>
#include <ctype.h>
#include <float.h>
#include <process.h>
#include <setjmp.h>
#include <signal.h>
#include <stdio.h>
#include <string.h>

int yyparse(void);

/* Constante */
char ERRPROMPT[]  = "\n Eroare:";
char MERRPROMPT[] = "\n Eroare";
char ERRMARK[]    = "";

/* Variabile globale */
char *YYP_expr;               /* Expresia de analizat */


int yylex(void)
/* Analizorul lexical */
/* Intoarce: */
/* * T_SPATIU    - pentru ' ', '\t' si YYP_vedesp == TRUE */
/* * T_NUMREAL   - pentru un intreg si seteaza yylval.d = (double)numar */
/* * T_IDENTIF   - pentru un identificator si seteaza yylval.s = identificator */
/* * T_OPTOR1    - pentru PLUS, MINUS si yylval.c = PLUS, MINUS */
/* * T_OPTOR2    - pentru INMUL, IMPAR si yylval.c = INMUL, IMPAR */
/* * T_OPTOR3    - pentru PUTERE si yylval.c = PUTERE */
/* * T_ATRIB     - pentru ATRIB */
/* * T_PARDES    - pentru PARDES */
/* * T_PARINC    - pentru PARINC */
/* * T_SEPINSTR  - pentru SEPINSTR */
/* * T_HELP      - pentru HELP */
/* * T_TERMIN    - pentru terminatorul de expresie */
/* * caracter    - pentru orice alt caracter */
{
#define CARURM       (YYP_expr[YYP_index++])
#define UNGET        (YYP_index--)
#define SPATIU(c)    (c == ' ' || c == '\t')
#define CIFRA(c)     (isdigit(c))
#define LITERA(c)    (isalpha(c))
#define PARANTEZA(c) (c == PARDES || c == PARINC)
#define OPERATOR(c)  (c == PLUS   || c == MINUS  || \
                      c == INMUL  || c == IMPAR  || \
                      c == PUTERE || c == ATRIB)
#define IDENTIF(c)   (CIFRA(c) || LITERA(c))
#define EXPONENT(c)  (tolower(c) == REALEXP)
#define SEPZECIM(c)  (c == REALSEPZ)
#define SEMN(c)      (c == PLUS || c == MINUS)

    int c;                      /* Caracterul curent */

    c = CARURM;                 /* Citeste urmatorul caracter */
    if (SPATIU(c)) {            /* c este spatiu */
        do
            c = CARURM;
        while (SPATIU(c));      /* Sare spatiile */
        if (YYP_vedesp) {       /* Este sensibil la spatii */
            SPINSENSIB;
            UNGET;              /* La inceputul cuvintului */
            YYP_errindex = YYP_index;
            return(T_SPATIU);
        }
    }
    /* c nu este spatiu */
    SPINSENSIB;
    YYP_errindex = YYP_index;

    if (CIFRA(c) || c == '.') {      /* Cifra - posibil numar */
#define MAX 30   /* Numarul de semne la care se trunchiaza un numar */
        char buf[MAX+1], *pb;        /* Buffer si poz. curenta in buffer */
        int  z = FALSE,  e = FALSE;  /* Nu a intilnit nici punct, nici exponent */
        int  s = FALSE;              /* Nu permite citirea semnului */

        for (pb = buf; pb-buf < MAX; pb++, c = CARURM) {
            *pb = c;
            if (CIFRA(c)) {          /* Cifra */
                s = FALSE;           /* Dupa o cifra nu mai admite semn */
                continue;
            }
            if (SEMN(c))             /* Semn */
                if (s) continue;     /* Semnul exponentului */
            if (SEPZECIM(c)) {       /* Separatorul zecimal */
                if (z) break;        /* Punctul zecimal a mai fost o data */
                z = TRUE;            /* A citit punctul zecimal */
                continue;
            }
            if (EXPONENT(c)) {       /* Exponentul */
                if (e) break;        /* Exponentul a mai fost o data */
                s = e = TRUE;        /* Va citi exponentul, admite semnul */
                continue;
            }
            break;                   /* Alt caracter, sfirsitul numarului */
        }
        *pb = '\0';                  /* Sfirsitul numarului */
        UNGET;
        yylval.d = atof(buf);        /* Valoarea numarului */
        return(T_NUMREAL);
#undef MAX
    }

    if (LITERA(c)) {    /* Litera - inceput de identificator */
        int i = 0;      /* Index */

        do {
            yylval.s[i++] = tolower(c);        /* Conversie la litera mica */
            c = CARURM;
        } while (IDENTIF(c) && i <= MAXNUME);  /* Litera sau cifra, trunchiere la MAXNUME */
        yylval.s[i] = '\0';                    /* Sfirsitul numelui */
        UNGET;
        return(T_IDENTIF);
    }

    if (OPERATOR(c)) {
        yylval.c = c;
        switch (c) {
            case PUTERE: return(T_OPTOR3);
            case INMUL :
            case IMPAR : return(T_OPTOR2);
            case PLUS  :
            case MINUS : return(T_OPTOR1);
            case ATRIB : return(T_ATRIB);
        }
    }

    if (c == PARDES)                /* Paranteza deschisa */
        return(T_PARDES);

    if (c == PARINC)                /* Paranteza inchisa */
        return(T_PARINC);

    if (c == SEPINSTR)              /* Separator de instructiuni */
        return(T_SEPINSTR);

    if (c == HELP)                  /* Help */
        return(T_HELP);

    if (c == '\0' || c == '\n') {   /* Terminator expresie */
        UNGET;
        return(T_TERMIN);
    }

    yylval.c = c;
    return(c);                      /* Altceva - intoarce ca atare */

#undef CARURM
#undef SPATIU(c)
#undef CIFRA(c)
#undef LITERA(c)
#undef PARANTEZA(c)
#undef OPERATOR(c)
#undef IDENTIF(c)
#undef EXPONENT(c)
#undef SEPZECIM(c)
#undef SEMN(c)
}

int YYP_memor(char *s, double x)
/* Memoreaza variabila cu numele s si valoarea x */
/* Intoarce TRUE daca memorarea a fost posibila, altfel FALSE */
{
    int i;                             /* Index in tabelul variabilelor */

    for (i = 0; i < MAXVAR; i++) {     /* Parcurge tabelul */
        if (*YYP_var[i].s == '\0') {   /* Sfirsitul listei */
            strcpy(YYP_var[i].s, s);   /* Memoreaza denumirea variabilei */
            if (i < MAXVAR)
                *YYP_var[i+1].s = '\0';/* Noul sfirsit al listei */
        }
        else
            if (!COMPARA(s, YYP_var[i].s)) /* Nu a gasit simbolul */
                continue;
        YYP_var[i].x = x;              /* Memoreaza valoarea variabilei */
        return(TRUE);                  /* Memorarea a fost posibila */
    }
    return(FALSE);                     /* Depasirea tabelei */
}

int YYP_rememor(char *s, double *x)
/* Rememoreaza variabila cu numele s in valoarea x */
/* Intoarce TRUE daca variabila a fost definita, altfel FALSE */
{
    int i;                             /* Index in tabelul variabilelor */

    for (i = 0; i < MAXVAR; i++) {     /* Parcurge tabelul variabilelor */
        if (*YYP_var[i].s == '\0')     /* Sfirsitul tabelului */
            return(FALSE);
        if (!COMPARA(s, YYP_var[i].s)) /* Nu e variabila cautata */
            continue;
        *x = YYP_var[i].x;             /* A gasit varabila cautata */
        return(TRUE);
    }
    return(FALSE);
}

int YYP_veriffunc(char *s)
/* Verifica daca functia cu numele s e definita */
/* Intoarce indexul din tabelul de definitie daca e definita, altfel FALSE */
{
    /* Parcurge tabelul numelor functiilor definite */
    for (YYP_findex = 0; *numefunc[YYP_findex] != '\0'; YYP_findex++)
        if (COMPARA(s, numefunc[YYP_findex]))   /* A gasit numele functiei */
            return(YYP_findex+1);               /* Functie identificata */
    return(FALSE);                              /* Functie neidentificata */
}

double YYP_step(double x)
/*                            / 0 pentru  x  < 0;
Calculeaza functia step(x) = <
                              \ 1 pentru  x >= 0.
*/
{
    return( (x < 0) ? 0. : 1. );
}

double YYP_sgn(double x)
/*                            / -1 pentru  x < 0;
                             |
Calculeaza functia sgn(x) = <   0 pentru  x = 0;
                             |
                              \ +1 pentru  x > 0.
*/
{
    if (x < 0.)      return(-1.);
    else if (x > 0.) return(1.);
    else             return(0.);
}

void YYP_markerr(void)
/* Afiseaza expresia si marcherul de eroare pe pozitia erorii */
{
    char eformat[] = "%nnns\n%s\n";     /* Formatul pt. pozitionarea marcherului de eroare */

    /* Initializarea formatului pentru afisarea marcherului de eroare */
    sprintf(eformat, "%%%ds\n%%s\n", YYP_errindex);
    printf(eformat, ERRMARK, YYP_expr);     /* Afisarea marcherului si expresiei */
}

void yyerror(char *s)
/* Afiseaza mesajul de eroare cu codul YYP_errcod */
{
    s = YYP_errmsg[YYP_errcod-1];
    printf("%s %s", ERRPROMPT, s);      /* Afisarea mesajului */
    if (*YYP_errstr != '\0')
       printf(" \"%s\"", YYP_errstr);
    printf(".\n");
    YYP_markerr();
    YYP_existaerr = TRUE;               /* A fost detectata o eroare */
}

int matherr(struct exception *a)
/* Intercepteaza eroarea produsa la apelul unei functii */
/* matematice. Afiseaza un mesaj de eroare din care reiese */
/* numele functiei matematice, tipul erorii si argumentele */
/* cu care a fost apelata. */
/* Seteaza YYP_existamaterr la o valoare nenula. */
{
    char *es;            /* Tipul erorii */

    switch (a->type) {
        case DOMAIN   : es = "domeniul incorect";                  break;
        case SING     : es = "singularitate a argumentului";       break;
        case OVERFLOW : es = "depasire superioara";                break;
        case UNDERFLOW: es = "depasire inferioara";                break;
        case TLOSS    : es = "pierderea totala a semnificatiei";   break;
        case PLOSS    : es = "pierderea partiala a semnificatiei"; break;
    }
    printf("%s la apelul functiei \"%s\": %s.\n", MERRPROMPT, numefunc[YYP_findex], es);
    printf("Argument de apel: %.3lG\n", a->arg1);
    YYP_markerr();
    YYP_existamaterr = TRUE; /* A fost detectata o eroare matematica */
    return(1);               /* Inhiba alte afisari */
}

void YYP_matoperr(int tip, int cod)
/* Intercepteaza eroarea produsa in timpul unui calcul */
/* matematic. Afiseaza un mesaj de eroare din care reiese */
/* codul erorii. */
/* Seteaza YYP_existamaterr la o valoare nenula. */
{
    if (cod == 140) return;
    /* Afiseaza mesajul de eroare */
    printf("%s la o operatie aritmetica. Tip: %d, cod: %d\n", MERRPROMPT, tip, cod);
    YYP_existamaterr = TRUE;         /* A fost detectata o eroare matematica */
    signal(SIGFPE, YYP_matoperr);
}

void YYP_help(void)
/* Afiseaza operatiile si functiile disponibile */
{
    int  i, j;                      /* Contoare */
    char hfformat[] = "%nns";       /* Format pentru siruri */
    char hoformat[] = "%nnc";       /* Format pentru caractere */

    sprintf(hfformat, "%%%ds", NCOLMON / NCOLHELP);
    sprintf(hoformat, "%%%dc", NCOLMON / NCOLHELP);  /* Init. formatelor */

    printf("\nOperatiile aritmetice in ordinea crescatoare a prioritatii:\n\n");
    for (i = 0; ; i += NCOLHELP) {
        for (j = 0; j < NCOLHELP; j++) {             /* Afiseaza pe o linie */
            if (loperatori[i+j] == '\0') break;      /* Sfirsitul listei */
            printf(hoformat, loperatori[i+j]);       /* Afiseaza operatiile */
        }
        printf("\n");                                /* Trece la linia urmatoare */
        if (loperatori[i+j] == '\0') break;          /* Sfirsitul listei */
    }

    printf("\nFunctiile disponibile:\n\n");
    for (i = 0; ; i += NCOLHELP) {
        for (j = 0; j < NCOLHELP; j++) {             /* Afiseaza pe o linie */
            if (*numefunc[i+j] == '\0') break;       /* Sfirsitul listei */
            if (isalpha(*numefunc[i+j]))
                printf(hfformat, numefunc[i+j]);     /* Afiseaza functiile */
        }
        printf("\n");                                /* Trece la linia urmatoare */
        if (*numefunc[i+j] == '\0') break;           /* Sfirsitul listei */
    }

    printf("\nSeparatorul de expresii: %c\n", SEPINSTR);
}

void yaccinit(void)
/* Face initializarea variabilelor folosite in parser. */
{
    YYP_index = 0;             /* Pozitioneaza citirea inainte de inceputul expresiei */
    SETERR(ERRSINT);           /* Initializarea tipului de eroare */
    *YYP_var[0].s = '\0';      /* Initializarea tabelei de simboluri */
    YYP_evaluat   = 1;         /* Expresia nu e evaluata */
    YYP_existaerr = FALSE;     /* Nu a aparut nici o eroare */
    YYP_existamaterr = FALSE;  /* Nu a aparut nici o eroare matematica */
    YYP_errstr = "";           /* Numele care a produs eroarea */
}

double parser(char *s, int *coderr, int verifsint)
/*
 - Analizeaza si evalueaza expresia matematica continuta in s.
 - coderr va contine o valoare nula daca expresia este fara erori
   si o valoare nenula in caz contrar.
 - Daca verifsint contine o valoare nenula se face numai o verificare
   sintactica a expresiei (fara evaluare), altfel se face si evaluarea.
 - Functia intoarce rezultatul evaluarii expresiei (daca verifsint = 0).
*/
{
    signal(SIGFPE, YYP_matoperr); /* Init. tratarea erorilor fct. aritmetice */
    yaccinit();                   /* Init. var. */
    YYP_versint = verifsint;      /* Evaluare sau numai verificare sintactica */
    YYP_expr = s;                 /* Expresia de analizat */
    yyparse();                    /* Analizor si calculator */
    *coderr = YYP_evaluat;        /* Evaluare posibila sau nu */
    raise(SIGFPE);
    return(YYP_rezultat);         /* Rezultatul evaluarii */
}
short yyexca[] ={
-1, 1,
	0, -1,
	-2, 0,
	};
#define YYNPROD 30
#define YYLAST 270
short yyact[]={

  10,  41,   9,  40,  14,  18,   5,  13,  19,  10,
   8,   9,   3,   4,  25,   5,  13,  33,  23,   8,
  22,  16,  17,  18,  17,  18,  24,  21,  29,  28,
  27,  39,  20,  30,  31,  32,  26,   1,   6,  11,
  34,  35,  36,  37,  38,   2,   7,  12,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
   0,   0,   0,   0,   0,   0,   0,   0,   0,  15 };
short yypact[]={

-256,-1000,   4,-1000,-237,-1000,-249,-1000,-1000,-1000,
-1000,-246,-247,-1000,-1000,  14,-1000,-1000,-1000,-1000,
-247,-247,-1000,-1000,-237,-1000,-247,-247,-247,-247,
-247,-237,-1000,-264,-237,-235,-255,-1000,-237,-266,
-1000,-1000 };
short yypgo[]={

   0,  47,  13,  46,  45,  39,  38,  37,  36,  33,
  32,  31,  30,  29,  28,  27,  17 };
short yyr1[]={

   0,   7,   7,   7,   4,   8,   4,   2,   2,   2,
   9,   2,  10,  11,   2,  12,   2,  13,   2,  14,
   2,  15,   2,   2,  16,   3,   1,   5,   6,   6 };
short yyr2[]={

   0,   2,   3,   1,   1,   0,   4,   1,   1,   1,
   0,   4,   0,   0,   5,   0,   4,   0,   4,   0,
   4,   0,   3,   1,   0,   4,   2,   1,   1,   2 };
short yychk[]={

-1000,  -7,  -4, 268,  -2, 262,  -6,  -3, 266, 258,
 256,  -5,  -1, 263,   0, 265, 258, 259, 260, 257,
 -10, -15, 266, 264,  -2,   0,  -8, -12, -13, -14,
  -9,  -2,  -2, -16,  -2,  -2,  -2,  -2,  -2, -11,
 267, 267 };
short yydef[]={

   0,  -2,   0,   3,   4,   7,   8,   9,  12,  21,
  23,  28,   0,  27,   1,   5,  15,  17,  19,  10,
   0,   0,  26,  29,  24,   2,   0,   0,   0,   0,
   0,  13,  22,   0,   6,  16,  18,  20,  11,   0,
  25,  14 };
#define YYFLAG	 -1000
#define YYERROR  goto yyerrlab
#define YYACCEPT return(0)
#define YYABORT  return(1)

/* parser for yacc output  */

#ifndef __YYDEBUGGER
int yydebug = 0;          /* 1 for debugging */
#else
int yydebug = 1;          /* 0 fara debuger */
#endif
YYSTYPE yyv[YYMAXDEPTH];  /* where the values are stored */
int yychar = -1;          /* current input token number */
int yynerrs = 0;          /* number of errors */
short yyerrflag = 0;      /* error recovery flag */

int yyparse(void)
{
 short yys[YYMAXDEPTH];
 short yyj, yym;
 register YYSTYPE *yypvt;
 register short yystate, *yyps, yyn;
 register YYSTYPE *yypv;
 register short *yyxi;

 yystate = 0;
 yychar = -1;
 yynerrs = 0;
 yyerrflag = 0;
 yyps= &yys[-1];
 yypv= &yyv[-1];

 yystack:    /* put a state and value onto the stack */

 if( yydebug  ) printf( "state %d, char 0%o\n", yystate, yychar );
 if( ++yyps> &yys[YYMAXDEPTH] ){
    yyerror( "yacc stack overflow" );
    return(1);
 }
 *yyps = yystate;
 ++yypv;
 *yypv = yyval;
yynewstate:
 yyn = yypact[yystate];
 if( yyn<= YYFLAG )
   goto yydefault; /* simple state */
 if( yychar<0 )
   if( (yychar=yylex())<0 ) yychar=0;
 if( (yyn += yychar)<0 || yyn >= YYLAST )
   goto yydefault;

 if( yychk[ yyn=yyact[ yyn ] ] == yychar ){ /* valid shift */
   yychar = -1;
   yyval = yylval;
   yystate = yyn;
   if( yyerrflag > 0 )
     --yyerrflag;
   goto yystack;
 }

yydefault:
        /* default state action */

  if( (yyn=yydef[yystate]) == -2 ) {
    if( yychar<0 )
      if( (yychar=yylex())<0 ) yychar = 0;
  /* look through exception table */

  for( yyxi=yyexca;
       (*yyxi!= (-1)) || (yyxi[1]!=yystate) ;
       yyxi += 2 ) ; /* VOID */

  while( *(yyxi+=2) >= 0 ){
    if( *yyxi == yychar )
      break;
  }
  if( (yyn = yyxi[1]) < 0 )
    return(0);   /* accept */
  }
  if( yyn == 0 ){ /* error */
    /* error ... attempt to resume parsing */

    switch( yyerrflag ){
      case 0:   /* brand new error */
	yyerror( "syntax error" );
      yyerrlab:
	++yynerrs;
      case 1:
      case 2: /* incompletely recovered error ... try again */
	yyerrflag = 3;
      /* find a state where "error" is a legal shift action */
	while ( yyps >= yys ) {
	   yyn = yypact[*yyps] + YYERRCODE;
	   if( yyn>= 0 && yyn < YYLAST && yychk[yyact[yyn]] == YYERRCODE ){
	     yystate = yyact[yyn];  /* simulate a shift of "error" */
	     goto yystack;
	   }
	   yyn = yypact[*yyps];
	   /* the current yyps has no shift onn "error", pop stack */
	   if( yydebug )
	     printf( "error recovery pops state %d, uncovers %d\n",
		     *yyps, yyps[-1] );
	   --yyps;
	   --yypv;
	}

	/* there is no state on the stack with an error shift ... abort */
      yyabort:
	return(1);

      case 3:  /* no shift yet; clobber input char */
	if( yydebug )
	   printf( "error recovery discards char %d\n", yychar );
	if( yychar == 0 )
	  goto yyabort; /* don't discard EOF, quit */
	yychar = -1;
	goto yynewstate;   /* try again in the same state */
    }
  }
  /* reduction by production yyn */
  if( yydebug )
    printf("reduce %d\n",yyn);
  yyps -= yyr2[yyn];
  yypvt = yypv;
  yypv -= yyr2[yyn];
  yyval = yypv[1];
  yym=yyn;
  /* consult goto table to find next state */
  yyn = yyr1[yyn];
  yyj = yypgo[yyn] + *yyps + 1;
  if( yyj>=YYLAST || yychk[ yystate = yyact[yyj] ] != -yyn )
    yystate = yyact[yypgo[yyn]];
  switch(yym){
    
case 1:
#line 196 "\LMT\YACC\FUNC.Y"
{
                if (YYP_versint)        /* Numai o verificare sintactica */
                    YYP_rezultat = 0.0;
                else                    /* Evaluare si verificare sintactica */
                    YYP_rezultat = yypvt[-1].d;
                if (!YYP_existaerr && !YYP_existamaterr)
                    YYP_evaluat = 0;
            } break;
case 2:
#line 204 "\LMT\YACC\FUNC.Y"
{
                if (YYP_versint)        /* Numai o verificare sintactica */
                    YYP_rezultat = 0.0;
                else                    /* Evaluare si verificare sintactica */
                    YYP_rezultat = yypvt[-2].d;
                if (!YYP_existaerr && !YYP_existamaterr)
                    YYP_evaluat = 0;
            } break;
case 3:
#line 213 "\LMT\YACC\FUNC.Y"
{
                YYP_help();
            } break;
case 4:
#line 219 "\LMT\YACC\FUNC.Y"
{
                SETERR(ERROPAND);
            } break;
case 5:
#line 223 "\LMT\YACC\FUNC.Y"
{
                SETERR(ERROPAND);
            } break;
case 6:
#line 226 "\LMT\YACC\FUNC.Y"
{
                if (!YYP_versint)
                    yyval.d = yypvt[-0].d;
            } break;
case 7:
#line 234 "\LMT\YACC\FUNC.Y"
{
                SETERR(ERROPTOR);
            } break;
case 8:
#line 238 "\LMT\YACC\FUNC.Y"
{
                if (!YYP_rememor(yypvt[-0].s, &yyval.d)) {
                    SETERR(ERRIDNEC);    /* Identificator necunoscut */
                    YYP_errstr = yypvt[-0].s;
                    yyerror(yypvt[-0].s);
                    YYERROR;
                }
            } break;
case 10:
#line 249 "\LMT\YACC\FUNC.Y"
{
                SETERR(ERROPAND);
            } break;
case 11:
#line 252 "\LMT\YACC\FUNC.Y"
{
                if (!YYP_memor(yypvt[-3].s, yypvt[-0].d)) {
                    SETERR(ERRNRVAR);
                    yyerror("");
                    YYERROR;
                }
                else
                    if (!YYP_versint)
                        yyval.d = yypvt[-0].d;
            } break;
case 12:
#line 265 "\LMT\YACC\FUNC.Y"
{
                SETERR(ERROPAND);
            } break;
case 13:
#line 268 "\LMT\YACC\FUNC.Y"
{
                SETERR(ERRPARINC);
            } break;
case 14:
#line 271 "\LMT\YACC\FUNC.Y"
{
                SETERR(ERROPTOR);
                if (!YYP_versint)
                    yyval.d = yypvt[-2].d;
            } break;
case 15:
#line 277 "\LMT\YACC\FUNC.Y"
{
                SETERR(ERROPAND);
            } break;
case 16:
#line 280 "\LMT\YACC\FUNC.Y"
{
                SETERR(ERROPTOR);
                if (!YYP_versint)
                    switch (yypvt[-2].c) {
                        case '+':
                            yyval.d = yypvt[-3].d + yypvt[-0].d;
                            break;
                        case '-':
                            yyval.d = yypvt[-3].d - yypvt[-0].d;
                            break;
                    }
                if (YYP_existamaterr) YYERROR;
            } break;
case 17:
#line 294 "\LMT\YACC\FUNC.Y"
{
                SETERR(ERROPAND);
            } break;
case 18:
#line 297 "\LMT\YACC\FUNC.Y"
{
                SETERR(ERROPTOR);
                if (!YYP_versint)
                    switch (yypvt[-2].c) {
                        case '*':
                            yyval.d = yypvt[-3].d * yypvt[-0].d;
                            break;
                        case '/':
                            yyval.d = yypvt[-3].d / yypvt[-0].d;
                            break;
                    }
                if (YYP_existamaterr) YYERROR;
            } break;
case 19:
#line 311 "\LMT\YACC\FUNC.Y"
{
                SETERR(ERROPAND);
            } break;
case 20:
#line 314 "\LMT\YACC\FUNC.Y"
{
                SETERR(ERROPTOR);
                if (!YYP_versint)
                    switch (yypvt[-2].c) {
                        case '^':
                            YYP_veriffunc("^"); /* Seteaza YYP_findex pt. eroare */
                            yyval.d = pow(yypvt[-3].d, yypvt[-0].d);   /* Calcul pr.-zis */
                            break;
                    }
                if (YYP_existamaterr) YYERROR;
            } break;
case 21:
#line 326 "\LMT\YACC\FUNC.Y"
{
                SETERR(ERROPAND);
                SPSENSIB; /* Nu permite spatiul intre operator si expresie */
            } break;
case 22:
#line 330 "\LMT\YACC\FUNC.Y"
{
                SETERR(ERROPTOR);
                if (!YYP_versint)
                    switch (yypvt[-2].c) {
                        case '+':
                            yyval.d = yypvt[-0].d;
                            break;
                        case '-':
                            yyval.d = -yypvt[-0].d;
                            break;
                    }
            } break;
case 23:
#line 342 "\LMT\YACC\FUNC.Y"
{
                if (!YYP_versint)
                    yyval.d = 0.0;
            } break;
case 24:
#line 350 "\LMT\YACC\FUNC.Y"
{
                SETERR(ERRPARINC);
            } break;
case 25:
#line 353 "\LMT\YACC\FUNC.Y"
{
                SETERR(ERROPTOR);            /* Asteapta un operator */
                if (!YYP_versint)
                    yyval.d = YYP_f[yypvt[-3].i-1](yypvt[-2].d);    /* Calculeaza functia */
                if (YYP_existamaterr) YYERROR;
            } break;
case 26:
#line 363 "\LMT\YACC\FUNC.Y"
{         /* Nu permite spatii intre id. si '(' */
                SETERR(ERRPARAM);      /* Parametru incorect */
                if ((yyval.i = YYP_veriffunc(yypvt[-1].s)) == 0) { /* Intoarce 0 daca functia */
                    SETERR(ERRFNEC);             /* nu exista, altfel intoarce */
                    YYP_errstr = yypvt[-1].s;             /* numarul functiei */
                    yyerror(yypvt[-1].s);
                    YYERROR;
                }
            } break;
case 27:
#line 375 "\LMT\YACC\FUNC.Y"
{
                SETERR(ERROPTOR); /* Identificator necunoscut */
                SPSENSIB;         /* Analizorul lexical devine sensibil la spatii */
           } break;
  }
  goto yystack;  /* stack new state and value */

}
