
// DEMOREZ - program pentru demonstratia convergentei
//     iteratiilor la rezolvarea urmatoarelor ecuatii
//     algebrice neliniare:
//   
//     e^x = a0
//     x + a0.lnx = a1
//     a0.sinx + a1.cosx = a2
//     a0 + a1.x + a2.x^2 + a3.x^3 = 0

//     prin metodele:
//        - bisectiei;
//        - iteratiei simple;
//        - tangentelor paralele (Newton-Kantorovici);
//        - tangentelor (Newton);
//        - secantelor.

//Intrare: coeficientii a0,a1,..
//       intervalul xmin,xmax
//Iesire : graficul functiei si punctele corespunzatoare iteratiilor

function [val] = f(neq, x, a)
//Evalueaza functia f(x) asociata ecuatiei f(x) =0
if (neq < 1 | neq > 4) then
    return 0;
end;
select neq
case 1 then
    val = (exp(x) - a(1));
case 2 then
    val = (x + a(1) * log(abs(x)) - a(2));
case 3 then
    val = a(1)*sin(x) + a(2)*cos(x) - a(3);
case 4 then
    val = (a(1) + a(2)*x + a(3)*x*x + a(4)*x*x*x);
end;
return(val);


function [val] = fd(neq, x, a)
//Evalueaza derivata functiei f(x) asociata ecuatiei f(x) =0
if (neq < 1 | neq > 4) then
    return 0;
end;
select neq
case 1 then
    val = (exp(x));
case 2 then
    val = (1 + a(1) * 1/(abs(x)));
case 3 then
    val = (a(1)*cos(x) - a(2)*sin(x));
case 4 then
    val = (a(2) + 2*a(3)*x + 3*a(4)*x*x);
end;
return val;


function [xnou, x1, x2] = fiter(xvechi, neq, nmet, x1, x2, alpha, a, xvv)
//Realizeaza o iteratie si intoarce noua valoare a solutiei numerice
select nmet
case 1 then //metoda bisectiei
    xnou = (x1 + x2)/2.;
    if (f(neq, xnou, a)*f(neq, x1, a)>0.)
        x1 = xnou;
    else
        x2 = xnou;
    end;
case 2 then //metoda iteratiei simple 
     xnou = xvechi + alfa * f(neq,xvechi, a);
case 3 then //metoda tangentelor parlele
     xnou = xvechi + alfa * f(neq,xvechi, a);
case 4 then //metoda tangentelor
     xnou = xvechi - f(neq, xvechi, a)/fd(neq, xvechi, a);
case 5 then //metoda secantelor
     xnou = xvechi - f(neq,xvechi, a)*(xvechi-xvv)/(f(neq,xvechi, a)-f(neq,xvv, a));
end;

return;


function [] = tri(xv, xn, neq, a)
//Repezinta grafic triunghiul xvechi, f(xvechi), xnou
y = f(neq, xv, a);
mat(1, 1) = xv;
mat(1, 2) = xv;
mat(1, 3) = xn;
mat(2, 1) = 0;
mat(2, 2) = y;
mat(2, 3) = 0;

plot2d([mat(1,:)], [mat(2, :)], [5], '100');
printf("valoare: %g ", xn);
//halt();
return;



function [] = iter(neq, nmet, xmin, xmax, a, nr_iter, err)
getf('../L01/alte_functii.sci');
xvechi = xmin;
xvv = xmax;
x1 = xmin;
x2 = xmax;
N = 100;
//err = 1e-3;
nrit = nr_iter;

//ceea ce apare pe ecran
printf("\n=======================================================\n");

if (neq == 1)
    printf("functia: e^x = %g \n", a(1));
end;
if (neq == 2)
    printf("functia: x + %g*ln x = %g \n", a(1), a(2));
end;
if (neq == 3)
    printf("functia: %g*sin(x) + %g*cos(x) = %g \n", a(1), a(2), a(3));
end;
if (neq == 4)
    printf("functia: %g*x^3 + %g*x^2 + %g*x + %g = 0 \n", a(1), a(2), a(3), a(4));
end;

if (nmet == 1)
    printf("metoda bisectie\n");
    printf("initializare a solutiei: %g\n", xmin);
end;
if (nmet == 2)
    printf("metoda iteratiei simple\n");
    printf("initializare a solutiei: %g\n", xmax);
end;
if (nmet == 3)
    printf("metoda tangentelor paralele\n");
    printf("initializare a solutiei: %g\n", xmax);
end;
if (nmet == 4)
    printf("metoda tangentelor (Newton)\n");
    printf("initializare a solutiei: %g\n", xmax);
end;
if (nmet == 5)
    printf("metoda secantelor\n");
    printf("initializare a solutiei: %g\n", xmin);
end;
printf("domeniul de cautare a solutiei: [xmin = %g, xmax = %g]\n", xmin, xmax);
printf("Numarul maxim de iteratii: %d \n", nr_iter);
printf("Eroarea acceptabila: %g \n", err);
//ceea ce apare pe ecran

for i=1:N
    mat(1, i) = x1 + (x2-x1)*i/(N-1);
    mat(2, i) = 0;
    mat(3, i) = f(neq, mat(1, i), a);
end;
mmmax = max(mat(3, :));
mmmin = min(mat(3, :));
xclear();
delmenu(0,'File');
delmenu(0,'3D Rot.');
actions = ['xdel(0);'];
addmenu(0,'Exit');
Exit_0 = [actions(1)];
set("figure_style","old");xname('Functia originala si iteratiile metodelor de rezolvare');
xtitle('Functia originala','x','F(x)');

//plot2d(mat(1, :), mat(3, :), [2], '151','functia originala', [x1-(sign(x1)/10), round(mmmin)+sign(mmmin), x2+(sign(x2)/10), round(mmmax)+sign(mmmax)]);
plot2d(mat(1, :), mat(3, :), [2], '161','functia originala');
//, [x1, round(mmmin)+sign(mmmin), x2, round(mmmax)+sign(mmmax)]);
vv1 = x1-1:x2+1;
vv2 = linspace(0, 0, length(vv1));
plot2d(vv1, vv2, [1], '100');
halt();
//xgrid();

hold = 1;
if (nmet == 1) then
    tri(xmin,xmin,neq, a);
    tri(xmax,xmax,neq, a);
end;

if (nmet == 3)
    alfa = -1/fd(neq, xvechi, a);
else
    alfa = -1;
end;

k = 1;
eps = 1;
while (eps > err & k < nrit)
    k = k + 1;
    [xnou, x1, x2] = fiter(xvechi,neq, nmet, x1, x2, alfa, a, xvv);
    printf("iter: %d ", k-1);
    if (nmet == 1)
        tri(xnou, xnou, neq, a);
    else
        tri(xvechi, xnou, neq, a);
    end;
    eps = abs(f(neq,xnou, a));
    printf("err = %g\n ", eps); 
    halt()
    xvv = xvechi;
    xvechi = xnou;
end;
printf("\n=======================================================\n");
if (eps < err)
    printf("\n S-a gasit solutia. Apasati Exit in fereastra grafica.\n");
else
    printf("\nNu s-a gasit solutia, si s-a efectuat nr maxim de iteratii.\n");
    printf("\nApasati Exit in fereastra grafica.\n");
end;
wait2close();
return;
