
//NELAN - program pentru analiza metodelor iterative
//de rezolvare a ecuatiilor algebrice neliniare.
//Rezolva ecuatia:

//a0 + a1.x + a2.x^2 = 0

//prin metodele:
//- bisectiei;
//- iteratiei simple;
//- tangentelor paralele (Newton-Kantorovici);
//- iteratiei simple;
//- tangentelor paralele (Newton-Kantorovici);
//- tangentelor (Newton);
//- secantelor.

//Intrare: coeficientii a0,a1,a2
//intervalul xmin,xmax
//Iesire : graficul erorii in functiei de ordinul iteratiei


// Evalueaza functia f(x) asociata ecuatiei f(x) = 0
function [val] = f(x, a)
    val = a(1) + a(2)*x + a(3)*x*x;
return;

// Evalueaza derivata functiei f(x) asociata ecuatiei f(x) =0
function [val] = fd(x, a)
    val = a(2) + 2 * a(3)*x;
return;


// Realizeaza iteratii si afiseaza rezultatul
function [] = iter(nmet, xini, xminim, xmaxim, a)
    getf('../L01/alte_functii.sci');
    N = 100;
    k = 0;              
    nrit = N-2;          
    err = 1e-20;  
    eps0 = 1e-20;

    xvechi = xini;
    xvv = xmaxim;
    x1 = xminim;
    x2 = xmaxim;

    printf("\n=====================================================================\n");
    if (nmet == 1)
        printf("metoda bisectie\n");
    end;
    if (nmet == 2)
        printf("metoda iteratiei simple\n");
    end;
    if (nmet == 3)
        printf("metoda tangentelor paralele\n");
    end;
    if (nmet == 4)
        printf("metoda tangentelor (Newton)\n");
    end;
    if (nmet == 5)
        printf("metoda secantelor\n");
    end;
	
    if (nmet == 3)
        alfa = -1/(fd(xvechi, a) + eps0);
    else
        alfa = -1;
    end;

    for k = 1: N
        mat(1, k) = k;
    end;
    
    printf("\nk  Solutia exacta  Solutia num.  Eroarea    Corectia    Reziduu ");
    timer();

    k = 1;
    eps = 1;
    while (eps > err & k < nrit)
        [xnou, x1, x2] = fiter(xvechi, nmet, x1, x2, alfa, a, xvv);
	y = f(xnou, a);
	eps = abs(xnou - xvechi);
	xvv = xvechi;
	xvechi = xnou;
	mat(2, k) = abs(xa - xnou);
	mat(3, k) = eps;
	mat(4, k) = y;
	if ( (0<k & k<=10) | (modulo(k, ((nrit+2)/10))) == 0 )
	    printf("\n%d  %10g  %10g    %10g  %10g   %10g",k,xa,xnou,mat(2, k),eps,y);
	end;
	k = k + 1;
    end;
    second = timer();
    t = (second)/k;
    printf("\n\n  Timpul de calcul: %g sec/iter.\n",t);
    n = k; // numar efectiv de iteratii 
    for k = 1: n
        if (mat(1, k) < eps0) 
	      mat(1, k) = eps0; 
	end;
	if (mat(2, k) < eps0) 
	      mat(2, k) = eps0; 
	end;
	if (mat(3, k) < eps0) 
	      mat(3, k) = eps0; 
	end;
	if (mat(4, k) < eps0) 
	      mat(4, k) = eps0; 
	end;
    end;
   
    //label_x("Numar de iteratii");
    //title("Eroarea, corectia si reziduul");
    xclear();
    delmenu(0,'File');
    delmenu(0,'3D Rot.');
    actions = ['xdel(0);'];
    addmenu(0,'Exit');
    Exit_0 = [actions(1)];
    
    set("figure_style","old");xname('Eroarea corectia si reziduul');
    xtitle('Erorile, corectia si reziduul','Nr. iteratii','Erori');
    xgrid();
    plot2d("ll",[mat(1,1:n)'], [mat(2:4, 1:n)'], [5 2 3], '121', 'Eroarea@Corectia@Reziddul');  
    printf("\n=====================================================================\n");

    wait2close();
return;


//Realizeaza o iteratie si intoarce noua valoare a solutiei numerice

function [xnou, x1, x2] = fiter(xvechi, nmet, x1, x2, alfa, a, xvv)
    xnou = 0;
    select nmet
    case 1 then //metoda bisectiei
        xnou = (x1 + x2)/2.;
	if (f(xnou, a)*f(x1, a)>0.)
	    x1 = xnou;
	else
	    x2 = xnou;
	end;
    case 2 then //metoda iteratiei simple
        xnou = xvechi + alfa * f(xvechi, a);
    case 3 then //metoda tangentelor parlele 
        xnou = xvechi + alfa * f(xvechi, a);
    case 4 then //metoda tangentelor
        xnou = xvechi - f(xvechi, a)/(fd(xvechi, a)+1e-20);
    case 5 then //metoda secantelor
        xnou = xvechi - f(xvechi, a)*(xvechi-xvv)/(f(xvechi, a)-f(xvv, a)+1e-20);
    end; 
return;


